/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.lang.reflect.Field;
import java.net.Socket;
import java.net.URLClassLoader;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Performs basic check of the environment to check if RASP will run at all.
 */
public class RaspEnvironmentCheck {

    @SuppressWarnings("unused")
    private static final String SERIALNUMBER_CVR_RID = "SERIALNUMBER=CVR:25767535-RID:1118061020232";

    private static final String OKMSG = "Check OK...";

    public static void main(String[] args) {
        RaspEnvironmentCheck raspEnv = new RaspEnvironmentCheck();

        System.out.print("Checking OS ");
        System.out.println(" - " + raspEnv.checkOS());
        System.out.println();

        System.out.print("Checking Axis2");
        System.out.println(" - " + raspEnv.checkForAxis());
        System.out.print("Checking Mercury");
        System.out.println(" - " + raspEnv.checkForMercury());
        System.out.print("Checking Rampart");
        System.out.println(" - " + raspEnv.checkForRampart());
        System.out.println();

        System.out.print("Checking encryption ");
        System.out.println(" - " + raspEnv.checkEncryption());

        System.out.print("Checking for Ant ");
        System.out.println(" - " + raspEnv.checkForAnt());
        System.out.println();

        System.out.print("Checking for Bouncy-castle ");
        System.out.println(" - " + raspEnv.checkForBouncyCastle());
        System.out.println();

        System.out.print("Connecting to server (Tomcat or similar) ");
        System.out.println(" - " + raspEnv.checkForServer());
        System.out.println();

    }

    /**
     * Checks for proper upgrade from 1.2.3 to 1.3.0.
     * <p/>
     * In addition to dk.gov.oiosi.library.XXX.jar 2 new jars should replace one old:
     * <p/>
     * bcprov-jdk16-141.jar should be replaced with bcpkix-jdk15on-147.jar and bcprov-jdk15on-147.jar
     *
     * @since OIORASP 1.3.0
     */
    /*
     * TODO DLK: Why do we need this new JAR bcpkix-jdk15on-147.jar?
     */
    public String checkForBouncyCastle() {
        String check = OKMSG;
        String checkClassName = "org.bouncycastle.asn1.x509.X509Name";
        String checkFieldName = "SERIALNUMBER";
        String errorSuffix = ", which is expected to be in library bcprov-jdk15on-147.jar, but is absent in previously used library bcprov-jdk16-141.jar. Please check that it is present in your classpath together with bcpkix-jdk15on-147.jar";
        try {
            Class<?> c = Class.forName(checkClassName);
            Field field = c.getDeclaredField(checkFieldName);
            if (field == null) {
                check = "Cannot find static field " + checkFieldName + " in class " + checkClassName + errorSuffix;
            }
        } catch (ClassNotFoundException cnfe) {
            check = "Cannot find class " + checkClassName + errorSuffix;
        } catch (Throwable t) {
            check = "Cannot find static field " + checkFieldName + " in class " + checkClassName + errorSuffix;
        }
        return check;
    }


    public String checkForServer() {
        String check = OKMSG;
        String host = "localhost";
        int port = 8080;
        try {
            new Socket(host, port);
        } catch (Exception e) {
            check = "Unable to connect to '" + host + ":" + port + "' ! Is the server started ?";
        }
        return check;
    }

    public String checkForTomcat() {
        String check = OKMSG;
        String catalina = "catalina.properties";
        try {
            File rootFile = File.listRoots()[0];
            if (!(getTomcatPath(rootFile, catalina).size() > 0)) {
                check = "Checking for Tomcat - not able to find " + catalina;
            }
        } catch (FileNotFoundException e) {
            check = "Checking for Tomcat - not able to find " + catalina;
        }
        return check;
    }


    public String checkForAxis() {
        String check = OKMSG;
        try {
            URLClassLoader sysloader = (URLClassLoader) ClassLoader.getSystemClassLoader();
            sysloader.loadClass("org.apache.axiom.om.OMNode");
        } catch (ClassNotFoundException e) {
            return "Couldn't find Axis2 on classpath. ";
        }

        return check;
    }

    public String checkForMercury() {
        String check = OKMSG;
        try {
            URLClassLoader sysloader = (URLClassLoader) ClassLoader.getSystemClassLoader();
            sysloader.loadClass("org.wso2.mercury.handlers.MercuryOutHandler");
        } catch (ClassNotFoundException e) {
            return "Couldn't find Mercury on classpath. ";
        }

        return check;
    }

    public String checkForRampart() {
        String check = OKMSG;
        try {
            URLClassLoader sysloader = (URLClassLoader) ClassLoader.getSystemClassLoader();
            sysloader.loadClass("org.apache.rampart.RampartEngine");
        } catch (ClassNotFoundException e) {
            return "Couldn't find Rampart on classpath. ";
        }

        return check;
    }

    public String checkForAnt() {
        String check = OKMSG;
        String path = System.getProperty("java.library.path");
        if (!path.contains("ant") && !path.contains("Ant")) {
            check = "Checking for Ant - unable to locate Ant library in path: " + path + "! ";
            check += " - Is Ant installed ? Can ant be run from a command prompt ?";
        }
        return check;
    }

    public String checkOS() {
        String check = OKMSG;
        String os = System.getProperty("os.name");
        if (!os.startsWith("Windows") && !os.startsWith("Mac") && !os.startsWith("Linux")) {
            check = "Untested os: " + os;
        }
        return check;
    }

    public String checkEncryption() {
        String check = OKMSG;
        try {
            KeyGenerator kgen = KeyGenerator.getInstance("AES");
            kgen.init(256); // 192 and 256 bits may not be available
            kgen.init(new SecureRandom());
            SecretKey derived_key = kgen.generateKey();
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
            cipher.init(Cipher.ENCRYPT_MODE, derived_key);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            baos.write(cipher.getIV());
            baos.write(cipher.doFinal("Very secret secret".getBytes()));
            baos.close();

        } catch (Exception e) {
            check = "Checking encryption - not able to encrypt due to: " + e.getMessage();
            check += " - Is the policy files installed in the JRE ?";
        }
        return check;
    }

    /**
     * Directory is valid if it exists, does not represent a file, and can be read.
     */
    private void validateDirectory(File aDirectory) throws FileNotFoundException {
        if (aDirectory == null) {
            throw new IllegalArgumentException("Directory should not be null.");
        }
        if (!aDirectory.exists()) {
            throw new FileNotFoundException("Directory does not exist: " + aDirectory);
        }
        if (!aDirectory.isDirectory()) {
            throw new IllegalArgumentException("Is not a directory: " + aDirectory);
        }
        if (!aDirectory.canRead()) {
            throw new IllegalArgumentException("Directory cannot be read: " + aDirectory);
        }
    }

    private List<File> getTomcatPath(File rootFile, String stopFileName) throws FileNotFoundException {
        validateDirectory(rootFile);
        List<File> result = new ArrayList<File>();

        File[] filesAndDirs = rootFile.listFiles();
        List<File> filesDirs = Arrays.asList(filesAndDirs);
        for (File file : filesDirs) {
            if (!file.isFile()) {
                // must be a directory
                // recursive call!
                List<File> deeperList = getTomcatPath(file, stopFileName);
                result.addAll(deeperList);
                if (result.size() > 0) {
                    break;
                }
            } else if (file.getName().equalsIgnoreCase(stopFileName)) {
                result.add(file);
                break;
            }
        }
        Collections.sort(result);
        return result;
    }
}