/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.addressing;

import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.exception.NullOrEmptyArgumentException;
import dk.gov.oiosi.exception.UnexpectedNumberOfCharactersException;

/**
 * OVT Identifier.
 */
public class IdentifierOvt extends Identifier {

    /**
     * ContryCode part of OVT.
     */
    private String countryCode;

    private static final String keyTypeValue = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Identifiers/ovtNumber/";
    
    /**
     * BusinessIdentifier part of OVT.
     */
    private String businessIdentifier;

    /**
     * SerialNumber part of OVT.
     */
    private String serialNumber;

    /**
     * ContryCode part of OVT.
     * 
     * @return country code
     */
    public String getCountryCode() {
        return countryCode;
    }

    /**
     * BusinessIdentifier of OVT.
     * 
     * @return BusinessIdentifier
     */
    public String getBusinessIdentifier() {
        return businessIdentifier;
    }

    /**
     * SerialNumber part of OVT.
     * 
     * @return SerialNumber
     */
    public String getSerialNumber() {
        return serialNumber;
    }

    /**
     * Tries to return the business identifier as a CVR number identifier. Throws an exception if the format is not compatible.
     * 
     * @return The business identifier as a CVR number identifier.
     */
    public IdentifierCvr getBusinessIdentifierAsCvr() {
        return new IdentifierCvr(businessIdentifier);
    }

    private void validateCountryCode(String countryCode) {
        try {
            if (countryCode == null || countryCode.length() == 0) {
                throw new NullOrEmptyArgumentException("countryCode");
            }

            if (countryCode.length() != 4) {
                throw new UnexpectedNumberOfCharactersException("countryCode", 4);
            }
            Long.parseLong(countryCode);
        } catch (Exception ex) {
            throw new IncorrectCountryCodeException(countryCode, ex);
        }
    }

    private void validateBusinessIdentifier(String businessIdentifier) {
        // reuse test in IdentifierCvr
        new IdentifierCvr(businessIdentifier);
    }

    private void validateSerialNumber(String serialNumber) {
        try {
            if (serialNumber == null || serialNumber.length() == 0) {
                throw new NullOrEmptyArgumentException("serialNumber");
            }

            if (serialNumber.length() != 5) {
                throw new UnexpectedNumberOfCharactersException("serialNumber", 5);
            }
            Long.parseLong(serialNumber);
        } catch (Exception ex) {
            throw new IncorrectSerialNumberException(serialNumber, ex);
        }
    }

    /**
     * constructor setting the ovtnumber
     * 
     * @param ovtNumber
     *            the ovt number to set
     */
    public IdentifierOvt(String ovtNumber) {
        set(ovtNumber);
    }

    /**
     * 
     * Constructor
     * 
     * @param countryCode
     *            countrycode
     * @param cvrNumber
     *            cvr number
     * @param serialNumber
     *            certificate serialnumber
     */
    public IdentifierOvt(String countryCode, IdentifierCvr cvrNumber, String serialNumber) {
        if (cvrNumber == null) {
            throw new ArgumentNullException("cvrNumber");
        }

        validateCountryCode(countryCode);
        validateSerialNumber(serialNumber);
        this.countryCode = countryCode;
        this.serialNumber = serialNumber;
        this.businessIdentifier = cvrNumber.getAsString();
    }

    /**
     * 
     * Constructor
     * 
     * @param countryCode
     *            countrycode
     * @param businessIdentifier
     *            businessidentifier
     * @param serialNumber
     *            certificate serialnumber
     */
    public IdentifierOvt(String countryCode, String businessIdentifier, String serialNumber) {
        validateCountryCode(countryCode);
        validateSerialNumber(serialNumber);
        validateBusinessIdentifier(businessIdentifier);

        this.countryCode = countryCode;
        this.serialNumber = serialNumber;
        this.businessIdentifier = new IdentifierCvr(businessIdentifier).getAsString();
    }

    /**
     * Validates and sets ovt number
     * 
     * @param ovtNumber
     */
    public void set(String ovtNumber) {
        try {
            if (ovtNumber == null || ovtNumber.length() == 0)
                throw new NullOrEmptyArgumentException("ovtNumber");
            // 1. Check length:
            if (ovtNumber.length() != 17)
                throw new UnexpectedNumberOfCharactersException("ovtNumber", 17);
        } catch (Exception ex) {
            throw new IncorrectOvtNumberException(ovtNumber, ex);
        }

        // 2. Split:
        countryCode = ovtNumber.substring(0, 4);
        businessIdentifier = ovtNumber.substring(4, 12);
        serialNumber = ovtNumber.substring(12);

        // 3. Validate:
        validateBusinessIdentifier(businessIdentifier);
        validateCountryCode(countryCode);
        validateSerialNumber(serialNumber);
    }

    /**
     * 
     * returns ovt number as a string
     * 
     * @return the Ovt number as a string
     */
    public String getAsString() {
        return countryCode + businessIdentifier + serialNumber;
    }

    /**
     * 
     * Compares the two objects and returns true if they have equal values
     * 
     * @param other
     *            The object to compare to
     * @return Returns true if the two objects have identical values
     */
    public boolean equals(IdentifierOvt other) {
        if (other == null) {
            return false;
        }

        if (!getAsString().equals(other.getAsString())) {
            return false;
        } else {
            return true;
        }
    }

    @Override
    public String getType() {
        return "OVT";
    }

	@Override
	public String getKeyTypeValue() {
		return keyTypeValue;
	}

	@Override
	public boolean isAllowedInPublic() {
		return true;
	}

}
