/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.axis2.module.xml;

import dk.gov.oiosi.axis2.module.RaspHandler;
import dk.gov.oiosi.axis2.module.RaspModule;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.AxisFaultUtil;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.policy.OiosiPolicyBean;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schematron.*;
import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.description.Parameter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.xml.transform.Transformer;

/**
 * Schematron validation in axis stack.
 */
public class SchematronHandler extends RaspHandler {

    private static final Log log = LogFactory.getLog(SchematronHandler.class);

    public static final String NAME = "SchematronHandler";

    /**
     * Description.
     */
    private static HandlerDescription EMPTY_HANDLER_METADATA = new HandlerDescription("default Handler");

    private HandlerDescription handlerDesc;

    /**
     * Constructor.
     */
    public SchematronHandler() {
        this.handlerDesc = EMPTY_HANDLER_METADATA;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#init(org.apache.axis2.description.HandlerDescription)
     */
    public void init(HandlerDescription handlerDescription) {
        this.handlerDesc = handlerDescription;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#invoke(org.apache.axis2.context.MessageContext)
     */
    public InvocationResponse invoke(MessageContext msgContext) throws AxisFault {
        log.debug("SchematronHandler : invoke");
        if (!msgContext.isEngaged(RaspModule.MODULE_NAME)) {
            return InvocationResponse.CONTINUE;
        }
        if (RaspModule.getInstance().isRMNamespaceByAction(msgContext.getSoapAction())) {
            // only for payload requests
            return InvocationResponse.CONTINUE;
        }
        OiosiPolicyBean oiosiPolicyBean = getPolicyBean(msgContext);
        if (!oiosiPolicyBean.doSchematron()) {
            return InvocationResponse.CONTINUE;
        }
        log.debug("SchematronHandler : do schematron ");
        OMElement doc = msgContext.getEnvelope().getBody().getFirstElement();
        XmlDocument xmlDocument = new XmlDocument(doc.cloneOMElement());
        try {
            // Find Document
            DocumentTypeConfig documentTypeConfig = getDocumentTypeConfig(xmlDocument);
            // Schematron check document
            doSchematronCheck(documentTypeConfig, xmlDocument);
        } catch (MainException e) {
            throw AxisFaultUtil.buildAxisFault(msgContext, e);
        }
        return InvocationResponse.CONTINUE;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#flowComplete(org.apache.axis2.context.MessageContext)
     */
    public void flowComplete(MessageContext arg0) {
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#getName()
     */
    public String getName() {
        return NAME;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#getParameter(java.lang.String)
     */
    public Parameter getParameter(String name) {
        return this.handlerDesc.getParameter(name);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#getHandlerDesc()
     */
    public HandlerDescription getHandlerDesc() {
        return handlerDesc;
    }

    private DocumentTypeConfig getDocumentTypeConfig(XmlDocument xmlDocument) throws MainException {
        try {
            log.debug("Find DocumentTypeConfig");
            DocumentTypeConfigSearcher documentSearcher = new DocumentTypeConfigSearcher();
            return documentSearcher.findUniqueDocumentType(xmlDocument);

        } catch (MainException e) {
            log.error("Failed find document config : " + e);
            throw e;
        } catch (Exception e) {
            log.error("Failed find document config : " + e, e);
            throw new MainException(e);
        }
    }

    private void doSchematronCheck(DocumentTypeConfig documentTypeConfig, XmlDocument xmlDocument) throws MainException {
        log.debug("Schematron Validate Document.");
        if (documentTypeConfig.getSchematronValidationConfig() == null) {
            log.debug(" - configuration of schematron was null, skipping.");
            return;
        }
        SchematronValidationConfig schematronValidationConfig = documentTypeConfig.getSchematronValidationConfig();
        SchematronStore store = SchematronStoreFactory.getSchematronStore();
        Transformer transform = store.getCompiledSchematron(schematronValidationConfig.getSchematronDocumentPath());
        try {
            SchematronValidator validator = new SchematronValidator(schematronValidationConfig.getErrorXPath(), schematronValidationConfig.getErrorMessageXPath());
            validator.schematronValidateXmlDocument(xmlDocument, transform);
            log.debug("- Schematron validate OK ");
        } catch (SchematronErrorException e) {
            log.warn("Failed schematron document validation: " + e);
            throw e;
        } catch (Exception e) {
            log.warn("Failed schematron document validation: " + e, e);
            throw new MainException(e);
        }
    }
}