/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.common;

import java.io.Serializable;
import java.util.concurrent.TimeUnit;

/**
 * A class modeling the .NET class TimeSpan, which allows to have time span in different time units. Unfortunately, before JDK 7 there is no
 * such class out of the box.
 */
public class TimeSpan implements Serializable {

    private static final long serialVersionUID = 5433707766357587568L;

    /**
     * Time span in millis.
     */
    private long span;

    /*
      * Let's use java.util.concurrent.TimeUnit as a base for time unit tracking,
      * it has many convenient methods for this.
      */
    /**
     * Together with timeUnitSpan, keeps time unit, by which this class was initialized for logging purposes - it is more clear to read "1
     * HOURS" than "3600000".
     *
     * @since OIORASP 1.3.0
     */
    private TimeUnit timeUnit;

    /**
     * Together with timeUnit, keeps time unit amount, by which this class was initialized for logging purposes - it is more clear to read
     * "1 HOURS" than "3600000".
     *
     * @since OIORASP 1.3.0
     */
    private long timeUnitSpan;

    /**
     * Default Constructor with a time span of 0.
     */
    public TimeSpan() {
        timeUnit = TimeUnit.MILLISECONDS;
        timeUnitSpan = 0;
        updateSpan();
    }

    /**
     * Constructs a TimeSpan in TimeUnits
     *
     * @param unit   time unit
     * @param amount time amount
     */
    public TimeSpan(TimeUnit unit, long amount) {
        this.timeUnit = unit;
        this.timeUnitSpan = amount;
        updateSpan();
    }

    /**
     * Constructs a TimeSpan of spanMillis milliseconds.
     *
     * @param spanMillis The number of milliseconds in this time span
     */
    public TimeSpan(final long spanMillis) {
        this.timeUnit = TimeUnit.MILLISECONDS;
        this.timeUnitSpan = spanMillis;
        updateSpan();
    }

    /**
     * Returns the timespan.
     *
     * @return The time span in milliseconds.
     */
    public final long getSpanMillis() {
        return span;
    }

    /**
     * @return Similar to getSpanMillis(), returns the time span in minutes, but not less than 1, if there are more than 1 millisecond
     *         inside
     * @since OIORASP 1.3.0
     */
    public int getSpanMinutes() {
        if (timeUnitSpan == 0) {
            return 0;
        }
        /*
           * If we have more then 1 millis span, conversion to minutes will give 1
           * minute always to avoid problems at client code... It is better then 0
           * minutes time span. It is a question of rounding - let's say that we
           * round up when convert from millis to minutes
           */
        int minutes = (int) this.timeUnit.toMinutes(timeUnitSpan);
        if (minutes < 1) {
            minutes = 1;
        }
        return minutes;
    }

    /**
     * Set the timespan.
     *
     * @param millis timespan.
     * @return timespan in millis.
     */
    public final TimeSpan setMillis(final long millis) {
        timeUnit = TimeUnit.MILLISECONDS;
        timeUnitSpan = millis;
        updateSpan();
        return this;
    }

    /**
     * Set the timespan in seconds. eg. new TimeSpan().setSeconds( 42 );
     *
     * @param seconds timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setSeconds(final long seconds) {
        timeUnit = TimeUnit.SECONDS;
        timeUnitSpan = seconds;
        updateSpan();
        return this;
    }

    /**
     * Set the timespan in minutes. eg. new TimeSpan().setMinuttes( 42 ); Kept for backward compatibility, please use setMinutes() method with
     * fixed mistyping.
     *
     * @param minuttes timespan.
     * @return the TimeSpan object.
     * @deprecated Since 2.0.0: Please use setMinutes without mistyping in method name
     */
    public final TimeSpan setMinuttes(final long minuttes) {
        return setMinutes(minuttes);
    }

    /**
     * Set the timespan in minutes. eg. new TimeSpan().setMinutes( 42 );
     *
     * @param minutes timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setMinutes(final long minutes) {
        timeUnit = TimeUnit.MINUTES;
        timeUnitSpan = minutes;
        updateSpan();
        return this;
    }

    /**
     * Set the timespan in hours. eg. new TimeSpan().setHours( 42 );
     *
     * @param hours timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setHours(final long hours) {
        timeUnit = TimeUnit.HOURS;
        timeUnitSpan = hours;
        updateSpan();
        return this;
    }

    /*
      * General method to convert timeUnit way of keeping time duration to
      * milliseconds
      */
    private void updateSpan() {
        span = timeUnit.toMillis(timeUnitSpan);
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append(timeUnitSpan);
        builder.append(" ");
        builder.append(timeUnit);
        return builder.toString();
    }

    public TimeUnit getTimeUnit() {
        return timeUnit;
    }

    public long getTimeUnitSpan() {
        return timeUnitSpan;
    }
}