package dk.gov.oiosi.common.cache.stat;

import dk.gov.oiosi.common.cache.ICache;

import java.io.Serializable;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Generic cache statistics container for {@link ICache} implementations.
 * <p>
 * Incapsulates thread-safe container, which can be accessed concurrently in
 * many threads.
 * <p>
 * In general, this statistics is supposed to be stored in logs (or even
 * accessible via JMX).
 * <p>
 * Such statistics is important to be able to make decision regarding best
 * configuration of caches in OIORASP.
 * <p>
 * There can be 2 critical situations:
 * <p>
 * 1) OutOfMemory error - so potentially some cache is too big
 * <p>
 * 2) Too often lookup for resource, which is missing in cache - so potentially
 * some cache is too little.
 * <p>
 * Dumping state of this object into logs can help to identify one of these
 * situations.
 *
 * @author Dmitriy Lapko, TrueLink A/S
 *
 * @since OIORASP 1.3.0
 */
public class CacheStatisticsData implements Serializable {

	private static final long serialVersionUID = -7324520552442490232L;

	/*
	 * Let's use JRE 1.5 AtomicInteger to guarantee that even in multithreaded
	 * environment statistics values are absolutely correct. It is important to
	 * have trust in this statistics otherwise at least at server side of
	 * OIORASP usage there are possible invalid results in case of many
	 * simultanious calls.
	 *
	 * Also, let's set separate counter for each ICache method to get full
	 * generic overview. In general it is not that necessary to have all these
	 * details, but at least nothing will be lost, and it is cheap to keep it.
	 *
	 * Important: all fields should be independent from each other.
	 *
	 * E.g. we should not have containsTotalCount, because at some point
	 * containsMatchCount + containsMissCount could not be equal to
	 * containsTotalCount.
	 *
	 * Important: in order to hide internal details of this class
	 * (AtimicInteger), all access to these fields should be hidden from outside
	 * client code.
	 */
	/**
	 * Number of times the method {@link ICache}.contains was called and
	 * returned true - so there was such value in the cache
	 */
	protected AtomicInteger containsMatchCount;
	/**
	 * Number of times the method {@link ICache}.contains was called but
	 * returned false - so there was no such value in the cache
	 */
	protected AtomicInteger containsMissCount;

	/**
	 * Number of times the method {@link ICache}.tryGet was called and returned
	 * true with found value from the cache
	 */
	protected AtomicInteger tryGetMatchCount;
	/**
	 * Number of times the method {@link ICache}.tryGet was called but returned
	 * false - so there was no such value in the cache
	 */
	protected AtomicInteger tryGetMissCount;

	/**
	 * Number of times the method {@link ICache}.remove was called
	 */
	protected AtomicInteger removeTotalCount;
	/**
	 * Number of times the method {@link ICache}.set was called
	 */
	protected AtomicInteger setTotalCount;
	/**
	 * Number of times the method {@link ICache}.add was called
	 */
	protected AtomicInteger addTotalCount;

	/**
	 * Number of times the method {@link ICache}.checkExpired was called
	 */
	protected AtomicInteger checkExpiredTotalCount;
	/**
	 * Number of times the method {@link ICache}.checkExpired removed something
	 * from the cache
	 */
	protected AtomicInteger checkExpiredTotalRemovedCount;

	public CacheStatisticsData() {
		init();
	}

	public void init() {
		this.addTotalCount = new AtomicInteger();
		this.checkExpiredTotalCount = new AtomicInteger();
		this.checkExpiredTotalRemovedCount = new AtomicInteger();
		this.containsMatchCount = new AtomicInteger();
		this.containsMissCount = new AtomicInteger();
		this.removeTotalCount = new AtomicInteger();
		this.setTotalCount = new AtomicInteger();
		this.tryGetMatchCount = new AtomicInteger();
		this.tryGetMissCount = new AtomicInteger();
	}

	/*
	 * Get methods
	 */

	public int getContainsMatchCount() {
		return containsMatchCount.get();
	}

	public int getContainsMissCount() {
		return containsMissCount.get();
	}

	public int getTryGetMatchCount() {
		return tryGetMatchCount.get();
	}

	public int getTryGetMissCount() {
		return tryGetMissCount.get();
	}

	public int getRemoveTotalCount() {
		return removeTotalCount.get();
	}

	public int getSetTotalCount() {
		return setTotalCount.get();
	}

	public int getAddTotalCount() {
		return addTotalCount.get();
	}

	public int getCheckExpiredTotalCount() {
		return checkExpiredTotalCount.get();
	}

	public int getCheckExpiredTotalRemovedCount() {
		return checkExpiredTotalRemovedCount.get();
	}

	/*
	 * Increment methods by 1
	 */

	public int incrementContainsMatchCount() {
		return containsMatchCount.incrementAndGet();
	}

	public int incrementContainsMissCount() {
		return containsMissCount.incrementAndGet();
	}

	public int incrementTryGetMatchCount() {
		return tryGetMatchCount.incrementAndGet();
	}

	public int incrementTryGetMissCount() {
		return tryGetMissCount.incrementAndGet();
	}

	public int incrementRemoveTotalCount() {
		return removeTotalCount.incrementAndGet();
	}

	public int incrementSetTotalCount() {
		return setTotalCount.incrementAndGet();
	}

	public int incrementAddTotalCount() {
		return addTotalCount.incrementAndGet();
	}

	public int incrementCheckExpiredTotalCount() {
		return checkExpiredTotalCount.incrementAndGet();
	}

	public int incrementCheckExpiredTotalRemovedCount() {
		return checkExpiredTotalRemovedCount.incrementAndGet();
	}

	/*
	 * Increment methods by more than 1.
	 */

	/*
	 * Let's do it only for one counter where it is really needed in current
	 * implementation.
	 */
	public int incrementCheckExpiredTotalRemovedCount(int delta) {
		return this.checkExpiredTotalRemovedCount.addAndGet(delta);
	}

	/**
	 * Simple conversion to state string, supposed to be used only for logging
	 */
	public String toCompactStateString() {
		StringBuilder sb = new StringBuilder();
		if (tryGetMatchCount.intValue() > 0 || tryGetMissCount.intValue() > 0) {
			sb.append("tryGet(match/miss)=");
			sb.append(tryGetMatchCount);
			sb.append("/");
			sb.append(tryGetMissCount);
		}
		if (containsMatchCount.intValue() > 0 || containsMissCount.intValue() > 0) {
			if (sb.length() > 0) {
				sb.append(" ");
			}
			sb.append("contains(match/miss)=");
			sb.append(containsMatchCount);
			sb.append("/");
			sb.append(containsMissCount);
		}

		if (removeTotalCount.intValue() > 0 || setTotalCount.intValue() > 0 || addTotalCount.intValue() > 0) {
			if (sb.length() > 0) {
				sb.append(" ");
			}
			sb.append("remove/set/add=");
			sb.append(removeTotalCount);
			sb.append("/");
			sb.append(setTotalCount);
			sb.append("/");
			sb.append(addTotalCount);
		}
		if (checkExpiredTotalCount.intValue() > 0 || checkExpiredTotalRemovedCount.intValue() > 0) {
			if (sb.length() > 0) {
				sb.append(" ");
			}
			sb.append("checkExpired(total/removedCount)=");
			sb.append(checkExpiredTotalCount);
			sb.append("/");
			sb.append(checkExpiredTotalRemovedCount);
		}
		if (sb.length() > 0) {
			return sb.toString();
		}
		return "empty";
	}

	@Override
	public String toString() {
		return toCompactStateString();
	}

}
