package dk.gov.oiosi.common.schedule;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.concurrent.*;

/**
 * Main entry for scheduling tasks in OIORASP.
 * <p/>
 * Encapsulates exact implementation (for now it is JRE 1.5
 * java.util.concurrent.ScheduledExecutorService).
 * <p/>
 * Works with jobs, which extend AbstractScheduleJob. Uses
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class ScheduledExecutor {

    private static final Log log = LogFactory.getLog(ScheduledExecutor.class);

    protected static ScheduledExecutor instance = new ScheduledExecutor();
    protected ScheduledExecutorService executor;
    protected ScheduledThreadPoolExecutor executorExtInterface;

    protected ScheduledExecutor() {
        init();
    }

    protected void init() {
        int threadPoolSize = calculateThreadPoolSize();
        executor = Executors.newScheduledThreadPool(threadPoolSize);
        if (log.isInfoEnabled()) {
            StringBuilder sb = new StringBuilder();
            sb.append("Initialized ScheduledExecutor for ");
            sb.append(getAvailableCPU());
            sb.append(" cores CPU, thread pool size ");
            sb.append(threadPoolSize);
            sb.append(", impl. class ");
            sb.append(executor.getClass().getName());
            log.info(sb.toString());
        }
        /*
           * In case if implementation is default JDK, we can have additional
           * functionality to see number of active jobs via method getActiveCount
           */
        if (executor instanceof ScheduledThreadPoolExecutor) {
            executorExtInterface = (ScheduledThreadPoolExecutor) executor;
        }
    }

    protected int calculateThreadPoolSize() {
        int availableProcessors = getAvailableCPU();
        int threadPoolSize = availableProcessors < 2 ? 2 : availableProcessors;
        return threadPoolSize;
    }

    protected int getAvailableCPU() {
        return Runtime.getRuntime().availableProcessors();
    }

    public static ScheduledExecutor getInstance() {
        return instance;
    }

    public void scheduleJob(AbstractScheduleJob job, long interval, TimeUnit timeUnit) {
        try {
            executor.scheduleWithFixedDelay(job, interval, interval, timeUnit);
        } catch (RejectedExecutionException ree) {
            log.warn("ScheduledExecutor is requested to shutdown, skip scheduling a new job " + job);
            return;
        }
        if (log.isInfoEnabled()) {
            log.info("Scheduled " + job + " to run each " + interval + " " + timeUnit);
        }
    }

    public void runJobNow(AbstractScheduleJob job) {
        try {
            executor.execute(job);
        } catch (RejectedExecutionException ree) {
            log.warn("ScheduledExecutor is requested to shutdown, skip execution of a new job " + job);
        }
    }

    public void scheduleJob(AbstractScheduleJob job, Date nextUpdate, long interval, TimeUnit timeUnit) {
        long initialDelayMs = nextUpdate.getTime() - Calendar.getInstance().getTimeInMillis();
        long initialDelay = timeUnit.convert(initialDelayMs, TimeUnit.MILLISECONDS);
        try {
            executor.scheduleWithFixedDelay(job, initialDelay, interval, timeUnit);
        } catch (RejectedExecutionException ree) {
            log.warn("ScheduledExecutor is requested to shutdown, skip scheduling a new job " + job);
            return;
        }
        if (log.isInfoEnabled()) {
            StringBuilder sb = new StringBuilder();
            sb.append("Scheduled ");
            sb.append(job);
            sb.append(" to run each ");
            sb.append(interval);
            sb.append(" ");
            sb.append(timeUnit);
            sb.append(", first wait interval is ");
            sb.append(initialDelay);
            sb.append(" " + timeUnit + ", planned time of first run is ");
            sb.append(new Date(Calendar.getInstance().getTime().getTime() + timeUnit.toMillis(initialDelay)));
            log.info(sb.toString());
        }
    }

    /**
     * Initiates an orderly shutdown in which previously submitted tasks are
     * executed, but no new tasks will be accepted. Invocation has no additional
     * effect if already shut down.
     *
     * @throws SecurityException if a security manager exists and shutting down
     *                           this ExecutorService may manipulate threads that the caller is
     *                           not permitted to modify because it does not hold
     *                           {@link java.lang.RuntimePermission}<tt>("modifyThread")</tt>, or
     *                           the security manager's <tt>checkAccess</tt> method denies access.
     */
    public void shutdown() {
        executor.shutdown();
    }

    /**
     * Attempts to stop all actively executing tasks, halts the processing of
     * waiting tasks, and returns a list of the tasks that were awaiting
     * execution.
     * <p/>
     * <p/>
     * There are no guarantees beyond best-effort attempts to stop processing
     * actively executing tasks. For example, typical implementations will
     * cancel via {@link Thread#interrupt}, so any task that fails to respond to
     * interrupts may never terminate.
     *
     * @return list of tasks that never commenced execution
     * @throws SecurityException if a security manager exists and shutting down
     *                           this ExecutorService may manipulate threads that the caller is
     *                           not permitted to modify because it does not hold
     *                           {@link java.lang.RuntimePermission}<tt>("modifyThread")</tt>, or
     *                           the security manager's <tt>checkAccess</tt> method denies access.
     */
    public List<Runnable> shutdownNow() {
        return executor.shutdownNow();
    }

    /**
     * Returns <tt>true</tt> if all tasks have completed following shut down.
     * Note that <tt>isTerminated</tt> is never <tt>true</tt> unless either
     * <tt>shutdown</tt> or <tt>shutdownNow</tt> was called first.
     *
     * @return <tt>true</tt> if all tasks have completed following shut down
     */
    public boolean isTerminated() {
        return executor.isTerminated();
    }

    /**
     * Blocks until all tasks have completed execution after a shutdown request,
     * or the timeout occurs, or the current thread is interrupted, whichever
     * happens first.
     *
     * @param timeout the maximum time to wait
     * @param unit    the time unit of the timeout argument
     * @return <tt>true</tt> if this executor terminated and <tt>false</tt> if
     *         the timeout elapsed before termination
     * @throws InterruptedException if interrupted while waiting
     */
    public boolean awaitTermination(long timeout, TimeUnit unit) throws InterruptedException {
        return executor.awaitTermination(timeout, unit);
    }

    /**
     * Returns <tt>true</tt> if this executor has been shut down.
     *
     * @return <tt>true</tt> if this executor has been shut down
     */
    public boolean isShutdown() {
        return executor.isShutdown();
    }

    public int getCurrentActiveCount() {
        if (executorExtInterface != null) {
            return executorExtInterface.getActiveCount();
        }
        return -1;
    }
}