/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.communication.client.IAxisDispatcher;
import dk.gov.oiosi.communication.client.ProxyGenerationException;
import dk.gov.oiosi.communication.configuration.mail.IMailServerConfiguration;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.util.AxisUtil;
import dk.gov.oiosi.xml.XmlDocumentException;
import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.security.cert.X509Certificate;
import java.util.Map;

/**
 * Represents a request to a RASP HTTP or email endpoint.
 */
public class Request implements IRequest {

    /**
     * Local logger.
     */
    private Log log = LogFactory.getLog(Request.class);

    /**
     * Credentials set by caller.
     */
    private Credentials credentials;

    /**
     * Request/Destination URI.
     */
    private Uri requestUri;

    /**
     * Policy.
     */
    private SendPolicy policy = null;

    /**
     * Default constructor.
     */
    public Request() {
    }

    /**
     * Constructor
     *
     * @param endpointAddress endpoint URI
     * @throws MainException On error...
     */
    public Request(final Uri endpointAddress) throws MainException {
        // Test that a valid endpoint address was given
        testEndpointAddressCompability(endpointAddress.toString());
        setRequestUri(endpointAddress);
        setPolicy(ConfigurationHandler.getInstance().getSendPolicy());
    }

    /**
     * Constructor.
     *
     * @param endpointAddress endpoint uri
     * @param credentials     credentials
     * @throws MainException On error...
     */
    public Request(final Uri endpointAddress, final Credentials credentials) throws MainException {
        this(endpointAddress);
        setCredentials(credentials);
        setPolicy(ConfigurationHandler.getInstance().getSendPolicy());
    }

    /**
     * Creates a new RaspRequest instance bound to the specific endpoint. During
     * construction, the type of endpoint is inferred from the Uri scheme. After
     * construction, additional properties may be set for the request, such as
     * credentials
     *
     * @param credentials     Overrides the credentials set in config
     * @param endpointAddress Endpoint uri
     * @param sendPolicy      The send policy of the request
     * @throws MainException On error...
     */
    public Request(final Uri endpointAddress, final Credentials credentials, final SendPolicy sendPolicy) throws MainException {
        this(endpointAddress, credentials);
        setPolicy(sendPolicy);
    }

    /**
     * modify RASP call not to use RM.
     */
    private boolean useRM = true;

    /**
     * Set whether the RASP call should use reliable messaging (RM).
     *
     * @param useRM True, if the call should use RM
     */
    public void setUseRM(boolean useRM) {
        this.useRM = useRM;
    }

    /**
     * modify RASP call not to use WSS.
     */
    private boolean useWSS = true;

    /**
     * Set whether to use WS-Security in RASP call.
     *
     * @param useWSS True, if the call should use WS-Security
     */
    public void setUseWSS(boolean useWSS) {
        this.useWSS = useWSS;
    }

    /**
     * use this proxyport in HTTP soap request.
     */
    private int proxyPort = -1;

    /**
     * Use this proxyport in HTTP soap request.
     *
     * @param port The proxy port
     */
    public void setProxyPort(int port) {
        proxyPort = port;
    }

    /**
     * use this proxyhost in HTTP soap request.
     */
    private String proxyHost = null;

    /**
     * Use this proxyhost in HTTP soap request.
     *
     * @param host The proxy host
     */
    public void setProxyHost(String host) {
        proxyHost = host;
    }

    /**
     * Fallback axis configuration file.
     */
    private String axisConfigurationFile = "./conf/client_axis2.xml";

    /**
     * Set the fallback axis configuration file.
     *
     * @param value path to configuration file
     */
    public void setAxisConfigurationFile(String value) {
        axisConfigurationFile = value;
    }

    /**
     * Fallback axis repository folder.
     */
    private String axisRepositoryDir = "./test/conf/client";

    /**
     * Set the fallback axis repository folder.
     *
     * @param value path to Axis repository folder
     */
    public void setRepositoryDir(String value) {
        axisRepositoryDir = value;
    }

    /**
     * Tests that the endpoint address is compatible with Rasp requests
     *
     * @param endpointAddress endpoint URI
     * @throws RequestNoEndpointGivenException
     *          if input == null
     * @throws RequestNotSupportedSchemeException
     *          if unknown transport
     */
    protected void testEndpointAddressCompability(String endpointAddress) throws RequestNoEndpointGivenException, RequestNotSupportedSchemeException {

        if (endpointAddress == null) {
            throw new RequestNoEndpointGivenException();
        }

        // We only support 'http' and 'mailto'
        String scheme = endpointAddress.toLowerCase();
        if (!scheme.startsWith("mailto") && !scheme.startsWith("http")) {
            throw new RequestNotSupportedSchemeException(scheme);
        }
    }

    /**
     * The actual 'request'.
     */
    private IAxisDispatcher dispatcher = null;

    /**
     * Creates the proxy object. Note: This method is inherited from .Net...
     *
     * @throws ProxyGenerationException On error...
     */
    protected void createProxy() throws ProxyGenerationException {
        log.debug("Proxy being created");

        try {
            String wSSPolicyFileName = policy.getWSSPolicyFileName();
            RMPolicy httpRMPolicy = policy.getHttpRMPolicy();
            RMPolicy mailRMPolicy = policy.getMailRMPolicy();
            IMailServerConfiguration mailServerConfiguration = policy.getInboxMailConfiguration();

            dispatcher = IAxisDispatcher.Factory.getDispatcher(axisConfigurationFile, axisRepositoryDir, useWSS, useRM, wSSPolicyFileName, httpRMPolicy, mailRMPolicy, mailServerConfiguration, proxyPort, proxyHost);
        } catch (Exception e) {
            log.error("Failed to create Axis Dispatcher: " + e.getMessage(), e);
            throw new ProxyGenerationException(e);
        }
    }

    @SuppressWarnings("unused")
    private void validateParameters(OiosiMessage message) throws MainException {
        // test endpoint address
        if (requestUri == null) {
            throw new MainException("Missing : endpoint address");
        }

        // test credentials
        if (credentials == null) {
            throw new MainException("Missing : credentials");
        }

        // test policy
        if (policy == null) {
            throw new MainException("Missing : policy");
        }

    }

    /**
     * Synchronously sends a request and gets a response.
     *
     * @param message The message to send
     * @return The response message
     * @throws MainException
     */
    public Response getResponse(OiosiMessage message) throws MainException {
        if (log.isDebugEnabled()) {
            log.debug("Send Message - ID=" + message.getMessageIdentifier());
        }

        validateParameters(message);
        createProxy();
        return sendMessage(message);
    }


    /**
     * Send message. Gets response. Handles exception.
     *
     * @param message message
     * @return Response
     * @throws MainException On error...
     */
    private Response sendMessage(OiosiMessage message) throws MainException {
        Response response;
        OMElement soapReply;
        SignatureValidationProof[] signatureProof = new SignatureValidationProof[1];

        try {
            log.debug("Send request using Axis dispatcher");
            OMElement doc = message.getMessageXml().getOMDocument();
            String uri = getRequestUri().toString();
            String requestAction = message.getRequestAction();
            X509Certificate x509Certificate = credentials.getServerCertificate().getCertificate();
            Map<String, Object> messageHeaders = message.getMessageHeaders();
            Map<String, Object> properties = message.getProperties();

            soapReply = dispatcher.sendRaspRequest(doc, uri, requestAction, x509Certificate, messageHeaders, properties, signatureProof);

            if (soapReply == null) {
                // this is OK, and means that the server replied with an empty SOAP message
                response = new Response(null, null);
            } else {
                response = new Response(soapReply, null);
            }

            response.getResponseMessage().setSignatureProof(signatureProof[0]);
        } catch (XmlDocumentException e) {
            log.error("Failed to send document: " + e.getMessage(), e);
            throw e;
        } catch (AxisFault e) {
            /*
                * dlk 2012-04-04
                *
                * 1) Fixed error message - from "Failed to get document" to
                * "Failed to send document"
                *
                * 2) Extended logging of error message - added parsing of AxisFault
                * action and code. In case if a problem with .NET client receiving
                * 400 HTTP code is solved, it will allow to see reason of the
                * problem from server point of view - is it Sender or Receiver,
                * which is important for Sender to know, should it try to resend a
                * document or something wrong with it or with configuration.
                */
            String faultRole = AxisUtil.extractFaultCode(e.getFaultCode());
            boolean receiverError = "Receiver".equals(faultRole);
            String soapFaultAction = AxisUtil.extractFaultAction(e.getFaultSubCodes());

            log.error("Failed to send document (AxisFault): role=" + faultRole + ", receiverError=" + receiverError + ", parsedAction=" + soapFaultAction, e);

            OiosiCommunicationException oce = new OiosiCommunicationException(e.getMessage(), e);
            oce.setSoapFaultAction(soapFaultAction);
            oce.setReceiverError(receiverError);
            throw oce;
        } catch (MainException e) {
            throw e;
        } catch (Exception e) {
            log.error("Failed to send document: " + e.getMessage(), e);
            throw new RaspLibraryException(e.getMessage(), e);
        }

        // 4. Return
        return response;
    }

    /**
     * Gets the credentials
     *
     * @return the credentials
     */
    public Credentials getCredentials() {
        return credentials;
    }

    /**
     * Sets the credentials
     *
     * @param value The new credentials for the request
     */
    public void setCredentials(Credentials value) {
        credentials = value;
    }

    /**
     * Gets the request URI
     *
     * @return the request URI
     */
    public Uri getRequestUri() {
        return requestUri;
    }

    /**
     * Sets the request URI
     *
     * @param u The request URI to use
     */
    public void setRequestUri(Uri u) {
        requestUri = u;
    }

    /**
     * Gets the send policy
     *
     * @return the send policy
     */
    public SendPolicy getPolicy() {
        return policy;
    }

    /**
     * Sets the send policy
     *
     * @param value The send policy to use
     */
    public void setPolicy(SendPolicy value) {
        policy = value;
        setAxisConfigurationFile(value.getAxisConfigurationFile());
        setRepositoryDir(value.getAxisRepositoryDir());
    }
}