/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication.client;

import dk.gov.oiosi.communication.RMPolicy;
import dk.gov.oiosi.communication.configuration.mail.IMailServerConfiguration;
import dk.gov.oiosi.security.SignatureValidationProof;
import org.apache.axiom.om.OMElement;

import java.security.cert.X509Certificate;
import java.util.Map;

/**
 * Interface and Factory for Axis low-level dispatchers.
 */
public interface IAxisDispatcher {

    /**
     * Factory for constructing low level dispatchers.
     */
    public static class Factory {


        /**
         * Create dispatcher.
         *
         * @param axisXml                      axis configuration file
         * @param axisRepository               axis repository folder
         * @param wss                          use wss
         * @param rm                           use rm
         * @param wssPolicy                    wss policy file
         * @param httpPolicy                   rm http policy
         * @param mailPolicy                   rm mail policy
         * @param inboxMailServerConfiguration IMailServerConfiguration
         * @param proxyPort                    proxy port
         * @param proxyHost                    proxy host
         * @return dispatcher
         * @throws ProxyGenerationException On error...
         */
        public static IAxisDispatcher getDispatcher(String axisXml, String axisRepository, boolean wss, boolean rm,
                                                    String wssPolicy, RMPolicy httpPolicy, RMPolicy mailPolicy, IMailServerConfiguration inboxMailServerConfiguration, int proxyPort, String proxyHost)
                throws ProxyGenerationException {
            synchronized (Factory.class) {
                try {
                    IAxisDispatcher dispatcher = new AxisMercuryDispatcher(axisXml, axisRepository, wssPolicy);
                    dispatcher.setUseWSS(wss);
                    dispatcher.setUseRM(rm);

                    if (httpPolicy != null) {
                        dispatcher.setHttpRMPolicy(httpPolicy);
                    }
                    if (mailPolicy != null) {
                        dispatcher.setMailRMPolicy(mailPolicy);
                    }
                    if (inboxMailServerConfiguration != null) {
                        dispatcher.setInboxMailServerConfiguration(inboxMailServerConfiguration);
                    }
                    if (proxyPort > 0) {
                        dispatcher.setProxy(proxyHost, proxyPort);
                    }
                    return dispatcher;
                } catch (Exception e) {
                    throw new ProxyGenerationException(e);
                }
            }
        }

        /*public static void registerConfigurationContext(String axisXml, String repository, ConfigurationContext configurationContext) {

            AxisMercuryDispatcher.registerConfigurationContext(axisXml, repository, configurationContext);

        }*/

        public static void shutdownContext() throws Exception {
            AxisMercuryDispatcher.shutdownContext();
        }
    }

    /**
     * Method for dispatching message.
     *
     * @param doc             the document to send
     * @param endpointAddress endpoint uri
     * @param action          request action
     * @param cert            receiver certificate
     * @param customHeaders   custom headers
     * @return response message
     * @throws Exception On error...
     */
    public OMElement sendRaspRequest(OMElement doc, String endpointAddress, String action, X509Certificate cert,
                                     Map<String, Object> customHeaders, Map<String, Object> properties, SignatureValidationProof[] signatureValidationProof) throws Exception;

    public void setInboxMailServerConfiguration(IMailServerConfiguration inboxMailServerConfiguration);

    public void setMailRMPolicy(RMPolicy mailPolicy);

    public void setHttpRMPolicy(RMPolicy httpPolicy);

    /**
     * Use WSS.
     *
     * @param b use wss
     */
    public void setUseWSS(boolean b);

    /**
     * Use RM.
     *
     * @param b
     */
    public void setUseRM(boolean b);

    /**
     * Use Proxy.
     *
     * @param proxyHost host
     * @param proxyPort port
     */
    public void setProxy(String proxyHost, int proxyPort);
}