/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication.configuration;

import dk.gov.oiosi.common.Guid;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminatorConfigCollection;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A collection of DocumentTypeConfig.
 */
public class DocumentTypeCollectionConfig {

    /**
     * List of DocumentTypeConfig.
     */
    private List<DocumentTypeConfig> documentTypes = new ArrayList<DocumentTypeConfig>();

    /**
     * A list of RASP document types supported by the client (e.g. Invoices, Notifications...).
     *
     * @return DocumentTypes
     */
    public DocumentTypeConfig[] getDocumentTypes() {
        return documentTypes.toArray(new DocumentTypeConfig[documentTypes.size()]);
    }

    /**
     * A list of RASP document types supported by the client (e.g. Invoices, Notifications...).
     *
     * @param value DocumentTypes
     */
    public void setDocumentTypes(List<DocumentTypeConfig> value) {
        documentTypes = value;
    }

    /**
     * Adds a new RASP document type to the configuration.
     *
     * @param documentTypeConfig document type to add
     */
    public void addDocumentType(DocumentTypeConfig documentTypeConfig) throws DocumentAllreadyAddedException {
        if (documentTypeConfig == null) {
            throw new ArgumentNullException("documentType");
        }
        if (containsDocumentTypeByValue(documentTypeConfig)) {
            throw new DocumentAllreadyAddedException(documentTypeConfig.getFriendlyName());
        }
        documentTypes.add(documentTypeConfig);
    }

    /**
     * Removes a specific document type from the configuration.
     *
     * @param documentTypeConfig remove this
     */
    public void removeDocumentType(DocumentTypeConfig documentTypeConfig) {
        if (documentTypeConfig == null) {
            throw new ArgumentNullException("documentType");
        }
        documentTypes.remove(documentTypeConfig);
    }

    /**
     * Clears the document list.
     */
    public void clear() {
        if (documentTypes != null) {
            documentTypes.clear();
        }
    }

    /**
     * Returns whether a certain document type is in the collection. The document type is in the collection if it has the same id or has the
     * same root name, root namespace and identifier discriminators.
     *
     * @param documentTypeConfig
     * @return whether the documenttype is in the configuration
     */
    public boolean containsDocumentTypeByValue(DocumentTypeConfig documentTypeConfig) {
        return documentTypes.contains(documentTypeConfig);
    }

    /**
     * Returns whether a certain document type is in the collection. The document type is in the collectio if the reference is the same.
     *
     * @param documentTypeConfig
     * @return whether the documenttype is in the configuration
     */
    public boolean containsDocumentTypeByReference(DocumentTypeConfig documentTypeConfig) {
        return documentTypes.contains(documentTypeConfig);
    }

    /**
     * Gets the documents types with a certain root name and root namespace. This does not uniqely identfies a certain document type hence
     * multiple document types can be returned as result.
     *
     * @param rootName
     * @param rootNamespace
     * @return document types
     */
    public Iterator<DocumentTypeConfig> getDocumentTypes(String rootName, String rootNamespace) {
        List<DocumentTypeConfig> results = new ArrayList<DocumentTypeConfig>();
        for (DocumentTypeConfig config : documentTypes) {
            if (config.getRootName().equals(rootName) && config.getRootNamespace().equals(rootNamespace)) {
                results.add(config);
            }
        }
        return results.iterator();
    }

    /**
     * Get a document type from a root name, root namespace and a collection of identifier expressions.
     *
     * @param rootName
     * @param rootNamespace
     * @param identifierDiscriminators
     * @return a specific document type
     */
    public DocumentTypeConfig getDocumentType(String rootName, String rootNamespace,
                                              XPathDiscriminatorConfigCollection identifierDiscriminators) throws NoDocumentTypeFoundFromParametersException,
            AmbiguousDocumentTypeResultFromParametersException, ArgumentNullException {
        if (rootName == null) {
            throw new ArgumentNullException("rootName");
        }
        if (rootNamespace == null) {
            throw new ArgumentNullException("rootNamespace");
        }
        if (identifierDiscriminators == null) {
            throw new ArgumentNullException("identifierDiscriminators");
        }

        DocumentTypeConfig documentType = tryGetDocumentType(rootName, rootNamespace, identifierDiscriminators);
        if (documentType == null) {
            throw new NoDocumentTypeFoundFromParametersException(rootName, rootNamespace, identifierDiscriminators);
        }
        return documentType;
    }

    /**
     * Get a document type from the id
     *
     * @param id
     * @return a specific document type
     */
    public DocumentTypeConfig getDocumentType(Guid id) throws NoDocumentTypeFoundFromIdException,
            AmbiguousDocumentTypeResultFromIdException {
        DocumentTypeConfig documentType = tryGetDocumentType(id);
        if (documentType == null) {
            throw new NoDocumentTypeFoundFromIdException(id);
        }
        return documentType;
    }

    /**
     * Try to get the document type from a root name, root namespace and a collection of identifier expressions.
     *
     * @param rootName
     * @param rootNamespace
     * @param identifierDiscriminators
     * @return a specific document type
     */
    public DocumentTypeConfig tryGetDocumentType(String rootName, String rootNamespace,
                                                 XPathDiscriminatorConfigCollection identifierDiscriminators) throws AmbiguousDocumentTypeResultFromParametersException,
            ArgumentNullException {
        if (rootName == null) {
            throw new ArgumentNullException("rootName");
        }
        if (rootNamespace == null) {
            throw new ArgumentNullException("rootNamespace");
        }
        if (identifierDiscriminators == null) {
            throw new ArgumentNullException("identifierDiscriminators");
        }

        List<DocumentTypeConfig> searchList = new ArrayList<DocumentTypeConfig>();
        for (DocumentTypeConfig config : documentTypes) {
            if (config.getRootName().equals(rootName) && config.getRootNamespace().equals(rootNamespace)
                    && config.getIdentifierDiscriminators().equals(identifierDiscriminators)) {

                searchList.add(config);
            }
        }

        if (searchList.size() < 1) {
            return null;
        }
        if (searchList.size() > 1) {
            throw new AmbiguousDocumentTypeResultFromParametersException(rootName, rootNamespace, identifierDiscriminators);
        }
        return (DocumentTypeConfig) searchList.get(0);
    }

    /**
     * Try to get the document type with a certain id
     *
     * @param id
     * @return a specific document type
     */
    public DocumentTypeConfig tryGetDocumentType(Guid id) throws AmbiguousDocumentTypeResultFromIdException {
        List<DocumentTypeConfig> searchList = new ArrayList<DocumentTypeConfig>();

        for (DocumentTypeConfig config : documentTypes) {
            if (config.getId().equals(id)) {
                searchList.add(config);
            }
        }
        if (searchList.size() < 1) {
            return null;
        }
        if (searchList.size() > 1) {
            throw new AmbiguousDocumentTypeResultFromIdException(id);
        }
        return (DocumentTypeConfig) searchList.get(0);
    }
}