/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication.configuration;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

/**
 * Contains mapping expression.
 */
public class KeyTypeMappingExpression {

    private String name;
    private String xpathExpression;

    /**
     * mapping from string to KeyTypeMapping.
     */
    private Map<String, KeyTypeMapping> mappings;

    /**
     * Default constructor used by the XmlSerializer.
     * 
     * NOTE: Do not use this constructor
     */
    public KeyTypeMappingExpression() {
        this("", "");
    }

    /**
     * Constructor that creates a mapping expression with the given name and
     * xpath expression. Further it has no mappings.
     * 
     * @param n
     *            name
     * @param xpath
     *            xpath
     */
    public KeyTypeMappingExpression(String n, String xpath) {
        name = n;
        xpathExpression = xpath;
        mappings = new Hashtable<String, KeyTypeMapping>();
    }

    /**
     * Constructor that creates a mapping expression with the given name, xpath
     * expression and mappings. It copies the mappings from the IEnumerable
     * given.
     * 
     * @param n
     *            name
     * @param xpath
     *            xpath
     * @param initialMappings
     *            mappings
     */
    public KeyTypeMappingExpression(String n, String xpath, Iterator<KeyTypeMapping> initialMappings) {
        name = n;
        xpathExpression = xpath;
        setMappingsInternal(initialMappings);
    }

    /**
     * Gets and sets the name.
     * 
     * @return name
     */
    public String getName() {
        return name;
    }

    /**
     * Gets and sets the name.
     * 
     * @param value
     *            name
     */
    public void setName(String value) {
        name = value;
    }

    /**
     * Gets and sets the xpath expression.
     * 
     * @return xpath
     */
    public String getXPathExpression() {
        return xpathExpression;
    }

    /**
     * Gets and sets the xpath expression.
     * 
     * @param value
     *            xpath
     */
    public void setXPathExpression(String value) {
        xpathExpression = value;
    }

    /**
     * Gets and sets the mappings. It is used by the xml serilizer, it should
     * not be used otherwise.
     * 
     * <remarks> This property should not be used.</remarks>
     * 
     * @return all mappings
     */
    public KeyTypeMapping[] getMappings() {
        return (KeyTypeMapping[]) mappings.values().toArray();
    }

    /**
     * Gets and sets the mappings. It is used by the xml serilizer, it should
     * not be used otherwise.
     * 
     * @param value
     *            mappings
     */
    public void setMappings(Iterator<KeyTypeMapping> value) {
        setMappingsInternal(value);
    }

    /**
     * Gets a mapping with a given value.
     * 
     * @param value
     *            key
     * @return key type mapping
     * @throws KeyTypeMappingFailedException
     */
    public KeyTypeMapping getMapping(String value) throws KeyTypeMappingFailedException {
        try {
            return (KeyTypeMapping) mappings.get(value);
        } catch (Exception ex) {
            throw new KeyTypeMappingFailedException(value, ex);
        }
    }

    /**
     * Adds a mapping.
     * 
     * @param mapping
     */
    public void addMapping(KeyTypeMapping mapping) {
        mappings.put(mapping.getValue(), mapping);
    }

    /**
     * Removes the mapping with the given name
     * 
     * @param value
     */
    public void removeMapping(String value) {
        mappings.remove(value);
    }

    /**
     * Private set mapping.
     * 
     * @param initMappings
     */
    private void setMappingsInternal(Iterator<KeyTypeMapping> initMappings) {
        mappings.clear();
        while (initMappings.hasNext()) {
            KeyTypeMapping mapping = initMappings.next();
            mappings.put(mapping.getValue(), mapping);
        }
    }
}
