/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication.configuration;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

/**
 * XPath to a service endpoint unique ID (such as e.g. an EAN number), used for
 * UDDI lookups, and an endpoint ID type.
 */
public class ServiceEndpointKey {

    /**
     * XPath.
     */
    private String xPath = "";

    /**
     * Mappings.
     */
    private Map<String, KeyTypeMappingExpression> mappingExpressions = new Hashtable<String, KeyTypeMappingExpression>();

    /**
     * Default constructor.
     */
    public ServiceEndpointKey() { }

    /**
     * The key for finding the endpoint in a UDDI lookup.
     * 
     * @param x
     *            The XPath expression to where in the document the key can be
     *            found
     */
    public ServiceEndpointKey(String x) {
        xPath = x;
    }

    /**
     * XPath expression, to where in the UBL document the key can be found.
     * 
     * @return XPath
     */
    public String getXPath() {
        return xPath;
    }

    /**
     * XPath expression, to where in the UBL document the key can be found.
     * 
     * @param value
     *            XPath
     */
    public void setXPath(String value) {
        xPath = value;
    }

    /**
     * Sets the mapping expressions.
     * 
     * @param value
     *            mappings
     */
    public void setMappingExpressions(Iterator<KeyTypeMappingExpression> value) {
        setMappingExpressionsInternal(value);
    }

    /**
     * Gets a mapping expression with a given name.
     * 
     * @param name
     *            key
     * @return mapping expression
     */
    public KeyTypeMappingExpression getMappingExpression(String name) {
        return mappingExpressions.get(name);
    }

    /**
     * Adds a mapping expression.
     * 
     * @param mappingExpression
     *            mapping expression
     */
    public void addMappingExpression(KeyTypeMappingExpression mappingExpression) {
        mappingExpressions.put(mappingExpression.getName(), mappingExpression);
    }

    /**
     * Removes a mapping expression.
     * 
     * @param name
     *            key
     */
    public void removeMappingExpression(String name) {
        mappingExpressions.remove(name);
    }

    /**
     * Sets expressions.
     * 
     * @param initExpressions
     *            expressions
     */
    private void setMappingExpressionsInternal(Iterator<KeyTypeMappingExpression> initExpressions) {
        mappingExpressions.clear();
        while (initExpressions.hasNext()) {
            KeyTypeMappingExpression mapping = initExpressions.next();
            mappingExpressions.put(mapping.getName(), mapping);
        }
    }
}
