/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication.service;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.exception.AxisFaultUtil;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.util.AxisWSSUtil;
import dk.gov.oiosi.xml.XmlDocument;
import org.apache.axiom.om.OMDocument;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axiom.soap.SOAPHeaderBlock;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.receivers.AbstractInOutMessageReceiver;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.security.handler.WSHandlerConstants;

import java.security.cert.X509Certificate;
import java.util.List;

/**
 * AxisServiceReceiver.
 * <p/>
 * Receives message in Axis stack and passes on to Rasp service implementing :
 * 'IServiceContract'.
 */
public class AxisServiceReceiver extends AbstractInOutMessageReceiver {

    /**
     * Local log.
     */
    private static Log log = LogFactory.getLog(AxisServiceReceiver.class);

    private static final boolean AVOID_BODY_CLONE = true;

    public OMElement doOperation(MessageContext msgContext, @SuppressWarnings("unused") MessageContext newMsgContext) throws Exception {
        OMElement result = null;

        IServiceContract docService = (IServiceContract) getTheImplementationObject(msgContext);
        String messageId = msgContext.getMessageID();

        OMElement requestBody = msgContext.getEnvelope().getBody().getFirstElement();
        if (requestBody != null) {
            /*
                * dlk 2012-03-29
                *
                * Task 918
                *
                * Instead of coping potentially big document, we can reuse existing
                * loaded node from SOAPEnvelope and just wrap it in a new document.
                */
            if (AVOID_BODY_CLONE) {
                /*
                     * We cannot just use requestBody without attaching to new
                     * OMDocument, otherwise XPath would not be calculated in
                     * current implementation, as they start from the root of the
                     * document, which in case of getting it directly from
                     * SOAPEnvelope, would start from root tag /soap:Envelope.
                     *
                     * In future version of OIORASP it will be fixed by using
                     * relative XPath definitions in
                     * dk.gov.oiosi.xml.xpath.DocumentXPathResolver
                     * .getNodes(XmlDocument, String, PrefixedNamespace[])
                     */
                OMFactory factory = requestBody.getOMFactory();
                OMDocument document = factory.createOMDocument();
                document.addChild(requestBody);
                requestBody = document.getOMDocumentElement();
                /*
                     * For big documents it is very heavy operation - building the
                     * whole AXIOM. In future it should be removed.
                     */
                requestBody.build();
            } else {
                requestBody.build();
                try {
                    log.debug("Convert body");
                    requestBody = requestBody.cloneOMElement();
                } catch (Exception e) {
                    log.error("Error converting body: " + e.getMessage(), e);
                    throw new MainException("Internal Error : " + e, e);
                }
            }
        }

        // Prepare for Rasp service
        XmlDocument xmlDocument = new XmlDocument(requestBody);
        OiosiMessage oiosiMessage = new OiosiMessage(xmlDocument, messageId);
        oiosiMessage.setRequestAction(msgContext.getSoapAction());

        extractCustomHeaders(msgContext, oiosiMessage);
        extractSignatureInformation(msgContext, oiosiMessage);

        if (oiosiMessage.getMessageHeaders().containsKey(Constants.RASP_CUSTOM_HEADER_MESSAGE_IDENTIFIER)) {
            oiosiMessage.setMessageIdentifier((String) oiosiMessage.getMessageHeaders().get(Constants.RASP_CUSTOM_HEADER_MESSAGE_IDENTIFIER));
        }

        OiosiMessage response;
        try {
            // call service
            log.debug("Call Service class");
            response = docService.requestRespond(oiosiMessage);
        } catch (Exception e) {
            log.error("Service threw Exception: " + e.getMessage(), e);

            // call cancel on service to keep statistic in sync
            if (docService instanceof IServiceContractTransaction) {
                ((IServiceContractTransaction) docService).faultedOperation(messageId);
            }
            throw e;
        }

        if (response != null) {
            log.debug("- we have response");
            result = response.getMessageXml().getOMDocument();
        } else {
            log.debug("- response is null");
        }

        // no rm session call confirm
        if (docService instanceof IServiceContractTransaction) {
            ((IServiceContractTransaction) docService).confirmOperation(messageId);
        }
        return result;
    }

    /**
     * Extract custom headers.
     *
     * @param ctx message context
     * @param msg oiosi message
     */
    @SuppressWarnings("unchecked")
    private void extractCustomHeaders(MessageContext ctx, OiosiMessage msg) {
        log.debug("extractCustomHeaders");
        List<SOAPHeaderBlock> list = (List<SOAPHeaderBlock>) ctx.getProperty(Constants.RASP_CUSTOM_HEADER_LIST);
        if (list == null) {
            SOAPEnvelope envelope = ctx.getEnvelope();
            SOAPHeader header = envelope.getHeader();
            list = header.getHeaderBlocksWithNSURI(Constants.RASP_CUSTOM_HEADER_NS);
        }
        if (list != null) {
            for (SOAPHeaderBlock shb : list) {
                String key = shb.getLocalName();
                String value = shb.getText();

                // as as string
                msg.addMessageHeader(key, value);
            }
        }
        //
        String messageIdentifier = (String) ctx.getProperty(Constants.RASP_MESSAGE_IDENTIFIER);
        if (messageIdentifier != null) {
            msg.setMessageIdentifier(messageIdentifier);
        }

        Identifier senderIdentifier = ((Identifier) ctx.getProperty(Constants.RASP_SENDER_PARTY_IDENTIFIER));
        if (senderIdentifier != null) {
            msg.setSenderPartyIdentifier(senderIdentifier);
        }

        Identifier receiverIdentifier = ((Identifier) ctx.getProperty(Constants.RASP_RECEIVER_PARTY_IDENTIFIER));
        if (receiverIdentifier != null) {
            msg.setReceiverPartyIdentifier(receiverIdentifier);
        }
    }


    /**
     * Extract signatur information.
     *
     * @param ctx message context
     * @param msg oiosi message
     */
    private void extractSignatureInformation(MessageContext ctx, OiosiMessage msg) {
        log.debug("extractSignatureInformation");

        // If RASP_SIGNING_CERTIFICATE is found in SignatureProofHandler - set
        // on msg
        if (ctx.getProperty(Constants.RASP_SIGNING_CERTIFICATE) != null) {
            X509Certificate signerCertificate = (X509Certificate) ctx.getProperty(Constants.RASP_SIGNING_CERTIFICATE);
            msg.setSignerCertificate(signerCertificate);

            if (ctx.getProperty(Constants.RASP_SIGNATURE_PROOF) != null) {
                SignatureValidationProof signatureProof = (SignatureValidationProof) ctx
                        .getProperty(Constants.RASP_SIGNATURE_PROOF);
                msg.setSignatureProof(signatureProof);
            }

        } else {
            // try to find SignerCertificate - let user create signature proof.
            // No Ldap here
            Object o = ctx.getProperty(WSHandlerConstants.RECV_RESULTS);
            if (o != null) {
                X509Certificate signerCertificate = AxisWSSUtil.getCertificateFrom_RECV_RESULTS(o);
                if (signerCertificate != null) {
                    msg.setSignerCertificate(signerCertificate);
                }
            }
        }

    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.receivers.AbstractInOutSyncMessageReceiver#invokeBusinessLogic(org.apache.axis2.context.MessageContext,
     *      org.apache.axis2.context.MessageContext)
     */
    public void invokeBusinessLogic(MessageContext msgContext, MessageContext newmsgContext) throws AxisFault {
        log.debug("Invoking businessLogic - calling doOperation... : " + msgContext.getTo() + " : " + msgContext.getSoapAction());

        //
        SOAPFactory fac = getSOAPFactory(msgContext);
        try {
            OMElement result = doOperation(msgContext, newmsgContext);
            SOAPEnvelope envelope = fac.getDefaultEnvelope();
            getResult(msgContext, result, envelope);
            newmsgContext.setEnvelope(envelope);

            // move custom headers
            Object o = msgContext.getProperty(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_MANDATORY);
            if (o != null) {
                newmsgContext.getOptions().setProperty(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_LIST, o);
            }
            o = msgContext.getProperty(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_MESSAGE_IDENTIFIER);
            if (o != null) {
                newmsgContext.getOptions().setProperty(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_MESSAGE_IDENTIFIER, o);
            }
            if (msgContext.getProperty(WSHandlerConstants.RECV_RESULTS) != null
                    && newmsgContext.getProperty(WSHandlerConstants.RECV_RESULTS) == null) {
                newmsgContext.setProperty(WSHandlerConstants.RECV_RESULTS, msgContext.getProperty(WSHandlerConstants.RECV_RESULTS));
            }

        } catch (MainException e) {
            throw AxisFaultUtil.buildAxisFault(msgContext, e.getSoapFaultAction(), e.getSoapFaultCode(), e);
        } catch (Exception e) {
            throw AxisFaultUtil.buildAxisFault(msgContext, MainException.DEFAULT_SOAP_ACTION, "Receiver", e);
        }
    }

    /**
     * Assign result from service call to result envelope.
     *
     * @param msgContext message context
     * @param result     result from service call
     * @param envelope   result envelope
     */
    private void getResult(MessageContext msgContext, OMElement result, SOAPEnvelope envelope) {
        if (result != null) {
            AxisService service = msgContext.getAxisService();
            result.declareNamespace(service.getTargetNamespace(), service.getTargetNamespacePrefix());
            envelope.getBody().addChild(result);
        } else {
            log.debug("Body Still empty...");
        }
    }
}