/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.raspprofile.communication;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.addressing.IdentifierCvr;
import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.Utilities;
import dk.gov.oiosi.communication.*;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;

/**
 * This class encapsulates a Request object. It extends its functionality by adding Rasp custom headers to the message in the GetResponse
 * and BeginGetResponse methods as well as SOAP request and reply actions.
 */
public class RaspRequest implements IRaspRequest {

    private final Request encapsulatedRequest;

    /**
     * Constructs a new RaspRequest given a Request object
     *
     * @param request The Request object to be encapsulated
     */
    public RaspRequest(Request request) {
        encapsulatedRequest = request;
    }

    /**
     * Adds custom RASP headers to a given OiosiMessage
     *
     * @param oiosiMessage The message on which the headers should be added
     * @param documentId   The document Id used in the MessageIdentifier header
     * @throws MainException On error...
     */
    private void AddCustomHeaders(OiosiMessage oiosiMessage, String documentId) throws MainException {

        DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(oiosiMessage.getMessageXml());

        // Adds the obligatory RASP headers

        // Sets the SOAP request/response action based on document type configuration
        oiosiMessage.setRequestAction(documentTypeConfig.getEndpointType().getRequestAction());
        oiosiMessage.setReplyAction(documentTypeConfig.getEndpointType().getReplyAction());

        // Adds the obligatory RASP header <MessageIdentifier>, used for
        // removing duplicates
        Identifier receiverId = getReceiverID(oiosiMessage);
        Identifier senderId = getSenderID(oiosiMessage);
        oiosiMessage.addMessageHeader(Constants.RASP_MESSAGE_IDENTIFIER, documentId);
        oiosiMessage.setReceiverPartyIdentifier(receiverId);
        oiosiMessage.addMessageHeader(Constants.RASP_RECEIVER_PARTY_TYPE_HEADER, receiverId.getType().toLowerCase());
        oiosiMessage.setSenderPartyIdentifier(senderId);
        oiosiMessage.addMessageHeader(Constants.RASP_SENDER_PARTY_TYPE_HEADER, senderId.getType().toLowerCase());
    }

    private Identifier getReceiverID(OiosiMessage oiosiMessage) throws MainException {
        try {
            Identifier id = Utilities.getReceiverEndpointKey(oiosiMessage.getMessageXml());

            if (!id.isAllowedInPublic()) {
                OutVariable<String> cvrNumberString = new OutVariable<String>();
                if (getCredentials().getServerCertificate().TryGetCvrNumberString(cvrNumberString)) {
                    id = new IdentifierCvr(cvrNumberString.getVariable());
                } else {
                    throw new MainException("Receiver ID is not allowed to be added to header. It was not possible to retreive ID from certificate");
                }
            }

            return id;
        } catch (NullPointerException e) {
            throw new MainException("Receiver ID could not be retrieved from the message.", e);
        }
    }

    private Identifier getSenderID(OiosiMessage oiosiMessage) throws MainException {

        try {
            Identifier id = Utilities.getSenderEndpointKey(oiosiMessage.getMessageXml());

            if (!id.isAllowedInPublic()) {
                OutVariable<String> cvrNumberString = new OutVariable<String>();
                if (getCredentials().getClientCertificate().TryGetCvrNumberString(cvrNumberString)) {
                    id = new IdentifierCvr(cvrNumberString.getVariable());
                } else {
                    throw new MainException("Sender ID is not allowed to be added to header. It was not possible to retrieve ID from certificate");
                }
            }

            return id;
        } catch (NullPointerException e) {
            throw new MainException("Sender ID could not be retrieved from the message.", e);
        }
    }

    @Override
    public Credentials getCredentials() {
        return encapsulatedRequest.getCredentials();
    }

    @Override
    public SendPolicy getPolicy() {
        return encapsulatedRequest.getPolicy();
    }

    @Override
    public Uri getRequestUri() {
        return encapsulatedRequest.getRequestUri();
    }

    @Override
    public Response getResponse(OiosiMessage message, String documentId) throws MainException {
        AddCustomHeaders(message, documentId);
        return encapsulatedRequest.getResponse(message);
    }

    @Override
    public void setCredentials(Credentials cred) {
        encapsulatedRequest.setCredentials(cred);
    }

    @Override
    public void setPolicy(SendPolicy policy) {
        encapsulatedRequest.setPolicy(policy);
    }
}