/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security;

import dk.gov.oiosi.security.oces.OcesCertificateType;
import dk.gov.oiosi.security.revocation.CheckCertificateRevokedUnexpectedException;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationLookupFactory;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;
import dk.gov.oiosi.security.validation.CertificateValidator;

import java.security.cert.X509Certificate;
import java.util.Date;

/**
 * Methods for performing various certificate checks, such as revocation and
 * root certificate relations.
 * <p/>
 * TODO: There is a bug in dk.gov.oiosi.security.CertificateChecker
 * .checkCertificateRevocation(X509Certificate, RevocationResponse) - given
 * response is just returned without any modification
 *
 * @deprecated Deprecated from 1.3.0: These check are now present in dk.gov.oiosi.security.validation.CertificateValidator and other places in code.
 */
public class CertificateChecker {

    /**
     * Oces Root.
     */
    private X509Certificate defaultOCESrootCertificate;

    /**
     * Revocation Checker.
     */
    private IRevocationLookup lookup;

    /**
     * Instantiates CertificateChecker.
     *
     * @param defaultRootCertificate default OCES root certificate
     * @throws Exception On error...
     */
    public CertificateChecker(X509Certificate defaultRootCertificate) throws Exception {
        try {
            defaultOCESrootCertificate = defaultRootCertificate;

            // Initializes the component, that will do the actual ocsp lookup
            lookup = RevocationLookupFactory.createRevocationLookupClient();
        } catch (Exception e) {
            throw e;
        }
    }

    /**
     * Checks a certificate, with default rootcertificate.
     *
     * @param certificate certificate to check
     * @return The object that contains the result
     * @throws Exception
     */
    public CertificateCheckResult checkCertificate(X509Certificate certificate) throws Exception {
        CertificateCheckResult result = new CertificateCheckResult();
        RevocationResponse response = new RevocationResponse();

        try {
            // 1. that the certificate has the default rootcertificate as root
            checkCertificateChain(certificate, result);

            // 2. that the rootcertificate is not expired or not activated
            checkRootCertificateValidation(defaultOCESrootCertificate, result);
            checkRootCertificateActivated(defaultOCESrootCertificate, result);

            // 3. that the certificate is not expired or not activated
            if (checkCertificateValidation(certificate)) {
                result.setCertificateValid(true);
            }
            if (checkCertificateActivated(certificate)) {
                result.setCertificateActivated(true);
            }

            // 4. check if the certificate is revoked async
            response = checkCertificateRevocation(certificate, response);

            // 5. check certificatetype
            checkCertificateType(certificate, result);

            // 6. set the return values
            if (!response.isValid()) {
                result.setCertificateRevoked(true);
            }

            if (result.getCertificateActivated() && !result.getCertificateRevoked() && result.getCertificateValid()
                    && result.getRootCertificateActivated() && result.getRootCertificateAsRoot()
                    && result.getRootCertificateValid()) {
                result.setAllTestsPassed(true);
            }
        } catch (Exception e) {
            throw e;
        }
        return result;
    }

    /**
     * Checks a certificate, with a given rootcertificate.
     *
     * @param certificate     certificate to check
     * @param rootCertificate a given rootcertificate
     * @return The object that contains the result
     * @throws Exception
     */
    public CertificateCheckResult checkCertificate(X509Certificate certificate, X509Certificate rootCertificate) throws Exception {
        CertificateCheckResult result = new CertificateCheckResult();
        RevocationResponse response = new RevocationResponse();

        try {
            // 1. that the certificate has the default rootcertificate as root
            checkCertificateChain(certificate, result);

            // 2. that the rootcertificate is not expired or not activated
            checkRootCertificateValidation(rootCertificate, result);
            checkRootCertificateActivated(rootCertificate, result);

            // 3. that the certificate is not expired or not activated
            if (checkCertificateValidation(certificate)) {
                result.setCertificateValid(true);
            }
            if (checkCertificateActivated(certificate)) {
                result.setCertificateActivated(true);
            }

            // 4. check if the certificate is revoked async
            response = checkCertificateRevocation(certificate, response);

            // 5. set the return values
            if (!response.isValid()) {
                result.setCertificateRevoked(true);
            }

            if (result.getCertificateActivated() && !result.getCertificateRevoked() && result.getCertificateValid()
                    && result.getRootCertificateActivated() && result.getRootCertificateAsRoot()
                    && result.getRootCertificateValid()) {
                result.setAllTestsPassed(true);
            }
        } catch (Exception e) {
            throw e;
        }
        return result;
    }

    /**
     * Performs a certificate chain validation.
     *
     * @param certificate the certificate to validate
     * @param result      the object to store the result
     * @throws CheckCertificateChainUnexpectedException
     *          This exception is thrown, if an unexpected exception is
     *          thrown during the method
     */
    private void checkCertificateChain(X509Certificate certificate, CertificateCheckResult result)
            throws CheckCertificateChainUnexpectedException {
        try {
            CertificateValidator.validateCertificate(certificate);

            result.setRootCertificateAsRoot(true);
            result.setCertificateActivated(true);
            result.setCertificateValid(true);
            // } catch (CertificateFailedChainValidationException e) {
            // result.setRootCertificateAsRoot( false );
        } catch (CertificateNotActiveException e) {
            result.setCertificateActivated(false);
        } catch (CertificateExpiredException e) {
            result.setCertificateValid(false);
        } catch (Exception e) {
            throw new CheckCertificateChainUnexpectedException(e.getMessage());
        }
    }

    /**
     * Checks if the rootcertificate is activated.
     *
     * @param root   The rootcertificate to check
     * @param result The object to store the result of the check
     * @throws CheckRootCertificateActivatedUnexpectedException
     *          This exception is thrown, if an unexpected exception is
     *          thrown during the method
     */
    private void checkRootCertificateActivated(X509Certificate root, CertificateCheckResult result)
            throws CheckRootCertificateActivatedUnexpectedException {
        try {
            if (!root.getNotBefore().before(new Date())) {
                result.setRootCertificateActivated(true);
            }
        } catch (Exception e) {
            throw new CheckRootCertificateActivatedUnexpectedException(e);
        }
    }

    /**
     * Checks if the rootcertificate is valid.
     *
     * @param root   The rootcertificate to check
     * @param result The object to store the result of the check
     * @throws CheckRootCertificateValidUnexpectedException
     *          This exception is thrown, if an unexpected exception is
     *          thrown during the method
     */
    private void checkRootCertificateValidation(X509Certificate root, CertificateCheckResult result)
            throws CheckRootCertificateValidUnexpectedException {
        try {
            if (root.getNotAfter().after(new Date())) {
                result.setRootCertificateValid(true);
            }
        } catch (Exception e) {
            throw new CheckRootCertificateValidUnexpectedException(e);
        }
    }

    /**
     * Checks if the certificate is activated.
     *
     * @param certificate The certificate to check
     * @return boolean stating whether the certificate is activated (true) or not (false).
     * @throws CheckCertificateActivatedUnexpectedException
     *          This exception is thrown, if an unexpected exception is thrown during the method
     */
    public boolean checkCertificateActivated(X509Certificate certificate) throws CheckCertificateActivatedUnexpectedException {
        try {
            CertificateValidator.checkCertificateActivated(certificate);
        } catch (CertificateNotActiveException e) {
            return false;
        } catch (Exception e) {
            throw new CheckCertificateActivatedUnexpectedException(e.getMessage());
        }
        return true;
    }

    /**
     * Checks if the certificate is valid.
     *
     * @param certificate The certificate to check
     * @return boolean stating whether the certificate is valid (true) or not
     *         (false)
     * @throws CheckCertificateValidUnexpectedException
     *          This exception is thrown, if an unexpected exception is
     *          thrown during the method
     */
    public boolean checkCertificateValidation(X509Certificate certificate) throws CheckCertificateValidUnexpectedException {
        boolean isValid;
        try {
            CertificateValidator.checkCertificateExpired(certificate);
            isValid = true;
        } catch (CertificateExpiredException e) {
            isValid = false;
        } catch (Exception e) {
            throw new CheckCertificateValidUnexpectedException(e);
        }
        return isValid;
    }

    /**
     * Check if the certificate is revoced against ocsp server.
     *
     * @param certificate the certificate to check
     * @param response    OcspResponse object to store the result
     * @return OcspResponse object to store the result
     * @throws CheckCertificateRevokedUnexpectedException
     *          Thrown if an unexpected error occurred
     */
    private RevocationResponse checkCertificateRevocation(X509Certificate certificate, RevocationResponse response)
            throws CheckCertificateRevokedUnexpectedException {
        try {
            // Checks the certificate
            lookup.checkCertificate(certificate);
        } catch (Exception e) {
            throw new CheckCertificateRevokedUnexpectedException(e.getMessage(), e);
        }
        return response;
    }

    /**
     * Returns the type of OCES certificate, e.g. "company" or "employee" certificates.
     *
     * @param certificate The certificate to check
     * @return Returns the OCES certificate type
     */
    public static OcesCertificateType getCertificateType(X509Certificate certificate) throws CheckCertificateTypeUnexpectedException {
        OcesCertificateType certType = new OcesCertificateType(OcesCertificateType.NonOces);

        try {
            // no check included for LRA and Device
            String subjectString = certificate.getSubjectDN().toString();
            if (subjectString.indexOf("PID") != -1) {
                certType = new OcesCertificateType(OcesCertificateType.OcesPersonal);
            } else if (subjectString.indexOf("CVR") != -1) {
                if (subjectString.indexOf("RID") != -1) {
                    certType = new OcesCertificateType(OcesCertificateType.OcesEmployee);
                } else if (subjectString.indexOf("UID") != -1) {
                    certType = new OcesCertificateType(OcesCertificateType.OcesOrganisation);
                }
            }
        } catch (Exception e) {
            throw new CheckCertificateTypeUnexpectedException(e);
        }
        return certType;
    }

    /**
     * Sets the certificate type.
     *
     * @param certificate The certificate to check
     * @param result      The result object to store the result
     */
    private void checkCertificateType(X509Certificate certificate, CertificateCheckResult result) throws Exception {
        result.setCertificateType(getCertificateType(certificate));
    }
}