/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security;

import java.util.Date;

/**
 * Contains information about proof of signature validation. This is used to
 * verify that a message has been sent or received.
 */
public class SignatureValidationProof implements ISignatureValidationProof {

    /**
     * timestamp.
     */
    private Date timeStamp;

    /**
     * certificateSubject.
     */
    private String certificateSubject = "";

    /**
     * certificate was valid.
     */
    private boolean validCertificate;

    /**
     * signature was valid.
     */
    private boolean validSignature;

    /**
     * message was unchanged.
     */
    private boolean unchangedMessage;

    /**
     * message was encrypted.
     */
    private boolean encryptedMessage;

    /**
     * SignatureValidationProof has been completed - values must not change!.
     */
    private boolean completed;

    /**
     * public default constructor.
     */
    public SignatureValidationProof() {
    }

    /**
     * Constructor that creates a new proof that is completed now for the given
     * certificate subject.
     * 
     * @param subject certificate subject
     */
    public SignatureValidationProof(String subject) {
        timeStamp = new Date();
        this.certificateSubject = subject;
        this.validCertificate = true;
        this.validSignature = true;
        this.unchangedMessage = true;
        this.encryptedMessage = true;
        // Set that the signature validation proof is completed
        this.completed = true;
    }

    /**
     * Completes a proof of signature validation. Throws an exception if the
     * validation has been completed earlier.
     * 
     * @param subject certificate subject
     */
    public void completeValidation(String subject) throws SignatureValidationProofAllreadyCompletedException {
        if (completed)
            throw new SignatureValidationProofAllreadyCompletedException(subject);
        timeStamp = new Date();
        this.certificateSubject = subject;
        this.validCertificate = true;
        this.validSignature = true;
        this.unchangedMessage = true;
        this.encryptedMessage = true;
        // Set that the signature validation proof is completed
        this.completed = true;
    }

    /**
     * Validates if the signature validation proof has been set.
     */
    public void setCompleted() {
        completed = true;
    }

    /**
     * Is the signature validation proof already complete?
     * 
     * @return Completed
     */
    public boolean getCompleted() {
        return completed;
    }

    /**
     * Gets and sets the timestamp the signature validation proof was done.
     * 
     * @return TimeStamp
     */
    public Date getTimeStamp() {
        return timeStamp;
    }

    /**
     * Gets and sets the timestamp the signature validation proof was done.
     * 
     * @param value
     *            TimeStamp
     */
    public void setTimeStamp(Date value) throws SignatureValidationProofAllreadyCompletedException {
        if (completed)
            throw new SignatureValidationProofAllreadyCompletedException(certificateSubject);
        timeStamp = value;
    }

    /**
     * Gets and sets the certificate subject.
     * 
     * @return CertificateSubject
     */
    public String getCertificateSubject() {
        return certificateSubject;
    }

    /**
     * Gets and sets the certificate subject.
     * 
     * @param value
     *            CertificateSubject
     */
    public void setCertificateSubject(String value) throws SignatureValidationProofAllreadyCompletedException {
        if (completed)
            throw new SignatureValidationProofAllreadyCompletedException(certificateSubject);

        certificateSubject = value;
    }

    /**
     * Gets and sets the certificate validation.
     * 
     * @return ValidCertificate
     */
    public boolean getValidCertificate() {
        return validCertificate;
    }

    /**
     * Gets and sets the certificate validation.
     * 
     * @param value
     *            ValidCertificate
     */
    public void setValidCertificate(boolean value) throws SignatureValidationProofAllreadyCompletedException {
        if (completed)
            throw new SignatureValidationProofAllreadyCompletedException(certificateSubject);
        validCertificate = value;
    }

    /**
     * Gets and sets the signature valid flag.
     * 
     * @return ValidSignature
     */
    public boolean getValidSignature() {
        return validSignature;
    }

    /**
     * Gets and sets the signature valid flag.
     * 
     * @param value
     *            ValidSignature
     */
    public void setValidSignature(boolean value) throws SignatureValidationProofAllreadyCompletedException {
        if (completed)
            throw new SignatureValidationProofAllreadyCompletedException(certificateSubject);

        validSignature = value;
    }

    /**
     * Gets and sets the message unchanged flag.
     * 
     * @return UnchangedMessage
     */
    public boolean getUnchangedMessage() {
        return unchangedMessage;
    }

    /**
     * Gets and sets the message unchanged flag.
     * 
     * @param value
     *            UnchangedMessage
     */
    public void setUnchangedMessage(boolean value) throws SignatureValidationProofAllreadyCompletedException {
        if (completed)
            throw new SignatureValidationProofAllreadyCompletedException(certificateSubject);

        unchangedMessage = value;
    }

    /**
     * Gets and sets the encrypted message flag.
     * 
     * @return EncryptedMessage
     */
    public boolean getEncryptedMessage() {
        return encryptedMessage;
    }

    /**
     * Gets and sets the encrypted message flag.
     * 
     * @param value
     *            EncryptedMessage
     */
    public void setEncryptedMessage(boolean value) throws SignatureValidationProofAllreadyCompletedException {
        if (completed)
            throw new SignatureValidationProofAllreadyCompletedException(certificateSubject);

        encryptedMessage = value;
    }

    public String toString(){
    	return 	"Date: " + timeStamp +
    			"; Valid certificate: " + validCertificate +
    			"; Valid signature: " + validSignature +
    			"; Unchanged message: " + unchangedMessage +
    			"; Was encrypted: " + encryptedMessage +
    			"; Certificate subject: " + certificateSubject;
    }
    
}
