/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.ldap;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.ICache;
import dk.gov.oiosi.configuration.CacheFactory;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.lookup.CertificateLookupException;
import dk.gov.oiosi.security.lookup.ICertificateLookup;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;
import dk.gov.oiosi.security.validation.CertificateValidator;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.naming.CommunicationException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;
import java.io.ByteArrayInputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Collection;

/**
 * The LDAP certificate lookup uses the ldap protocol to query for a given certificate.
 * <p/>
 * It implements the ICertificate lookup to guarantee it has a specific interface for querying certificates.
 */
public class LdapCertificateLookup implements ICertificateLookup {

    private static final Log log = LogFactory.getLog(LdapCertificateLookup.class);

    private static CertificateFactory cf;

    private LdapSettings settings;

    /*
     * Certificate cache
     */
    private static ICache<CertificateSubject, X509Certificate> certCache = CacheFactory.getInstance().getCertificateCache();

    /**
     * The LdapCertificateLookup constructor takes an ldap settings object as a parameter. The setting object is used every time a lookup is
     * done in the GetCertificate function.
     *
     * @param s Setting object that contains the ldap settings to be used.
     */
    public LdapCertificateLookup(LdapSettings s) {
        settings = s;
    }

    /**
     * Default constructor. Attempts to read configuration settings from configuration file
     */
    public LdapCertificateLookup() {
        this(ConfigurationHandler.getInstance().getLdapSettings());
    }

    /**
     * The implementation of the interface ICertificateLookups function.
     * <p/>
     * It takes the subject string and queries the ldap server for a certificate that satisfies the condition in the subject string.
     *
     * @param subject The subject string of an OCES certificate.
     * @return The certificate that satisfies the conditions of the subject string.
     * @throws CertificateLookupException
     */
    public X509Certificate getCertificate(CertificateSubject subject) throws CertificateLookupException, CertificateNotActiveException, CertificateExpiredException {
        if (subject == null) {
            throw new ArgumentNullException("subject");
        }

        X509Certificate certificateToBeReturned;

        OutVariable<X509Certificate> value = new OutVariable<X509Certificate>();
        if (!certCache.tryGetValue(subject, value)) {
            // certificate not found in cache, so retrieve it online and put in cache.
            return getCertificateOnlineAndUpdateCache(subject);
        } else {
            // cache contain the certificate
            // now we must validate it is still valid
            certificateToBeReturned = value.getVariable();

            try {
                // if certificate is not valid, it is removed from cache
                /*
                     * We do not validate it for revocation, let's do only fast checks
                     */
                this.validateCertificateOnlyByDates(certificateToBeReturned);

            } catch (Exception e) {
                // if any error is thrown, the validation failed - certificate has expired.
                // remove it from cache, as it is not valid
                certCache.remove(subject);

                // get a new certificate online
                /*
                     * TODO DLK: What if LDAP returns still not active certificate?
                     * Or just the same certificate? In this case we reload it EACH
                     * time, which is quite possible on server side and can lead to
                     * overloading... We need to change this...
                     *
                     * Let's log it at least
                     */
                log.warn("LdapCertificateLookup: certificate with subject " + subject + " found in cache is not valid by dates: " + e.getMessage() + ", request certificate again");

                certificateToBeReturned = this.getCertificateOnlineAndUpdateCache(subject);
            }
        }

        return certificateToBeReturned;
    }


    private void validateCertificateOnlyByDates(X509Certificate certificateToBeReturned) throws CertificateNotActiveException, CertificateExpiredException {
        CertificateValidator.checkCertificateActivated(certificateToBeReturned);
        CertificateValidator.checkCertificateExpired(certificateToBeReturned);
    }

    /**
     * Retrive the certificate online, and if valid, put it in the cache
     *
     * @param subject CertificateSubject
     * @return X509Certificate
     * @throws CertificateLookupException    If the certificate cannot be found
     * @throws CertificateExpiredException   If the certificate has expired
     * @throws CertificateNotActiveException If the certificate is not yet active
     */
    protected X509Certificate getCertificateOnlineAndUpdateCache(CertificateSubject subject) throws CertificateLookupException, CertificateNotActiveException, CertificateExpiredException {
        /*
         * The method is marked as protected to hide it outside the package
         */
        X509Certificate certificateToBeReturned = getCertificateFromLDAP(subject);
        validateCertificateOnlyByDates(certificateToBeReturned);

        certCache.set(subject, certificateToBeReturned);

        return certificateToBeReturned;
    }


    public X509Certificate getCertificateFromLDAP(CertificateSubject subject) throws CertificateLookupException {
        if (subject == null) {
            throw new ArgumentNullException("subjectSerialNumber");
        }
        long start = 0;
        if (log.isDebugEnabled()) {
            log.debug("LdapCertificateLookup: search for " + subject);
            start = System.currentTimeMillis();
        }
        X509Certificate certificateToBeReturned = null;

        try {
            StringBuilder ldapQueryString = new StringBuilder();

            // protocol and server
            ldapQueryString.append("ldap://").append(settings.getHost()).append(":").append(settings.getPort());

            // search url
            ldapQueryString.append("/c=dk??sub?(").append(subject.getPrefixedSerialNumber()).append(")");
//            System.err.println(ldapQueryString);

            if (log.isTraceEnabled()) {
                log.trace("LdapCertificateLookup: LDAP query '" + ldapQueryString + "'");
            }

            NamingEnumeration<SearchResult> answer;
            boolean hasMore;
            try {
                // Create the initial context
                DirContext ctx = new InitialLdapContext();

                // Perform the search by using the URL
                answer = ctx.search(ldapQueryString.toString(), null);
                // Close the context when we're done
                ctx.close();

                hasMore = answer.hasMore();
            } catch (CommunicationException e) {
                log.error("LdapCertificateLookup: Could not perform search: " + e.getMessage(), e);
                throw new ConnectingToLdapServerFailedException(settings, e);
            } catch (NamingException e) {
                log.error("LdapCertificateLookup: Could not create Ldap Search Context : " + e.getMessage(), e);
                throw new LdapCertificateLookupInitializationFailedException("Could not create Ldap Search Context for: " + ldapQueryString, e);
            }

            if (!hasMore) {
                throw new LdapCertificateNotFoundException(subject);
            }

            byte[] certificateBytes;
            try {
                SearchResult sr = answer.next();
                Attributes attrs = sr.getAttributes();
                Attribute attribute = attrs.get("userCertificate;binary");

                if (attribute == null) {
                    throw new ConvertingSearchResultToCertificateFailedException("Could not get certificate attribute from Ldap Search: userCertificate;binary");
                }

                certificateBytes = (byte[]) attribute.get();
            } catch (NamingException e) {
                log.error("LdapCertificateLookup: Could not get certificate attribute from Ldap Search: " + e.getMessage(), e);
                throw new ConvertingSearchResultToCertificateFailedException("Could not get certificate attribute from Ldap Search", e);
            }

            try {
                Collection<?> c = getCf().generateCertificates(new ByteArrayInputStream(certificateBytes));
                certificateToBeReturned = (X509Certificate) c.iterator().next();
            } catch (CertificateException e) {
                log.error("LdapCertificateLookup: Could not create certificate form ldap result: " + e.getMessage(), e);
                throw new ConvertingSearchResultToCertificateFailedException("Could not create certificate form ldap result", e);
            }

            if (certificateToBeReturned == null) {
                log.info("LdapCertificateLookup: Certificate not found with LDAP query:" + subject.getSerialNumberValue());
                throw new LdapCertificateNotFoundException(subject);
            }

            try {
                hasMore = answer.hasMore();
            } catch (NamingException e) {
                log.error("LDAP lookup failed: " + e.getMessage(), e);
                throw new LdapCertificateLookupInitializationFailedException("LDAP lookup failed: " + e.getMessage(), e);
            }
            if (hasMore) {
                throw new LdapMultipleCertificatesFoundException(subject);
            }
        } finally {
            if (log.isDebugEnabled()) {
                StringBuilder sb = new StringBuilder();
                sb.append("LdapCertificateLookup: done query in ").append((System.currentTimeMillis() - start));
                sb.append("ms, subject '").append(subject).append("' ");
                sb.append((certificateToBeReturned != null ? "found" : "not found"));
                log.debug(sb.toString());
            }
        }
        return certificateToBeReturned;
    }

    CertificateFactory getCf() throws CertificateException {
        if (cf == null) {
            cf = CertificateFactory.getInstance("X.509");
        }
        return cf;
    }
}