/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation.crl;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.IThreadSafeCache;
import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import dk.gov.oiosi.configuration.CacheFactory;
import dk.gov.oiosi.security.revocation.CheckCertificateRevokedUnexpectedException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * Class used for storing CRLs retrieved from URL's in X509 certificates.
 */
class CrlCache {

	public static final int MINIMAL_CACHE_UPDATE_INTERVAL_MINUTES = 5;
	public static final int DEFAULT_ON_FAILURE_RETRY_INTERVAL_MINUTES = 5;

	private static final Log log = LogFactory.getLog(CrlCache.class);

	private CertificateFactory cf;

	/*
	 * dlk, 2012-04-09
	 * 
	 * Replaced IThreadSafeCache<URL, CrlInstance> with IThreadSafeCache<String,
	 * CrlInstance>
	 * 
	 * See CrlCacheDesignIssueAnalysis for explanation.
	 */
	private IThreadSafeCache<String, CrlInstance> cache = CacheFactory.getInstance().getCrlLookupCache();

	private static final Map<String, Object> SYNC_MAP = new HashMap<String, Object>();

	private int onFailureRetryIntervalMinutes = DEFAULT_ON_FAILURE_RETRY_INTERVAL_MINUTES;

	public CrlCache() throws CertificateException {
		cf = CertificateFactory.getInstance("X.509");
		if (cache.getConfiguration() != null) {
			String configValue = cache.getConfiguration().get(CrlLookup.CONFIG_PARAM_ON_FAILURE_RETRY_INTERVAL_MINUTES);

			if (configValue != null) {
				configValue = configValue.trim();
				try {
					int value = Integer.parseInt(configValue);
					if (value > 0) {
						onFailureRetryIntervalMinutes = value;
					} else {
						log.error("Invalid value for config field " + CrlLookup.CONFIG_PARAM_ON_FAILURE_RETRY_INTERVAL_MINUTES + ", found = '" + configValue + "', which was parsed as " + value + ". Value should be more than 0. Use default " + DEFAULT_ON_FAILURE_RETRY_INTERVAL_MINUTES);
					}
				} catch (Exception e) {
					log.error("Failed to parse config field " + CrlLookup.CONFIG_PARAM_ON_FAILURE_RETRY_INTERVAL_MINUTES + ", found = '" + configValue + "', use default " + DEFAULT_ON_FAILURE_RETRY_INTERVAL_MINUTES);
				}
			}
		}
		if (log.isInfoEnabled()) {
			log.info("CrlCache is started with " + this.cache + ", onFailureRetryIntervalMinutes is " + this.onFailureRetryIntervalMinutes);
		}
	}

	/**
	 * Retrieves the CRL located at key.
	 * <p/>
	 * Thread safe method.
	 * 
	 * @param url Get the CRL located at the URL "key"
	 * @return An instance of a CRL
	 */
	public CrlInstance getCRL(URL url) {
		CrlInstance instance;
		String key = url.toExternalForm();
		OutVariable<CrlInstance> out = new OutVariable<CrlInstance>();
		if (this.cache.tryGetValue(key, out)) {
			// instance retrieved from cache
			instance = out.getVariable();
		} else {
			/*
			 * Test carefully to exclude deadlocks in such double
			 * synchronization - cache implementation is usually synchronized
			 */
			Object syncObject = getSyncObject(key);
			synchronized (syncObject) {
				/*
				 * If some other thread waited by this synchronization block
				 * while first thread reloaded data, it should check whether
				 * instance is already created.
				 */
				if (this.cache.tryGetValue(key, out)) {
					return out.getVariable();
				}
				/*
				 * There is no required value in cache. First time it is asked,
				 * cache reloading is done synchronously. If we already have
				 * some data in cache and they are expired, cache reloading
				 * should be done asynchronously.
				 */
				instance = new CrlInstance(cf, url);
				this.cache.add(key, instance);
				try {
					instance.updateData();
					/*
					 * Cache was successfully updated, schedule next update
					 */
					scheduleCacheUpdate(url, instance.getNextUpdate());
				} catch (CheckCertificateRevokedUnexpectedException e) {
					log.error("Failed to initialize CRL cache for URL " + key + ", schedule new attempts with interval " + this.onFailureRetryIntervalMinutes + " minutes", e);
					scheduleCacheUpdate(url, null);
					return instance;
				}
			}
		}

		return instance;
	}

	/**
	 * Requests ScheduledExecutor to do initialization of update of CRL cache
	 * 
	 * @param crlUrl URL to CRL server, should not be null
	 * @param nextRun if set to null, initialization is supposed to be done,
	 *        otherwise - update of cache
	 * @since OIORASP 1.3.0
	 */
	protected void scheduleCacheUpdate(URL crlUrl, Date nextRun) {
		Calendar c = Calendar.getInstance();
		/*
		 * It is possible, that current computer, on which OIORASP is used, has
		 * invalid time (let's say 2 days before actual time).
		 * 
		 * In this case scheduled job will be executed immediately after previous
		 * execution finished.
		 * 
		 * Add protection for this case - if next update is before current time,
		 * do it at least in 1 minute from now.
		 * 
		 * Log warning message in this case.
		 */
		if (nextRun != null && nextRun.before(c.getTime())) {
			log.warn("CrlCache: downloaded CRL list requires update before current time (" + nextRun + "). Check your time settings to be sure that you use correct time. Next update date is changed to be not earlier than in 5 minutes from now");
			c.add(Calendar.MINUTE, MINIMAL_CACHE_UPDATE_INTERVAL_MINUTES);
			nextRun = c.getTime();
		}
		CrlCacheUpdateJob initJob = new CrlCacheUpdateJob(crlUrl, nextRun == null ? "init" : "update");
		if (nextRun == null) {
			ScheduledExecutor.getInstance().scheduleJob(initJob, onFailureRetryIntervalMinutes, TimeUnit.MINUTES);
		} else {
			ScheduledExecutor.getInstance().scheduleJob(initJob, nextRun, onFailureRetryIntervalMinutes, TimeUnit.MINUTES);
		}
	}

	/**
	 * Returns (or creates and returns) synchronization object. Should be used
	 * to synchronize modification of cache for given URL.
	 * 
	 * @param syncKey url to return synchronization object.
	 * @return synchronization object for given URL
	 * @since OIORASP 1.3.0
	 */
	/*
	 * In order to restrict multiple thread access to cache for the same URL,
	 * but allow for different URL, let's keep a special map for synchronization
	 * objects for each requested URL, because URLs can be equal by equals()
	 * method but different by link. URL.toString() can be used only if we have
	 * interned strings.
	 */
	private Object getSyncObject(String syncKey) {
		synchronized (SYNC_MAP) {
			Object syncObject = SYNC_MAP.get(syncKey);
			if (syncObject == null) {
				syncObject = new Object();
				SYNC_MAP.put(syncKey, syncObject);
			}
			return syncObject;
		}
	}
}