/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation.crl;

import dk.gov.oiosi.security.oces.CertificateUtil;
import dk.gov.oiosi.security.revocation.CheckCertificateRevokedUnexpectedException;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Class for checking certificate revocation status against a CRL (Certificate Revocation List).
 * <p/>
 * Makes use of a cache for storing CRLs
 * <p/>
 * Since 1.3.0 is extended with support of OCES2 certificates, which can have intermediate certificates between root and last certificate.
 * <p/>
 * Root certificate should be present in the trust store, but intermediate certificates cannot be put there, so they should be validated
 * additionally.
 */
public class CrlLookup implements IRevocationLookup {

    private static final Log log = LogFactory.getLog(CrlLookup.class);

    public static final String CONFIG_PARAM_ON_FAILURE_RETRY_INTERVAL_MINUTES = "onFailureRetryIntervalMinutes";

    final static Date ZERO_DATE = new Date(0);

    static CertificateUtil _certificateUtil;

    /**
     * Cache used for storing CRLs
     */
    /*
      * SimpleCrlLookup and CrlLookup are not supposed to be used in the same
      * environment.
      */
    protected static CrlCache cache;

    static {
        try {
            cache = new CrlCache();
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    /**
     * Checks a certificate status in a CRL.
     *
     * @param certificate The certificate to check
     * @return The RevocationResponse object that contains the result
     * @throws RevocationException
     */
    public RevocationResponse checkCertificate(X509Certificate certificate) throws RevocationException {
        /*
        * Assumptions: - Certificate has an CRL Distribution Points extension
        * value. - An HTTP distribution point is present.
        */

        // Fail fast!
        if (certificate == null) {
            throw new RevocationException("Fault happened at CrlLookup.checkCertificate: null certificate was passed");
        }

        RevocationResponse response = new RevocationResponse();
        response.setIsValid(true);

        /*
         * First we need to create a certificate chain/list. But the root
         * certificate should not be part of this chain. The question weather
         * the root certificates is trusted, is validated in OioraspPolicyBasedResultsValidator
         */
        List<X509Certificate> certificateList = new ArrayList<X509Certificate>();
        certificateList.add(certificate);

        X509Certificate currentCertificate = certificate;
        X509Certificate issuerCertificate;
        CertificateUtil certificateUtil = getCertificateUtil();

        if (log.isDebugEnabled()) {
            log.debug("CrlLookup.checkCertificate: try to get list of issuer certificates for certificate " + currentCertificate.getSubjectX500Principal());
        }
        do {
            // Get the issuer certificate, until it can not be retrieved or is self signed (e.g. root certificate).
            issuerCertificate = certificateUtil.downloadIssuerCert(currentCertificate);
            if (issuerCertificate != null) {
                if (log.isDebugEnabled()) {
                    log.debug("CrlLookup.checkCertificate: found issuers certificate " + issuerCertificate.getSubjectX500Principal() + " for certificate " + currentCertificate.getSubjectX500Principal() + ", continue cycle");
                }
                certificateList.add(issuerCertificate);

                /*
                    Break if found issuerCertificate (parent) is root certificate.
                    Case:
                      Chain: A -> B -> C (where A is root)
                      Having currentCertificate=B, issuerCertificate is A
                      If A's name is equal to A's issuers name, then break.
                 */
                if (issuerCertificate.getSubjectX500Principal().getName().equals(issuerCertificate.getIssuerDN().getName())) {
                    break;
                }

                currentCertificate = issuerCertificate;
            } else {
                if (log.isDebugEnabled()) {
                    log.debug("CrlLookup.checkCertificate: no issuer certificate is found for certificate " + currentCertificate.getSubjectX500Principal());
                }
            }
        } while (issuerCertificate != null);

        if (log.isDebugEnabled()) {
            log.debug("CrlLookup.checkCertificate: " + certificateList.size() + " certificate(s) should be validated to check certificate " + certificate.getSubjectX500Principal());
        }

        // We know have a list of certificate that should be validated
        int index = 0;
        List<URL> urlList;
        while (index < certificateList.size() && response.isValid()) {
            currentCertificate = certificateList.get(index);

            // Retrieve URL distribution points
            urlList = certificateUtil.getCrlURLs(currentCertificate);
            // The list should only contain one element so we are only interested in the first CRL list
            if (urlList.size() > 0) {
                URL url = urlList.get(0);
                CrlInstance crl = cache.getCRL(url);
                try {
                    /*
                    * So the certificate is not revoked. Remember, that the
                    * issuer certificate could be revoked. So the next update
                    * must be the earliest of the all
                    */
                    if (crl.isRevoked(certificate)) {
                        response.setIsValid(false);
                    } else {
                        response.setIsValid(true);

                        if (response.getNextUpdate().compareTo(ZERO_DATE) == 0) {
                            // No next update exist
                            response.setNextUpdate(crl.getNextUpdate());
                        } else if (response.getNextUpdate().before(crl.getNextUpdate())) {
                            /*
                            * No new update. The already registered
                            * 'NextUpdate' is before the crl.getNextUpdate
                            */
                        } else {
                            /*
                            * New update time. The already registered
                            * 'NextUpdate', is after (further in the future)
                            * then crl.getNextUpdate so we use the
                            * crl.getNextUpdate as next update
                            */
                            response.setNextUpdate(crl.getNextUpdate());
                        }
                    }
                } catch (CheckCertificateRevokedUnexpectedException e) {
                    response.setIsValid(false);
                }
            } else {
                // https://bugs.softwareborsen.dk/show_bug.cgi?id=1202: When no CRL data is available, then trust this certificate!
                response.setIsValid(true);
            }

            // Increase the index, to check the next certificate
            index++;
        }

        return response;
    }

    CertificateUtil getCertificateUtil() {
        if (_certificateUtil == null) {
            _certificateUtil = new CertificateUtil();
        }
        return _certificateUtil;
    }

    /**
     * Enable test code to replace CertificateUtil with other code
     *
     * @param _certificateUtil CertificateUtil to use.
     */
    void set_certificateUtil(CertificateUtil _certificateUtil) {
        CrlLookup._certificateUtil = _certificateUtil;
    }

    /**
     * Gets a list of URLs from the specified certificate.
     *
     * @param cert The certificate to find the URLs in.
     * @return A list of CRL URLs in the certificate
     */
    /*
      * dlk, 2012-03-19
      *
      * Increase visibility of getURLs method to make it testable
      */
    protected List<URL> getURLs(X509Certificate cert) {
        CertificateUtil util = getCertificateUtil();
        return util.getCrlURLs(cert);
    }

    protected static CrlCache getCache() {
        return cache;
    }

    /**
     * @return Always returns CRL
     * @since OIORASP 1.3.0
     */
    @Override
    public RevocationSourceType getRevocationSourceType() {
        return RevocationSourceType.CRL;
    }
}