/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.uddi;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;

import java.net.URISyntaxException;
import java.util.List;

/**
 * UDDI lookup client that implements cross registry lookup.
 */
public class RegistryLookupClient implements IUddiLookupClient {

    private LookupRegistryFallbackConfig _configuration;

    /**
     * Default constructor.
     * It will read the configuration from RaspConfiguration.xml
     *
     * @throws URISyntaxException On error...
     */
    public RegistryLookupClient() throws URISyntaxException {
        ConfigurationHandler handler = ConfigurationHandler.getInstance();
        UddiConfig uddiConfig = handler.getUddiConfig();
        _configuration = uddiConfig.getLookupRegistryFallbackConfig();
    }

    /**
     * Constructor that takes the configuration as parameter.
     *
     * @param configuration The configuration of the lookup registry
     */
    public RegistryLookupClient(LookupRegistryFallbackConfig configuration) {
        _configuration = configuration;
    }

    /**
     * Implements the lookup method of the IUddiLookupClient interface.
     */
    @Override
    public List<UddiLookupResponse> lookup(LookupParameters parameters) throws MainException {
        List<UddiLookupResponse> response = null;
        //Uses different parameters for the lookup mechanism below.

        for (Registry registry : _configuration.getRegistries()) {
            IUddiLookupClient uddiLookupClient = new UddiFallbackClient(registry.getEndpoints());
            response = uddiLookupClient.lookup(parameters);
            // Continue only as long as no result was found in the current registry
            if (response != null && response.size() != 0) {
                break;
            }
        }
        return response;
    }

    @Override
    public List<ProcessDefinition> GetProcessDefinitions(List<UddiId> processDefinitionIds) throws MainException {
        List<ProcessDefinition> processDefinitions = null;
        for (Registry registry : _configuration.getRegistries()) {
            IUddiLookupClient uddiLookupClient = new UddiFallbackClient(registry.getEndpoints());
            processDefinitions = uddiLookupClient.GetProcessDefinitions(processDefinitionIds);
            // Continue until some was found
            if (processDefinitions != null && processDefinitions.size() != 0) {
                break;
            }
        }

        return processDefinitions;
    }
}