/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.uddi;

import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.exception.MainException;

import java.util.*;

/**
 * An UDDI lookup client that implements fallbacks.
 */
public class UddiFallbackClient implements IUddiLookupClient {

    private Collection<Uri> _fallbackList;
    private static Map<Uri, IUddiLookupClient> cache_uddiLookupClient = new HashMap<Uri, IUddiLookupClient>();

    /**
     * Constructor that takes the fallback list as parameter.
     *
     * @param fallbackList The fallback list
     */
    public UddiFallbackClient(Collection<Uri> fallbackList) {
        _fallbackList = fallbackList;
    }

    /**
     * Implementation of the lookup method from the IUddiLookupClient interface
     *
     * @param parameters The parameters used to make a lookup
     * @return A list of responses from the UDDI
     */
    @Override
    public List<UddiLookupResponse> lookup(LookupParameters parameters) throws MainException {
        List<UddiLookupResponse> result;

        MainException exception = null;
        for (Uri uri : _fallbackList) {
            try {
                IUddiLookupClient client = getCachedUddiLookupClient(uri);
                result = client.lookup(parameters);
                return result;
            } catch (MainException e) {
                exception = e;
            }
        }
        // The fallback list was empty, so we return an empty list as result.
        if (exception == null) {
            return new ArrayList<UddiLookupResponse>();
        }
        // We never got a valid result, so the last known exception is thrown
        throw exception;
    }

    @Override
    public List<ProcessDefinition> GetProcessDefinitions(List<UddiId> processDefinitionIds) throws MainException {
        List<ProcessDefinition> result;

        MainException exception = null;
        for (Uri uri : _fallbackList) {
            try {
                IUddiLookupClient client = getCachedUddiLookupClient(uri);
                result = client.GetProcessDefinitions(processDefinitionIds);
                return result;
            } catch (MainException e) {
                exception = e;
            }
        }
        //The fallback list was empty we return an empty list as result.
        if (exception == null) {
            return new ArrayList<ProcessDefinition>();
        }
        // We never got a valid result, so the last known exception is thrown
        throw exception;
    }

    IUddiLookupClient getCachedUddiLookupClient(Uri uri) throws MainException {
        if (!cache_uddiLookupClient.containsKey(uri)) {
            cache_uddiLookupClient.put(uri, UddiLookupClientFactory.createUddiLookupClient(uri));
        }
        return cache_uddiLookupClient.get(uri);
    }

    /**
     * Used for unit testing
     */
    int getUddiLookupClientCacheSize(){
        return cache_uddiLookupClient.size();
    }
}