package dk.gov.oiosi.uddi;

import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.itst.uddi.client.types.core.KeyedReference;
import dk.itst.uddi.client.types.core.TModel;

public class UddiTModel {

    private final static String businessProcessIdentifierId = "uddi:9111dd24-6734-407f-b949-d601ab427520";
    private final static String businessProcessIdentifierTypeId = "uddi:e03ae1dd-634f-4c6b-abd2-814c165ab5bf";
    private final static String businessProcessRoleIdentifierId = "uddi:4b2e5d7e-8e5d-4c03-92ca-3597b7f52444";
    private final static String businessProcessRoleIdentifierTypeId = "uddi:8dd0fa3e-be33-47f9-847b-8d974952a8dc";
    private final static String businessProcessDefinitionReferenceId = "uddi:d474ac8c-ec5d-4679-90b0-a227a517d745";
    private final static String registrationConformanceClaimId = "uddi:80496ef5-4d24-4788-a3f8-12fb54a75106";
    private final static String registrationConformanceClaimKeyValue = "http://oio.dk/profiles/OIOSI/1.0/UDDI/registrationModel/1.1/";

    private TModel tModel;
    private UddiCategoryBag categoryBag;

    public UddiTModel(TModel tModel) {
        if (tModel == null) {
            throw new ArgumentNullException("tModel");
        }

        this.tModel = tModel;
        this.categoryBag = new UddiCategoryBag(tModel.getCategoryBag());
    }

    public UddiId getUddiId() {
        String key = tModel.getTModelKey();
        return IdentifierUtility.getUddiIDFromString(key);
    }

    public TModel getTModel() {
        return tModel;
    }

    public String getName() {
        if (tModel.getName() == null) {
            return "";
        }
        if (tModel.getName().getStringValue() == null) {
            return "";
        }
        return tModel.getName().getStringValue();
    }

    public String getDescription() {
        if (tModel.getDescriptionArray() == null) {
            return "";
        }
        if (tModel.getDescriptionArray().length == 0) {
            return "";
        }
        if (tModel.getDescriptionArray()[0].getStringValue() == null) {
            return "";
        }
        return tModel.getDescriptionArray()[0].getStringValue();
    }

    public String GetProfileId() {
        for (KeyedReference keyedRef : tModel.getIdentifierBag().getKeyedReferenceArray()) {
            if (!businessProcessIdentifierId.equalsIgnoreCase(keyedRef.getTModelKey())) {
                continue;
            }
            if (keyedRef.getKeyValue() == null) {
                return "";
            }
            return keyedRef.getKeyValue();
        }
        return "";
    }

    public String GetProfileTypeId() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!categoryBag.tryGetKeyedReference(businessProcessIdentifierTypeId, out)) {
            return "";
        }
        KeyedReference keyedRef = out.getVariable();
        if (keyedRef.getKeyValue() == null) {
            return "";
        }
        return keyedRef.getKeyValue();
    }

    public String getProfileRoleId() {
        for (KeyedReference keyedRef : tModel.getIdentifierBag().getKeyedReferenceArray()) {
            if (!businessProcessRoleIdentifierId.equalsIgnoreCase(keyedRef.getTModelKey())) {
                continue;
            }
            if (keyedRef.getKeyValue() == null) {
                return "";
            }
            return keyedRef.getKeyValue();
        }
        return "";
    }

    public String getProfileRoleTypeId() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!categoryBag.tryGetKeyedReference(businessProcessRoleIdentifierTypeId, out)) {
            return "";
        }
        KeyedReference keyedRef = out.getVariable();
        if (keyedRef.getKeyValue() == null) {
            return "";
        }
        return keyedRef.getKeyValue();
    }

    public String GetProcessDefinitionReferenceId() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!categoryBag.tryGetKeyedReference(businessProcessDefinitionReferenceId, out)) {
            return "";
        }
        KeyedReference keyedRef = out.getVariable();
        if (keyedRef.getKeyValue().isEmpty()) {
            return "";
        }
        return keyedRef.getKeyValue();
    }

    public String GetRegistrationConformanceClaim() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!categoryBag.tryGetKeyedReference(registrationConformanceClaimId, out)) {
            return "";
        }
        KeyedReference keyedRef = out.getVariable();
        if (keyedRef.getKeyValue().isEmpty()) {
            return "";
        }
        return keyedRef.getKeyValue();
    }

    public boolean IsProfileRole() {
        if (tModel.getCategoryBag() == null) {
            return false;
        }
        if (tModel.getCategoryBag().getKeyedReferenceArray().length < 1) {
            return false;
        }

        // Check registration conformance
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!categoryBag.tryGetKeyedReference(registrationConformanceClaimId, out)) {
            return false;
        }
        KeyedReference confClaimKeyref = out.getVariable();
        if (!confClaimKeyref.getKeyValue().equals(registrationConformanceClaimKeyValue)) {
            return false;
        }

        // Check that the businessProcessDefinitionReference category exists:
        OutVariable<KeyedReference> procKeyrefOut = new OutVariable<KeyedReference>();
        return categoryBag.tryGetKeyedReference(businessProcessDefinitionReferenceId, procKeyrefOut);
    }

    @Override
    public String toString() {
        return "UddiTModel{" +
                "tModel=" + tModel +
                ", categoryBag=" + categoryBag +
                '}';
    }
}