/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.schematron;

import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.schematron.sax.SchematronValidatorSAXHandlerConfig;
import org.apache.commons.configuration.ConfigurationUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Configuration to the schematron validation.
 */
public class SchematronValidationConfig {

	private static final Log log = LogFactory.getLog(SchematronValidationConfig.class);

    /**
     * Path to schematron document.
     */
    private String schematronDocumentPath = "";

    /**
     * XPath to find error
     */
    private String errorXPath = "";

    /**
     * XPath to find error message.
     */
    private String errorMessageXPath = "";

    /**
     * Schematron document.
     */
    private XmlDocument schematronDocument;

    /**
     * Configuration for SchematronValidatorSAXUtil
     * @since OIORASP 1.3.0
     */
	private SchematronValidatorSAXHandlerConfig saxHandlerConfig;

    /**
     * Default constructor, should not be used.
     */
    public SchematronValidationConfig() {
    }

    /**
     * Constructor that takes the path to the schematron document, the error
     * XPath and the error message XPath.
     *
     * @param schematronDocumentPath The schematron document path.
     * @param errorXPath             The error XPath
     * @param errorMessageXPath      The error message XPath
     */
    public SchematronValidationConfig(String schematronDocumentPath, String errorXPath, String errorMessageXPath) {
        if (schematronDocumentPath == null) {
            throw new ArgumentNullException("schematronDocumentPath");
        }
        if (errorXPath == null) {
            throw new ArgumentNullException("errorXPath");
        }
        if (errorMessageXPath == null) {
            throw new ArgumentNullException("errorMessageXPath");
        }
        this.schematronDocumentPath = schematronDocumentPath;
        this.errorXPath = errorXPath;
        this.errorMessageXPath = errorMessageXPath;
        rebuildConfig();
    }

    /**
     * Gets and sets the schematron document path.
     *
     * @return path to schematron document
     */
    public String getSchematronDocumentPath() {
        return schematronDocumentPath;
    }

    /**
     * Gets and sets the schematron document path.
     *
     * @param value to schematron document
     */
    public void setSchematronDocumentPath(String value) {
        if (value == null || value.isEmpty()) {
            throw new ArgumentNullException("SchematronDocumentPath.value is null or empty...");
        }

        /*
         * TODO DLK: Try to find another way to allow unit tests to run. It is
		 * meaningless for PROD environment...
         */
        value = modifyValueForUnitTests(value);

		this.schematronDocumentPath = value;

		/*
		 * dlk, 2012-03-30
		 *
		 * This logic, introduced in 2012, breaks real server systems - it
		 * searches only at file system, but on servers resources are located in
		 * classpath, e.g. dk.gov.oiosi.examples.service project
		 *
		 * Commons-configuration should be used instead, the same way as class
		 * XmlDocument uses it when it loads schema or schematron
		 */
//        File file = new File(value);
//        if (!file.exists()) {
//            // In case of unit tests, try one level out.
//            file = new File("dk.gov.oiosi.library/" + value);
//            if (!file.exists()) {
//                throw new IllegalArgumentException("File path [" + value + "] for SchematronDocumentPath from RaspConfiguration file does not exist: " + file.getAbsolutePath());
//            }
//        }
//        schematronDocumentPath = file.toString();
    }

	public static String modifyValueForUnitTests(String value) {
		boolean found = ConfigurationUtils.locate(value) != null;
		if (!found) {
			// In case of unit tests, try one level out.
			if (ConfigurationUtils.locate("dk.gov.oiosi.library/" + value) != null) {
				value = "dk.gov.oiosi.library/" + value;
			}
		}
		return value;
	}

    /**
     * Gets and sets the XPath expression that gets the error results from the
     * schematron result.
     *
     * @return XPath to error
     */
    public String getErrorXPath() {
        return errorXPath;
    }

    /**
     * Gets and sets the XPath expression that gets the error results from the
     * schematron result.
     *
     * @param value to error
     */
    public void setErrorXPath(String value) {
        if (value == null) {
            throw new ArgumentNullException("ErrorXPath.value");
        }

        errorXPath = value;
        /*
         * Rebuild sax config, which depends on this value
         */
        this.rebuildConfig();
    }

    /**
     * Gets and sets the XPath expression that gets the error messages from the
     * schematron result.
     *
     * @return XPath to error message
     */
    public String getErrorMessageXPath() {
        return errorMessageXPath;
    }

    /**
     * Gets and sets the XPath expression that gets the error messages from the
     * schematron result.
     *
     * @param value to error message
     */
    public void setErrorMessageXPath(String value) {
        if (value == null) {
            throw new ArgumentNullException("ErrorMessageXPath.value");
        }

        errorMessageXPath = value;
        /*
         * Rebuild sax config, which depends on this value
         */
        this.rebuildConfig();
    }

    /**
     * Gets the schematron document. The first time this method is called then
     * the XML-document will be loaded from disc into memory.
     *
     * @return XML-document holding stylesheet
     * @throws SchematronErrorException On error...
     */
    public XmlDocument getSchematronDocument() throws SchematronErrorException {
        synchronized (this) {
            if (schematronDocument == null) {
                loadSchematronDocument();
            }
        }
        return schematronDocument;
    }

    /**
     * Load the  schematron stylesheet from file
     *
     * @throws SchematronErrorException On error...
     */
    private void loadSchematronDocument() throws SchematronErrorException {
        try {
            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.load(schematronDocumentPath);
            schematronDocument = xmlDocument;
        } catch (Exception e) {
            throw new SchematronErrorException("Failed to load schematron document:" + e.getMessage());
        }
    }

	/**
	 * Rebuilds saxHandlerConfig by current valus of errorXPath and
	 * errorMessageXPath. If one of them is not set, config is cleared
	 */
    private void rebuildConfig() {
    	this.saxHandlerConfig = buildSAXHandlerConfig(this.getErrorXPath(), this.getErrorMessageXPath());
    }

	/**
	 * @return saxHandlerConfig by given values of errorXPath and
	 *         errorMessageXPath. If one of them is not set or they are not
	 *         supported, returns null and logs error message
	 * @since OIORASP 1.3.0
	 */
	public static SchematronValidatorSAXHandlerConfig buildSAXHandlerConfig(String errorXPath, String errorMessageXPath) {
		if (errorXPath == null || errorMessageXPath == null || errorXPath.length() == 0 || errorMessageXPath.length() == 0) {
			return null;
		}
		try {
			return SchematronValidatorSAXHandlerConfig.build(errorXPath, errorMessageXPath);
		} catch (Exception e) {
			log.error("Failed to build SAXHandlerConfig for errorXPath " + errorXPath + ", errorMessageXPath = " + errorMessageXPath, e);
			/*
			 * If configuration is not supported by optimized way of schematron
			 * validation, use old way
			 */
			return null;
		}
	}

    /**
     * @return Configuration for SchematronValidatorSAXUtil
     * @since OIORASP 1.3.0
     */
	public SchematronValidatorSAXHandlerConfig getSaxHandlerConfig() {
		return saxHandlerConfig;
	}

    /**
     * Overwrite configuration for SchematronValidatorSAXUtil
     * @since OIORASP 1.3.0
     */
	public void setSaxHandlerConfig(SchematronValidatorSAXHandlerConfig saxHandlerConfig) {
		this.saxHandlerConfig = saxHandlerConfig;
	}
}