/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.schematron;

import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlException;
import dk.gov.oiosi.xml.schematron.sax.SchematronValidatorSAXHandlerConfig;
import dk.gov.oiosi.xml.schematron.sax.SchematronValidatorSAXUtil;
import dk.gov.oiosi.xml.xpath.DocumentXPathResolver;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;
import dk.gov.oiosi.xml.xslt.XsltUtility;

import javax.xml.transform.Transformer;

/**
 * Schematron validates a given XML-document.
 */
public class SchematronValidator {

    /**
     * XSLT utility.
     */
    private static final XsltUtility UTILITY = new XsltUtility();

    /**
     * XPath to error.
     */
    private String errorXPath;

    /**
     * XPath to error message.
     */
    private String errorMessageXPath;

    /**
     * Schematron document.
     */
    private XmlDocument schematronDocument;

    /**
     * Config for SchematronValidatorSAXUtil
     */
    private SchematronValidatorSAXHandlerConfig saxHandlerConfig;

    /**
     * Default constructor.
     */
    private SchematronValidator() {
    }

    /**
     * Constructor that takes the error XPath and the error message XPath as parameter.
     *
     * @param errorXPath        XPath to error
     * @param errorMessageXPath XPath to error message
     */
    public SchematronValidator(String errorXPath, String errorMessageXPath) {
        this();
        this.errorXPath = errorXPath;
        this.errorMessageXPath = errorMessageXPath;
        this.saxHandlerConfig = SchematronValidationConfig.buildSAXHandlerConfig(errorXPath, errorMessageXPath);
    }

    /**
     * Constructor that takes the configuration as a parameter.
     *
     * @param config configuration
     * @throws SchematronErrorException On error...
     */
    public SchematronValidator(SchematronValidationConfig config) throws SchematronErrorException {
        this();
        this.errorXPath = config.getErrorXPath();
        this.errorMessageXPath = config.getErrorMessageXPath();
        this.schematronDocument = config.getSchematronDocument();
        this.saxHandlerConfig = config.getSaxHandlerConfig();
    }

    /**
     * Schematron validates the given XML-document.
     *
     * @param xmlDocument The document to validate
     * @throws XmlException On error...
     */
    public void schematronValidateXmlDocument(XmlDocument xmlDocument) throws XmlException {
        if (schematronDocument == null) {
            throw new ArgumentNullException("No schematron document is set");
        }

        schematronValidateXmlDocument(xmlDocument, schematronDocument);
    }

    /**
     * Schematron validates the given XML-document with the given schematron document.
     *
     * @param xmlDocument             Document to validate
     * @param xmlSchematronStylesheet Stylesheet to use
     * @throws XmlException On error...
     */
    public void schematronValidateXmlDocument(XmlDocument xmlDocument, XmlDocument xmlSchematronStylesheet) throws XmlException {
        schematronValidateXmlDocument(xmlDocument, xmlSchematronStylesheet, true);
    }

    /**
     * Schematron validates the given XML-document with the given schematron
     * document.
     *
     * @param xmlDocument             Document to validate
     * @param xmlSchematronStylesheet Stylesheet to use
     * @param failOnFirstError        Stop XML processing when first error, defined by errorXPath
     *                                and errorMessageXPath, occurs
     * @throws XmlException             On error...
     * @throws SchematronErrorException
     * @since OIORASP 1.3.0
     */
    public void schematronValidateXmlDocument(XmlDocument xmlDocument, XmlDocument xmlSchematronStylesheet, boolean failOnFirstError)
            throws XmlException {
    	SchematronStore ss = SchematronStoreFactory.getSchematronStore();
    	ss.getCompiledSchematron(xmlSchematronStylesheet.getFileName());
        Transformer transformer;
        try {
            transformer = UTILITY.precompiledStyleSheet(xmlSchematronStylesheet);
        } catch (Exception e) {
            throw new FailedToTransformDocumentException(e);
        }
        schematronValidateXmlDocument(xmlDocument, transformer, failOnFirstError);
    }

    /**
     * Schematron validates the given XML-document with the given pre compiled schematron
     * document.
     *
     * @param document The document to validate
     * @param precompiledSchematronStylesheet
     *                 The pre compiled schematron stylesheet
     * @throws XmlException On error...
     */
    public void schematronValidateXmlDocument(XmlDocument document, Transformer precompiledSchematronStylesheet) throws XmlException {
        schematronValidateXmlDocument(document, precompiledSchematronStylesheet, true);
    }

    /**
     * Schematron validates the given XML-document with the given pre compiled schematron
     * document.
     *
     * @param document         The document to validate
     * @param precompiledSchematronStylesheet
     *                         The pre compiled schematron stylesheet
     * @param failOnFirstError XML processing when first error, defined by errorXPath and errorMessageXPath, occurs
     * @throws XmlException On error...
     * @since OIORASP 1.3.0
     */
    public void schematronValidateXmlDocument(XmlDocument document, Transformer precompiledSchematronStylesheet, boolean failOnFirstError) throws XmlException {
        if (!failOnFirstError || this.saxHandlerConfig == null) {
            XmlDocument result;
            try {
                result = UTILITY.transformXml(document, precompiledSchematronStylesheet);
            } catch (Exception e) {
                throw new FailedToTransformDocumentException(e);
            }

            validateResult(result);
            return;
        }
        SchematronValidatorSAXUtil.schematronValidateXmlDocument(document.getTransformationSource(), precompiledSchematronStylesheet, this.saxHandlerConfig);
    }

    private void validateResult(XmlDocument result) throws XmlException {
        PrefixedNamespace[] prefixedNamespaces = new PrefixedNamespace[0];
        boolean hasAnyErrors = DocumentXPathResolver.hasAnyElementsByXpath(result, errorXPath, prefixedNamespaces);
        if (hasAnyErrors) {
            String firstErrorMessage = DocumentXPathResolver.getFirstElementValueByXPath(result, errorMessageXPath,
                    prefixedNamespaces);
            throw new SchematronErrorException(result, firstErrorMessage);
        }
    }
}