package dk.gov.oiosi.xml.schematron.sax;

import dk.gov.oiosi.xml.schematron.SchematronErrorException;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.Serializable;

/**
 * OIORASP schematron-specific implementation of SAX ContentHandler.<br>
 *
 * Main goal - prevent transformation of the whole document when first error is
 * found.<br>
 *
 * Implements SAX XPath processing, throws {@link SchematronValidatorSAXException} when
 * first matched tag defined by errorMessageXPath is closed.<br>
 *
 * It is supposed to be used only via {@link SchematronValidatorSAXUtil}.<br>
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class SchematronValidatorSAXHandler extends DefaultHandler implements Serializable {

	private static final long serialVersionUID = -4973065739708054681L;

	/*
	 * Number of visited tags in tagList
	 */
	private int tagListMatchIndex;

	/*
	 * Number of parents of current tag
	 */
	private int depth;
	/*
	 * Collects data from all calls of characters() and ignorableWhitespace()
	 * when errorMessageTagActual is true
	 */
	private StringBuilder errorMessageResult;

	/*
	 * true when all tagList was visited and start of errorMessageTag found
	 */
	private boolean errorMessageTagFound;

	/*
	 * true when startElement method was called with tag equal to
	 * errorMessageTag and corresponding endElement not yet called
	 */
	private boolean errorMessageTagActual;

	/*
	 * To avoid collecting charecters inside nested tags of errorMessageTag,
	 * let's calculate opened/closed tags inside it. If it is 0, we
	 */
	private int errorMessageTagRelativeDepth = -1;

	private final SchematronValidatorSAXHandlerConfig config;

	/**
	 * Reduces visibility of this class by making its constructor protected.
	 *
	 * This class is supposed to be used only via {@link SchematronValidatorSAXUtil}.
	 *
	 * @param config configuration for handler
	 * @throws SchematronErrorException
	 */
	protected SchematronValidatorSAXHandler(SchematronValidatorSAXHandlerConfig config) throws SchematronErrorException {
		if (config == null) {
			throw new SchematronErrorException("SchematronValidatorSAXHandler was called with null config object");
		}
		this.config = config;
	}

	@Override
	public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
		if (!errorMessageTagFound) {
			if (tagListMatchIndex == depth && tagListMatchIndex < config.getTagList().length) {
				if (qName.equals(config.getTagList()[tagListMatchIndex])) {
					tagListMatchIndex++;
				}
			}
		}
		depth++;
		if (tagListMatchIndex == config.getTagList().length && depth == (config.getTagList().length + 1)) {
			/*
			 * We are at the end of tag list, let's check is current tag matches
			 * errorMessageTag
			 */
			if (config.getErrorMessageTag().equals(qName)) {
				errorMessageTagFound = true;
				errorMessageTagActual = true;
				errorMessageTagRelativeDepth = 0;
			}
		}
		if (errorMessageTagFound) {
			if (errorMessageTagActual) {
				if (!qName.equals(config.getErrorMessageTag())) {
					errorMessageTagActual = false;
					errorMessageTagRelativeDepth++;
				}
			} else {
				errorMessageTagRelativeDepth++;
			}
		}
	}

	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException {
		if (errorMessageTagFound && errorMessageTagActual) {
			if (depth == (config.getTagList().length + 1)) {
				String message = "";
				if (errorMessageResult != null) {
					message = errorMessageResult.toString();
				}
				throw new SchematronValidatorSAXException(message);
			}
		}
		depth--;
		if (errorMessageTagFound) {
			if (!errorMessageTagActual) {
				errorMessageTagRelativeDepth--;
				if (errorMessageTagRelativeDepth == 0) {
					errorMessageTagActual = true;
				}
			}
		}
		if (depth < tagListMatchIndex && tagListMatchIndex > 0) {
			tagListMatchIndex--;
		}
	}

	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		appendErrorMessageResult(ch, start, length);
	}

	@Override
	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
		appendErrorMessageResult(ch, start, length);
	}

	protected void appendErrorMessageResult(char[] ch, int start, int length) {
		if (errorMessageTagActual) {
			if (errorMessageResult == null) {
				errorMessageResult = new StringBuilder(length);
			}
			errorMessageResult.append(ch, start, length);
		}
	}

}
