package dk.gov.oiosi.xml.schematron.sax;

import dk.gov.oiosi.xml.XmlException;
import dk.gov.oiosi.xml.schematron.SchematronErrorException;
import dk.gov.oiosi.xml.schematron.SchematronValidationFailedException;

import javax.xml.transform.ErrorListener;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.sax.SAXResult;

/**
 * Main entry class for SAX-based schematron validation.<br>
 *
 * It allows to stop XML document transformation, when first error description
 * during schematron validation is found.<br>
 *
 * Performance analysis shows that it makes schematron validation for invalid
 * documents in general on big amoung of documents to be at least 50% faster (of
 * cause it depends on transformer and error check position...)<br>
 *
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 *
 */
public class SchematronValidatorSAXUtil {

	private static final FailoverErrorListener ERROR_LISTENER = new FailoverErrorListener();

	/**
	 * In case if config is defined, transforms given xmlSource by given
	 * transformer and stops processing when first match of tag
	 * config.errorMessageTag is found, whose parent tags are equal to
	 * config.tagList. Returns string with contents of matched tag.<br>
	 *
	 * Does nothing if config is undefined.<br>
	 *
	 * @param xmlSource
	 *            Source for xml document to process
	 * @param transformer
	 *            Transformer to apply
	 * @param config
	 *            configuration object
	 * @throws SchematronValidationFailedException
	 * @throws XmlException
	 * @throws SchematronErrorException
	 */
	public static void schematronValidateXmlDocument(Source xmlSource, Transformer transformer, SchematronValidatorSAXHandlerConfig config) throws SchematronValidationFailedException, XmlException, SchematronErrorException {
		if (config != null) {
			try {
				SchematronValidatorSAXHandler contentHandler = new SchematronValidatorSAXHandler(config);
				transformer.setErrorListener(ERROR_LISTENER);
				transformer.transform(xmlSource, new SAXResult(contentHandler));
			} catch (Exception e) {
				if (e instanceof TransformerException) {
					if (e.getCause() != null && e.getCause() instanceof SchematronValidatorSAXException) {
						SchematronValidatorSAXException s = (SchematronValidatorSAXException) e.getCause();
						throw new SchematronErrorException(s.getMessage());
					}
				}
				throw new SchematronValidationFailedException(e.getMessage(), e);
			}
		}
	}

	/*
	 * Custom ErrorListener implementation is required to inform client code
	 * about SchematronValidatorSAXException with resolved error description.
	 */
	private static class FailoverErrorListener implements ErrorListener {
		@Override
		public void warning(TransformerException exception) throws TransformerException {
			/*
			 * Ignore warnings
			 */
		}

		@Override
		public void fatalError(TransformerException exception) throws TransformerException {
			throw exception;
		}

		@Override
		public void error(TransformerException exception) throws TransformerException {
			throw exception;
		}
	}

}
