/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.xpath.discriminator;

import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlException;
import dk.gov.oiosi.xml.xpath.DocumentXPathResolver;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;

/**
 * Provides functions that can discriminate a document from one or more xpath
 * expressions with their corresponding expected result.
 */
public class XPathDiscriminator {

    /**
     * Discriminates the xml document from a single configuration element
     *
     * @param config
     *            XPathDiscriminatorConfig to match
     * @param xmlDocument
     *            document to search
     * @param namespaces
     *            namespaces to use
     * @return true if searched expresssion matches result in configCollection
     * @throws XmlException
     *             if xpath search cannot be performed
     */
    public static boolean discriminate(XPathDiscriminatorConfig config, XmlDocument xmlDocument, PrefixedNamespace[] namespaces)
            throws XmlException {
        String xpathExpression = config.getXPathExpression();
        String xpathResult = config.getXPathExpectedResult();

        String result = DocumentXPathResolver.getElementValueByXpath(xmlDocument, xpathExpression, namespaces);

        Pattern pattern = Pattern.compile(xpathResult); 
        Matcher matcher = pattern.matcher(result); 
        
        return matcher.matches();
    }

    /**
     * Discriminates the xml document from a collection of configuration
     * elements. The results are anded together.
     *
     * @param configCollection
     *            collection of XPathDiscriminatorConfig
     * @param xmlDocument
     *            document to search
     * @param namespaces
     *            namespaces to use
     * @return true if searched expresssion matches result in configCollection
     * @throws XmlException
     *             if xpath search cannot be performed
     */
    public static boolean discriminateCollectionAnded(XPathDiscriminatorConfigCollection configCollection, XmlDocument xmlDocument,
            PrefixedNamespace[] namespaces) throws XmlException {
        Iterator<XPathDiscriminatorConfig> iterator = configCollection.getIterator();
        while (iterator.hasNext()) {
            XPathDiscriminatorConfig config = iterator.next();
            if (!discriminate(config, xmlDocument, namespaces)) {
                return false;
            }
        }
        return true;
    }
}
