/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.xslt;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlDocumentException;
import dk.gov.oiosi.xml.XmlException;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.InputSource;

import javax.xml.stream.XMLStreamException;
import javax.xml.transform.*;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

/**
 * This utility is used for xslt transforming documents.
 * <p/>
 * It is a simple service class without state, thread safe and cachable.
 */
public class XsltUtility {

    /**
     * logger.
     */
    private static final Log log = LogFactory.getLog(XsltUtility.class);

    /**
     * Method to transform any xmlDocument with a given xslt.
     *
     * @param xmlDoc     The xml document to transform.
     * @param stylesheet The xslt to transform xml document.
     * @return The transformed xml document
     * @throws XmlException Xml transformations errors
     */
    public XmlDocument transformXML(XmlDocument xmlDoc, XmlDocument stylesheet) throws XmlException {
        try {
            Transformer trans = precompiledStyleSheet(stylesheet);
            return transformXml(xmlDoc, trans);
        } catch (TransformerConfigurationException e) {
            log.error("XsltUtility.transformXML Failed : " + e, e);
            throw new XmlException("XSL transformation error: " + e.getMessage(), e);
        } catch (TransformerException e) {
            log.error("XsltUtility.transformXML Failed : " + e, e);
            throw new XmlException("XSL transformation error: " + e.getMessage(), e);
        } catch (XMLStreamException e) {
            log.error("XsltUtility.transformXML Failed : " + e, e);
            throw new XmlException("XSL transformation error: " + e.getMessage(), e);
        } catch (Exception e) {
            log.error("XsltUtility.transformXML Failed : " + e, e);
            throw new RaspLibraryException("RASP transformation error ! " + e, e);
        }
    }

    /**
     * Method that returns the pre-compiled XSLT stylesheet from the given XML document.
     */
    public Transformer precompiledStyleSheet(XmlDocument stylesheet) throws XmlDocumentException, TransformerConfigurationException {
        /*
           * Since OIORASP 1.3.0 stop conversion of potentially Axiom model inside
           * XmlDocument to W3C DOM model - just use proper Source implementation
           */
        Source xsltSource = stylesheet.getTransformationSource();
        TransformerFactory transFact = TransformerFactory.newInstance();
        return transFact.newTransformer(xsltSource);
    }

    /**
     * Method that returns the pre-compiled XSLT stylesheet from the given XML document.
     *
     * @return Transformer object for given path to XSLT file
     * @since OIORASP 1.3.0
     */
    public Transformer precompiledStyleSheet(String path) throws XmlDocumentException, TransformerConfigurationException, FileNotFoundException {
        if (path == null) {
            throw new FileNotFoundException("Path to XSLT file is not defined");
        }
        InputSource inputStream = new InputSource(XmlDocument.locateInputStream(path));
        Source xsltSource = new SAXSource(inputStream);
        TransformerFactory transFact = TransformerFactory.newInstance();
        return transFact.newTransformer(xsltSource);
    }

    /**
     * Method that transforms the XML document from a precompiled XSLT stylesheet
     */
    public XmlDocument transformXml(XmlDocument document, Transformer transform) throws XmlDocumentException, TransformerException, XMLStreamException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        StreamResult streamResult = new StreamResult(byteArrayOutputStream);

        log.trace("Doing transformation...");
        /*
           * Since OIORASP 1.3.0 stop conversion of potentially Axiom model inside
           * XmlDocument to W3C DOM model - just use proper Source implementation
           */
        transform.transform(document.getTransformationSource(), streamResult);

        log.trace("Creating AXOMbuilder");
        InputStream inputStream = new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
        StAXOMBuilder builder = new StAXOMBuilder(inputStream);

        return new XmlDocument(builder.getDocumentElement());
    }
}