package dk.gov.oiosi;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * This class is required to run unit tests inside IDE. This allows to use code
 * coverage tools to check test coverage, also as to have more convenient way to
 * run unit tests.
 * 
 * IMPORTANT: synchronize list of exclusions in static field
 * EXCLUDE_PATTERN_LIST with build.xml configuration.
 * 
 * At the moment of commit both build.xml and this class executed 228 unit test
 * methods.
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * 
 */
public class AllBuildTests {

	private static final String TEST_PACKAGE = "dk.gov.oiosi";

	private static final String[] EXCLUDE_PATTERN_LIST = new String[] {
	
		"dk/gov/oiosi/test/**/integration/**",
	
		"dk/gov/oiosi/**/research/**",
	
		"dk/gov/oiosi/test/**/client/**"

	};
	private static List<String> EXCLUDE_PATTERN_LIST_PREPARED;

	private static final boolean SCAN_ONLY = false;

	static {
		EXCLUDE_PATTERN_LIST_PREPARED = new ArrayList<String>(EXCLUDE_PATTERN_LIST.length);
		for (int i = 0; i < EXCLUDE_PATTERN_LIST.length; i++) {
			String excludePattern = EXCLUDE_PATTERN_LIST[i];
			excludePattern = excludePattern.replace('/', '.');
			excludePattern = excludePattern.replaceAll("\\*\\*", "*");
			EXCLUDE_PATTERN_LIST_PREPARED.add(excludePattern);
		}
	}

	public static Test suite() {
		TestSuite suite = new TestSuite(AllBuildTests.class.getName());
		URL resource = AllBuildTests.class.getResource("/" + AllBuildTests.class.getName().replace('.', '/') + ".class");
		System.out.println("Scan parent folder of file " + resource + " for unit test classes...");

		UnitTestSearcher searcher = new UnitTestSearcher(suite, TEST_PACKAGE, "Test.class");
		searcher.setAddToSuite(!SCAN_ONLY);

		if ("file".equals(resource.getProtocol())) {
			String path = resource.getPath();
			File parentFolder = new File(path).getParentFile();
			searcher.findUnitTests(parentFolder);
			System.out.println(searcher);
		}
		return suite;
	}

	private static final class UnitTestSearcher {

		private final TestSuite suite;
		private int found;
		private int skipped;
		private final String suffix;
		private String searchPackage;
		private boolean addToSuite;

		public UnitTestSearcher(TestSuite suite, String searchPackage, String suffix) {
			this.suite = suite;
			this.searchPackage = searchPackage.replace('.', File.separatorChar);
			this.suffix = suffix;
		}

		public void findUnitTests(File folder) {
			scanFolder(folder);
		}

		@SuppressWarnings("unchecked")
		private void scanFolder(File folder) {
			File[] files = folder.listFiles();
			for (File file : files) {
				if (file.isDirectory()) {
					scanFolder(file);
				} else {
					String fileName = file.getAbsolutePath();
					if (fileName.endsWith(suffix)) {
						int packageStart = fileName.indexOf(searchPackage);
						String className = fileName.substring(packageStart).replace('/', '.').replace('\\', '.');
						className = className.substring(0, className.length() - ".class".length());
						if (isExclude(className)) {
							System.out.println("SKIP " + className);
							skipped++;
						} else {
							try {
								Class<?> c = Class.forName(className);
								if (isDescendant(c, TestCase.class)) {
									found++;
									if (isAddToSuite()) {
										suite.addTestSuite((Class<? extends TestCase>) c);
									}
								} else {
									System.out.println("WARN: class " + className + " matches pattern but does not extend class " + TestCase.class);
								}
							} catch (Exception e) {
								e.printStackTrace();
							}
						}
					}
				}
			}
		}

		protected boolean isDescendant(Class<?> scanClass, Class<?> ancestorClass) {
			Class<?> parent = scanClass.getSuperclass();
			while (parent != null) {
				if (parent == ancestorClass) {
					return true;
				}
				parent = parent.getSuperclass();
			}
			return false;
		}

		private static boolean isExclude(String className) {
			for (int i = 0; i < EXCLUDE_PATTERN_LIST_PREPARED.size(); i++) {
				String excludePattern = EXCLUDE_PATTERN_LIST_PREPARED.get(i);
				if (className.matches(excludePattern)) {
					return true;
				}
			}
			return false;
		}

		@Override
		public String toString() {
			StringBuilder builder = new StringBuilder();
			builder.append("found=");
			builder.append(found);
			builder.append(", skipped=");
			builder.append(skipped);
			builder.append(", suffix=");
			builder.append(suffix);
			builder.append(", searchPackage=");
			builder.append(searchPackage);
			return builder.toString();
		}

		public boolean isAddToSuite() {
			return addToSuite;
		}

		public void setAddToSuite(boolean addToSuite) {
			this.addToSuite = addToSuite;
		}

	}
}
