package dk.gov.oiosi.common.schedule;

import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Calendar;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class ScheduledExecutorTest extends TestCase {

    private static final Log log = LogFactory.getLog(ScheduledExecutorTest.class);

    public void testAllMethods() {
        /*
           * Let's wait while ScheduledExecutor is free from active threads to be
           * sure that we can guarantee exact number of executions during the test
           */
        int activeCount = 0;
        ScheduledExecutor s = new ScheduledExecutor() {
            @Override
            protected int calculateThreadPoolSize() {
                /*
                     * In order to get stable results even if other tests were
                     * executed before in the same JVM, let's increase number of
                     * thread to a big figure
                     */
                return 10;
            }
        };
        while ((activeCount = s.getCurrentActiveCount()) > 0) {
            System.out.println("StartCacheStateLogScheduledJobStartupTaskTest: activeCount now is not 0 = " + activeCount + ", let's wait while everything finished");
            synchronized (this) {
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                }
            }
        }
        s.runJobNow(new EmptyJob("Run now 1"));
        s.runJobNow(new EmptyJob("Run now 2"));

        waitThis(50);

        assertEquals(2, s.getCurrentActiveCount());

        assertTrue(!s.isShutdown());
        assertTrue(!s.isTerminated());

        s.scheduleJob(new EmptyJob("Start each 50 ms, stop after first"), 50, TimeUnit.MILLISECONDS);

        Calendar c = Calendar.getInstance();
        c.add(Calendar.MILLISECOND, 30);

        s.scheduleJob(new EmptyJob("Delayed start in 100 ms"), c.getTime(), 50, TimeUnit.MILLISECONDS);

        waitThis(100);
        assertEquals(4, s.getCurrentActiveCount());

        try {
            s.awaitTermination(10, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        s.shutdown();

        s.runJobNow(new EmptyJob("Run after shutdown - job will not be started"));
        s.scheduleJob(new EmptyJob("Run after shutdown - job will not be started"), 5, TimeUnit.MILLISECONDS);
        s.scheduleJob(new EmptyJob("Run after shutdown - job will not be started"), c.getTime(), 5, TimeUnit.MILLISECONDS);

        waitThis(100);
        /*
           * Check that active count is still 4, not increased by one
           */
        assertEquals(4, s.getCurrentActiveCount());
        /*
           * Interrupt all jobs
           */
        List<Runnable> list = s.shutdownNow();
        assertEquals(0, list.size());
        /*
           * Wait while all terminated tasks stopped
           */
        waitThis(50);
        assertEquals(0, s.getCurrentActiveCount());
    }

    private void waitThis(int i) {
        synchronized (this) {
            try {
                this.wait(i);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    private static class EmptyJob extends AbstractScheduleJob {
        private static final long serialVersionUID = 8797734813277507784L;
        private final String desc;

        public EmptyJob(String desc) {
            this.desc = desc;
        }

        @Override
        public Log getLog() {
            return log;
        }

        @Override
        public String getJobDetails() {
            return "EmptyJob - wait 1 second, desc = " + desc;
        }

        @Override
        public boolean doJob() {
            synchronized (this) {
                try {
                    this.wait(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            return false;
        }
    }

    public void testGetCurrentActiveCount() {
        ScheduledExecutor s = new ScheduledExecutor();
        /*
           * Check that default implementation is ScheduledThreadPoolExecutor
           */
        assertTrue(s.getCurrentActiveCount() >= 0);
        /*
           * Just for test coverage:
           */
        s.executorExtInterface = null;
        assertEquals(-1, s.getCurrentActiveCount());
        s.shutdownNow();
    }

    public void testCalculateThreadPoolSize() {
        final int[] CPU = new int[1];
        CPU[0] = -1;
        ScheduledExecutor s = new ScheduledExecutor() {
            @Override
            protected int getAvailableCPU() {
                return CPU[0];
            }
        };
        /*
           * Check that thread pool size will never be less than 2
           */
        assertEquals(2, s.calculateThreadPoolSize());
        s.shutdownNow();
        waitThis(50);
        CPU[0] = 24;
        /*
           * Check that thread pool size will be equal to number of CPUs if there
           * are more or equal than 2
           */
        assertEquals(24, s.calculateThreadPoolSize());
        s.shutdownNow();
    }

    public void testGetInstance() {
        assertNotNull(ScheduledExecutor.getInstance());
    }
}