package dk.gov.oiosi.common.startup;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import junit.framework.TestCase;
import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import dk.gov.oiosi.common.schedule.ScheduledExecutorCleaner;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.configuration.ConfigurationHandlerCleaner;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.revocation.RevocationLookupFactoryCleaner;
import dk.gov.oiosi.security.revocation.crl.CrlCacheUpdateJob;
import dk.gov.oiosi.test.TestConstants;

public class DownloadCRLStartupTaskTest extends TestCase {

	public void setUp() throws Exception {
		ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
	}

	/*
	 * dlk, 2012-03-29
	 * 
	 * This test throws exception now - cannot get server URI from OCSP
	 * certificate.
	 * 
	 * Jacob, could you please check this OCES_Test.jks for valid certificates
	 * there?
	 */
	public void testDoTask() throws MainException, InterruptedException {
		Level initial = Logger.getLogger(CrlCacheUpdateJob.class).getLevel();
		try {
			Logger.getLogger(CrlCacheUpdateJob.class).setLevel(Level.DEBUG);
			List<StartupTaskConfiguration> configList = new ArrayList<StartupTaskConfiguration>();
			configList.add(new StartupTaskConfiguration(DownloadCRLStartupTask.CONFIG_DOWNLOAD_CRL_URL, "http://crl.oces.certifikat.dk/oces.crl"));
			configList.add(new StartupTaskConfiguration(DownloadCRLStartupTask.CONFIG_DOWNLOAD_CRL_URL, "http://crl.systemtest7.trust2408.com/systemtest7.crl"));
			configList.add(new StartupTaskConfiguration(DownloadCRLStartupTask.CONFIG_DOWNLOAD_CRL_URL, "http://crl.systemtest8.trust2408.com/systemtest8.crl"));
			configList.add(new StartupTaskConfiguration(DownloadCRLStartupTask.CONFIG_DOWNLOAD_CRL_URL, "http://unexistingurl.dk/unexisting.crl"));
			configList.add(new StartupTaskConfiguration(DownloadCRLStartupTask.CONFIG_DOWNLOAD_CRL_URL, "INVALID URL"));
			DownloadCRLStartupTask t = new DownloadCRLStartupTask(configList) {
				/*
				 * Override to be independent from current
				 * RaspConfiguration.TEST.xml
				 */
				@Override
				protected boolean isRevocationCrlSource() {
					return true;
				}
			};
			t.doTask();

			/*
			 * Give time all scheduled jobs to start
			 */
			Thread.sleep(1000);

			ScheduledExecutor.getInstance().shutdownNow();
			/*
			 * Wait all jobs to finish their current execution (new executions
			 * will not start) but not more than 1 day
			 */
			ScheduledExecutor.getInstance().awaitTermination(1, TimeUnit.DAYS);

			/*
			 * Let's check that we stopped everything correctly
			 */
			assertEquals(0, ScheduledExecutor.getInstance().getCurrentActiveCount());
		} finally {
			Logger.getLogger(CrlCacheUpdateJob.class).setLevel(initial);
			/*
			 * It is important to clear scheduled executor in tests, otherwise
			 * other tests for scheduled jobs can fail
			 */
			ScheduledExecutorCleaner.clear();
		}
	}

	public void testNonCrlImpl() throws MainException {
		final boolean res[] = new boolean[1];
		final boolean isRevocationCrlSource[] = new boolean[1];

		List<StartupTaskConfiguration> configList = new ArrayList<StartupTaskConfiguration>();
		configList.add(new StartupTaskConfiguration(DownloadCRLStartupTask.CONFIG_DOWNLOAD_CRL_URL, "http://crl.oces.certifikat.dk/oces.crl"));
		DownloadCRLStartupTask t = new DownloadCRLStartupTask() {
			/*
			 * Override to be independent from current
			 * RaspConfiguration.TEST.xml
			 */
			@Override
			protected boolean isRevocationCrlSource() {
				return isRevocationCrlSource[0];
			}

			@Override
			protected void downloadCrlUrl(URL url) {
				res[0] = true;
			}
		};
		t.setConfigList(configList);
		t.doTask();
		/*
		 * Check that downloadCrlUrl was not called
		 */
		assertTrue(!res[0]);
		/*
		 * Now let's check that if it was CRL source, downloadCrlUrl would be
		 * called
		 */
		isRevocationCrlSource[0] = true;
		t.doTask();
		assertTrue(res[0]);
	}

	public void testCoverageTests() throws MainException {
		/*
		 * The only goal for these tests - to be sure that all lines of code are
		 * executed
		 */
		DownloadCRLStartupTask t = new DownloadCRLStartupTask() {
			@Override
			protected boolean isRevocationCrlSource() {
				return true;
			}

			@Override
			protected void downloadCrlUrl(URL url) {
				throw new IllegalArgumentException("Test what happens if download fails");
			}
		};
		/*
		 * Run task without any parameters - log warning should be recorded
		 */
		t.doTask();
		t.setConfigList(null);
		/*
		 * Check that configuration list can never be null
		 */
		assertNotNull(t.getConfigList());
		/*
		 * Check that nothing fails
		 */
		t.doTask();
		/*
		 * Check with invalid URL - exception should be thrown as we override
		 * the downloadCrlUrl implementation.
		 */
		List<StartupTaskConfiguration> configList = new ArrayList<StartupTaskConfiguration>();
		configList.add(new StartupTaskConfiguration(DownloadCRLStartupTask.CONFIG_DOWNLOAD_CRL_URL, "http://crl.oces.certifikat.dk/oces.crl"));
		t.setConfigList(configList);
		/*
		 * Error should be logged
		 */
		t.doTask();
	}

	public void testIsRevocationCrlSource() {
		try {
			DownloadCRLStartupTask t = new DownloadCRLStartupTask();
			/*
			 * OCSP lookup is not CRL source
			 */
			assertFalse(isRevocationCrlSourceCase(t, "Ocsp"));
			assertTrue(isRevocationCrlSourceCase(t, "SimpleCrl"));
			/*
			 * Let's return back OCSP to be sure that CrlLookup 100% works
			 */
			assertFalse(isRevocationCrlSourceCase(t, "Ocsp"));
			/*
			 * Now check CrlLookup
			 */
			assertTrue(isRevocationCrlSourceCase(t, "Crl"));
			/*
			 * Check non-existing class - should return also false
			 */
			assertFalse(isRevocationCrlSourceCase(t, "Unknown"));
		} finally {
			/*
			 * Return back original configuration to avoid problems with other
			 * tests
			 */
			ConfigurationHandlerCleaner.clear();
			RevocationLookupFactoryCleaner.clear();
			ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
		}
	}

	private boolean isRevocationCrlSourceCase(DownloadCRLStartupTask t, String caseName) {
		ConfigurationHandlerCleaner.clear();
		RevocationLookupFactoryCleaner.clear();
		URL resource = this.getClass().getResource("DownloadCRLStartupTaskTest_RaspConfiguration_" + caseName + "Lookup.xml");
		ConfigurationHandler.setConfigurationFile(resource.getFile());
		return t.isRevocationCrlSource();
	}
}
