package dk.gov.oiosi.common.startup;

import dk.gov.oiosi.AbstractTestCase;
import dk.gov.oiosi.common.cache.stat.CacheStateLogScheduledJob;
import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class StartCacheStateLogScheduledJobStartupTaskTest extends AbstractTestCase {

    private int execCount;

    protected void setUp() throws Exception {
        super.setUp();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }

    public void testDoTaskFast() throws MainException {
        /*
           * Check that task is executed without exceptions in general - fast test
           */
        /*
           * At first with default log level
           */
        StartCacheStateLogScheduledJobStartupTask t = new StartCacheStateLogScheduledJobStartupTask();
        t.doTask();
    }

    public void testParamsParsing() throws MainException {
        assertTrue("Loglevel is above INFO", CacheStateLogScheduledJob.log.isInfoEnabled());
        /*
           * Test configuration parameters parsing
           */
        final FinalObjectParam expectedParams = new FinalObjectParam();
        execCount = 0;
        StartCacheStateLogScheduledJobStartupTask t = new StartCacheStateLogScheduledJobStartupTask() {
            @Override
            protected void scheduleJob(long scheduleJobRunInterval, TimeUnit scheduleJobRunIntervalTimeUnit) {
                assertEquals(expectedParams.value, scheduleJobRunInterval);
                assertEquals(expectedParams.unit, scheduleJobRunIntervalTimeUnit);
                execCount++;
            }
        };

        /*
        * Check valid unreal configuration - 300 days
        */
        expectedParams.set(300, TimeUnit.DAYS);
        t.setConfigList(buildConfigList(expectedParams));
        t.doTask();
        assertEquals(1, execCount);

        /*
        * Check valid real configuration, different to default - 10 mins
        */
        expectedParams.set(10, TimeUnit.MINUTES);
        t.setConfigList(buildConfigList(expectedParams));
        t.doTask();
        assertEquals(2, execCount);

        /*
        * Check real empty configuration - expect default
        */
        expectedParams.set(StartCacheStateLogScheduledJobStartupTask.DEFAULT_SCHEDULE_JOB_RUN_INTERVAL, StartCacheStateLogScheduledJobStartupTask.DEFAULT_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT);
        t.setConfigList(new ArrayList<StartupTaskConfiguration>());
        t.doTask();
        assertEquals(3, execCount);

        /*
        * Check possible invalid configuration 1 (wrong time unit) - expect
        * default by both values
        */
        /*
        * HOUR is not valid - HOURS is correct
        */
        t.setConfigList(buildConfigList("1", "HOUR"));
        t.doTask();
        assertEquals(4, execCount);
        t.setConfigList(buildConfigList("1", "MINUTE"));
        t.doTask();
        assertEquals(5, execCount);

        /*
        * Check possible invalid configuration 2 (unparsed amount of time)
        * - expect default by both values
        */
        /*
        * HOUR is not valid - HOURS is correct
        */
        t.setConfigList(buildConfigList("BALB", TimeUnit.HOURS.name()));
        t.doTask();
        assertEquals(6, execCount);
        t.setConfigList(buildConfigList("", TimeUnit.HOURS.name()));
        t.doTask();
        assertEquals(7, execCount);
    }

    public void testDoTaskSchedule() throws MainException {
        assertTrue("Loglevel is above INFO", CacheStateLogScheduledJob.log.isInfoEnabled());
        /*
        * Check that task is executed without exceptions in general - fast test
        */
        final EmptyCacheStateLogScheduledJob job = new EmptyCacheStateLogScheduledJob();
        StartCacheStateLogScheduledJobStartupTask t = new StartCacheStateLogScheduledJobStartupTask() {
            @Override
            protected CacheStateLogScheduledJob buildJobImpl() {
                return job;
            }
        };

        t.setConfigList(buildConfigList(100, TimeUnit.MILLISECONDS));

        /*
        * Let's wait while ScheduledExecutor is free from active threads to
        * be sure that we can guarantee exact number of executions during
        * the test
        */
        int activeCount;
        while ((activeCount = ScheduledExecutor.getInstance().getCurrentActiveCount()) > 0) {
            System.out.println("StartCacheStateLogScheduledJobStartupTaskTest: activeCount now is not 0 = " + activeCount + ", let's wait while everything finished");
            synchronized (this) {
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                }
            }
        }
        t.doTask();

        /*
        * We just executed task so it had to schedule a job to run each 100 millis.
        *
        * Let's sleep 500 millis and check that number of executions is between 4 and 6
        */

        synchronized (this) {
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
            }
        }

        assertTrue("Unexpected number of job executions: " + job.execCount, job.execCount >= 4 && job.execCount <= 6);
    }

    private class EmptyCacheStateLogScheduledJob extends CacheStateLogScheduledJob {
        private static final long serialVersionUID = 5078780370968253223L;

        int execCount;

        @Override
        public boolean doJob() {
            execCount++;
            return true;
        }
    }

    protected static class FinalObjectParam {
        int value;
        TimeUnit unit;

        void set(int value, TimeUnit unit) {
            this.value = value;
            this.unit = unit;
        }
    }

    private List<StartupTaskConfiguration> buildConfigList(FinalObjectParam p) {
        return buildConfigList(p.value, p.unit);
    }

    private List<StartupTaskConfiguration> buildConfigList(int value, TimeUnit unit) {
        return buildConfigList(String.valueOf(value), String.valueOf(unit));
    }

    private List<StartupTaskConfiguration> buildConfigList(String value, String unit) {
        List<StartupTaskConfiguration> list = new ArrayList<StartupTaskConfiguration>();
        list.add(new StartupTaskConfiguration(StartCacheStateLogScheduledJobStartupTask.CONFIG_SCHEDULE_JOB_RUN_INTERVAL, value));
        list.add(new StartupTaskConfiguration(StartCacheStateLogScheduledJobStartupTask.CONFIG_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT, unit));
        return list;
    }
}