/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.ldap;

import dk.gov.oiosi.common.DateUtil;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import java.math.BigInteger;
import java.security.cert.X509Certificate;

public class LdapCertificateLookupTest extends TestCase {

    private LdapCertificateLookup ldapCertificateLookup;

    @Override
    protected void setUp() throws Exception {
        super.setUp();

        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);

        LdapSettings ldapProdSettings = new LdapSettings();
        ldapProdSettings.setHost("crtdir.certifikat.dk");
        ldapProdSettings.setPort(389);

        ldapCertificateLookup = new LdapCertificateLookup(ldapProdSettings);
    }

    public void testGetCertificate_foces1() throws Exception {
        Logger.getLogger(LdapCertificateLookup.class).setLevel(Level.TRACE);
        String certSubject = "SERIALNUMBER=CVR:34051178-FID:1359538750995 + CN=DIGST Demo endpoint (funktionscertifikat), O=Digitaliseringsstyrelsen // CVR:34051178, C=DK";
        X509Certificate result = ldapCertificateLookup.getCertificateFromLDAP(new CertificateSubject(certSubject));
//        System.err.println(result);
        assertEquals("Wrong cert. found...", BigInteger.valueOf(1170829175), result.getSerialNumber());
        assertEquals("Wrong start date found...", "2013-01-30T09:13:13 UTC", DateUtil.formatDate(result.getNotBefore()));
    }

    public void testGetCertificate_foces2() throws Exception {
        Logger.getLogger(LdapCertificateLookup.class).setLevel(Level.TRACE);
        String certSubject = "SERIALNUMBER=CVR:34051178-FID:55310689 + CN=Digst Demo Endpoint Foces2 (funktionscertifikat), O=Digitaliseringsstyrelsen // CVR:34051178, C=DK";
        X509Certificate result = ldapCertificateLookup.getCertificateFromLDAP(new CertificateSubject(certSubject));
//        System.err.println(result);
        assertEquals("Wrong cert. found...", BigInteger.valueOf(1284387110), result.getSerialNumber());
        assertEquals("Wrong expire date found...", "2016-06-07T09:07:23 UTC", DateUtil.formatDate(result.getNotAfter()));
    }

    public void testGetCertificate_nonExistingSubject() throws Exception {
        String certSubject = "CN=Y (funktionscertifikat) + OID.2.5.4.5=CVR:11111111-FID:2222222222222, O=X, C=DK";
        try {
            ldapCertificateLookup.getCertificateFromLDAP(new CertificateSubject(certSubject));
            fail("Exception not thrown - as expected");
        } catch (LdapCertificateNotFoundException e) {
            assertEquals("Exception changed", certSubject, e.getMessage());
        }
    }

    public void testWrongLdapServer() {
        LdapSettings setting = new LdapSettings();
        setting.setHost("WrongLdapServer");
        LdapCertificateLookup ldapLookup = new LdapCertificateLookup(setting);
        try {
            CertificateSubject certSubject = new CertificateSubject(TestConstants.TEST_CERTIFICATE_SUBJECT_COMPANY);
            X509Certificate result = ldapLookup.getCertificateFromLDAP(certSubject);
            fail("This test should fail, but found: " + result);
        } catch (ConnectingToLdapServerFailedException e) {
            // OK case!
        } catch (Exception e) {
            fail("Wrong Exception: " + e.getMessage());
        }
    }

    public void testLdapTestServer() throws Exception {
        LdapSettings setting = new LdapSettings();
        setting.setHost("crtdir.pp.certifikat.dk");
        setting.setPort(389);
        LdapCertificateLookup ldapLookup = new LdapCertificateLookup(setting);

        CertificateSubject certSubject = new CertificateSubject(TestConstants.TEST_CERTIFICATE_SUBJECT_FUNCTION);
        X509Certificate result = ldapLookup.getCertificate(certSubject);
        assertEquals("Wrong cert. found...", BigInteger.valueOf(1077410633), result.getSerialNumber());
        assertEquals("Wrong start date found...", "2011-11-01T07:56:11 UTC", DateUtil.formatDate(result.getNotBefore()));
    }
}