/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation.ocsp;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.configuration.ConfigurationHandlerCleaner;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.ldap.LdapCertificateLookup;
import dk.gov.oiosi.security.ldap.LdapSettings;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * jUnit test class for testing certificate revocation against crl.
 */
public class OcspLookupTest extends TestCase {

    private OcspLookup ocspLookup;

    @Override
    public void setUp() throws Exception {
        super.setUp();
        ConfigurationHandlerCleaner.clear();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
        ocspLookup = new OcspLookup();
    }


    /**
     * Test OCSP for revoked certificate.
     *
     * @throws Exception On error...
     */
    public void testRealCertificateFromSubject_Foces1() throws Exception {
        // Use PROD RASP, to get the real root certificates.
        ConfigurationHandlerCleaner.clear();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_PRODUCTION);
        ocspLookup = new OcspLookup();

        LdapSettings ldapProdSettings = new LdapSettings();
        ldapProdSettings.setHost("crtdir.certifikat.dk");
        ldapProdSettings.setPort(389);
        LdapCertificateLookup ldapCertificateLookup = new LdapCertificateLookup(ldapProdSettings);

        String certSubject = "SERIALNUMBER=CVR:34051178-FID:1359538750995 + CN=DIGST Demo endpoint (funktionscertifikat), O=Digitaliseringsstyrelsen // CVR:34051178, C=DK";
        X509Certificate cert = ldapCertificateLookup.getCertificateFromLDAP(new CertificateSubject(certSubject));
//        System.err.println(cert);
        assertNotNull("Certificate not found", cert);

        RevocationResponse revocationResponse = ocspLookup.revocationResponseOnline(cert);
        assertTrue("Certificate should be OCSP valid...", revocationResponse.isValid());
    }

    /**
     * Test OCSP for revoked certificate.
     *
     * @throws Exception On error...
     */
    public void testRealCertificateFromSubject_Foces2() throws Exception {
        // Use PROD RASP, to get the real root certificates.
        ConfigurationHandlerCleaner.clear();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_PRODUCTION);
        ocspLookup = new OcspLookup();

        LdapSettings ldapProdSettings = new LdapSettings();
        ldapProdSettings.setHost("crtdir.certifikat.dk");
        ldapProdSettings.setPort(389);
        LdapCertificateLookup ldapCertificateLookup = new LdapCertificateLookup(ldapProdSettings);

        String certSubject = "SERIALNUMBER=CVR:32433030-FID:27705483 + CN=mySupply NemHandel - Foces2 (funktionscertifikat), O=mySupply NemHandel // CVR:32433030, C=DK";
        X509Certificate cert = ldapCertificateLookup.getCertificateFromLDAP(new CertificateSubject(certSubject));
        assertNotNull("Certificate not found", cert);

        RevocationResponse revocationResponse = ocspLookup.revocationResponseOnline(cert);
        assertTrue("Certificate should be OCSP valid...", revocationResponse.isValid());
    }

    /**
     * Test crl for revoked certificate.
     *
     * @throws Exception On error...
     */
    public void testRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc_revoked");
        assertNotNull("Certificate not found", cert);

        RevocationResponse response = ocspLookup.checkCertificate(cert);
        assertFalse(response.isValid());
    }

    /**
     * Test crl for non-revoked certificate.
     *
     * @throws Exception On error...
     */
    public void testNonRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc");

        RevocationResponse response = ocspLookup.revocationResponseOnline(cert);
        assertTrue(response.isValid());
    }

    public void testExpiredButNotRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc_expired");

        RevocationResponse response = ocspLookup.checkCertificate(cert);
        assertTrue(response.isValid());
    }

    public void testFoces2NotExpiredNotRevoked() throws Exception {
        X509Certificate cert = getCertificate("tu_generel_foces2");

        RevocationResponse response = ocspLookup.checkCertificate(cert);
        assertTrue(response.isValid());
    }

    public void testFoces2Revoked() throws Exception {
        X509Certificate cert = getCertificate("tu_generel_foces2_revoked");

        RevocationResponse response = ocspLookup.checkCertificate(cert);
        assertFalse(response.isValid());
    }

    public void testFoces2Expired() throws Exception {
        X509Certificate cert = getCertificate("tu_generel_foces2_expired");
        RevocationResponse response = ocspLookup.checkCertificate(cert);

        // NOTE - the certificate is not revoked, even though it is expired
        // So this test should show not revoked
        assertTrue(response.isValid());
    }

    public void testLookTestMultiThread() throws InterruptedException {
        List<Thread> threads = new ArrayList<Thread>();
        for (int i = 0; i < 32; i++) {
            Runnable runnable = new ThreadCertificateCheck();
            Thread thread = new Thread(runnable);
            thread.start();
            threads.add(thread);
        }

        boolean run = true;
        while (run) {
            boolean anyAlive = false;
            for (Thread thread : threads) {
                anyAlive = thread.isAlive();
                if (anyAlive) {
                    break;
                }
            }
            if (anyAlive) {
                Thread.sleep(1000);
                continue;
            }
            run = false;
        }
    }

    class ThreadCertificateCheck implements Runnable {
        public void run() {
            try {
                for (int i = 0; i < 16; i++) {
                    Random random = new Random();
                    int select = random.nextInt(2);
                    if (select < 1) {
//                        System.out.println("{0} ThreadCertificateCheck number:{1} certificate:0");
                        X509Certificate cert = getCertificate("server_tdc");
                        RevocationResponse response = ocspLookup.checkCertificate(cert);
                        assertTrue(response.isValid());
                    } else {
//                        System.out.println("{0} ThreadCertificateCheck number:{1} certificate:1");
                        X509Certificate cert = getCertificate("server_tdc_revoked");
                        RevocationResponse response = ocspLookup.checkCertificate(cert);
                        assertFalse(response.isValid());
                    }
//                    System.out.println("{0} ThreadCertificateCheck number:{1} done");
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    /**
     * Read certificate from keystore.
     *
     * @param keyLabel Key on the certificate to load from JKS.
     * @return X509Certificate
     * @throws Exception On error...
     */
    protected X509Certificate getCertificate(String keyLabel) throws Exception {
        // define receiver certificate
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST), TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

        //Enumeration<String> enumeration = ks.aliases();

        return (X509Certificate) ks.getCertificate(keyLabel);
    }
}