/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.test.client;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.IdentifierCvr;
import dk.gov.oiosi.communication.*;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.NoDocumentTypeFoundFromXmlDocumentException;
import dk.gov.oiosi.xml.schema.SchemaValidationFailedException;
import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;

/**
 * abstract class for running Java/Axis - .Net/WCF interop suite.
 */
public abstract class AbstractTestRaspStackInterop extends TestCase {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(AbstractTestRaspStackInterop.class);

    static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }

    public abstract String getEndpointUrl(String service);

    /**
     * Interop service names.
     */
    public static final String SERVICE_FULL = "InteropFull.svc";
    public static final String SERVICE_WSS = "InteropWSS.svc";
    public static final String SERVICE_RM = "InteropRM.svc";
    public static final String SERVICE_PLAIN = "InteropPlain.svc";

    // add ix (1-5) + ".svc"
    public static final String SERVICE_FULL_CUNCURRENT = "InteropFullConcurrent";

    /**
     * Interop service names - delay services.
     */
    public static final String SERVICE_DELAY_RM = "InteropDelayRM.svc";
    public static final String SERVICE_DELAY_FULL = "InteropDelayFull.svc";

    /**
     * Interop service names - sender faults.
     */
    public static final String SERVICE_SENDER_FAULT_FULL = "InteropSenderFaultFull.svc";
    public static final String SERVICE_SENDER_FAULT_WSS = "InteropSenderFaultWSS.svc";
    public static final String SERVICE_SENDER_FAULT_RM = "InteropSenderFaultRM.svc";
    public static final String SERVICE_SENDER_FAULT_PLAIN = "InteropSenderFaultPlain.svc";

    /**
     * Interop service names - receiver faults.
     */
    public static final String SERVICE_RECEIVER_FAULT_FULL = "InteropReceiverFaultFull.svc";
    public static final String SERVICE_RECEIVER_FAULT_WSS = "InteropReceiverFaultWSS.svc";
    public static final String SERVICE_RECEIVER_FAULT_RM = "InteropReceiverFaultRM.svc";
    public static final String SERVICE_RECEIVER_FAULT_PLAIN = "InteropReceiverFaultPlain.svc";

    /**
     * Interop service names - Rasp document services.
     */
    public static final String SERVICE_DOCUMENT_SERVICE_FULL = "RaspFull.svc";
    public static final String SERVICE_DOCUMENT_SERVICE_WSS = "RaspWSS.svc";
    public static final String SERVICE_DOCUMENT_SERVICE_RM = "RaspRM.svc";
    public static final String SERVICE_DOCUMENT_SERVICE_PLAIN = "RaspPlain.svc";

    /**
     * test documnet.
     */
    private static final String INVOICE_FILE_PATH = "./test/resources/xml/OIOUBL_Invoice_v2p1.xml";

    /**
     * action for sending 2.01 invoice.
     */
    private static final String INVOICE_ACTION = "http://rep.oio.dk/oiosi.ehandel.gov.dk/xml/schemas/2007/09/01/Invoice201Interface/SubmitInvoiceRequest";

    /**
     * INTEROP_ACTION ...
     */
    private static final String INTEROP_ACTION = "http://rep.oio.dk/oiosi.ehandel.gov.dk/xml/schemas/2007/09/01/InteropRequest";

    /**
     * return proxy port.
     *
     * @return proxy port
     */
    protected abstract int getProxy();

    /**
     * Setup Rasp Request.
     *
     * @param endpoint receiver endpoint
     * @param cert     receiver certificate
     * @return request
     * @throws Exception On error...
     */
    protected Request createRequest(String endpoint, X509Certificate cert) throws Exception {
        // Axis setup handled by request - for now maybe policy...
        Credentials credentials = new Credentials(null, new OcesX509Certificate(cert));

        // SendPolicy sendPolicy = new SendPolicy();
        SendPolicy sendPolicy = ConfigurationHandler.getInstance().getSendPolicy();

        if (endpoint.startsWith("http")) {
            sendPolicy.getHttpRMPolicy().setExponentialBackoff(true);
            sendPolicy.getHttpRMPolicy().setRetransmissionInterval(5000);
            sendPolicy.getHttpRMPolicy().setMaximumRetransmissionCount(2);
        } else {
            sendPolicy.getHttpRMPolicy().setExponentialBackoff(false);
            sendPolicy.getHttpRMPolicy().setRetransmissionInterval(30000);
            sendPolicy.getHttpRMPolicy().setMaximumRetransmissionCount(2);
        }

        Request request = new Request(new Uri(endpoint), credentials, sendPolicy);

        if (getProxy() > 0) {
            request.setProxyHost("localhost");
            request.setProxyPort(getProxy());
        }

        return request;
    }

    /**
     * Init message to send.
     *
     * @param addDefaultHeaders Add Default Headers?
     * @return OiosiMessage
     * @throws Exception On error...
     */
    protected OiosiMessage createMessage(boolean addDefaultHeaders) throws Exception {
        return createMessage(null, addDefaultHeaders);
    }

    /**
     * Init message to send.
     *
     * @param file              file to send
     * @param addDefaultHeaders Add Default Headers?
     * @return OiosiMessage
     * @throws Exception On error...
     */
    protected OiosiMessage createMessage(String file, boolean addDefaultHeaders) throws Exception {
        XmlDocument document;
        String action;
        if (file == null) {
            document = new XmlDocument();
            document.load(INVOICE_FILE_PATH);
            action = INTEROP_ACTION;
        } else {
            document = new XmlDocument();
            document.load(file);
            action = INVOICE_ACTION;
        }
        OiosiMessage message = new OiosiMessage(document);
        message.setRequestAction(action);
        // message.setRequestAction( APP_MSG_ACTION );

        if (addDefaultHeaders) {
            message.addMessageHeader(Constants.RASP_MESSAGE_IDENTIFIER, "urn:uuid:" + System.nanoTime());
            message.setReceiverPartyIdentifier(new IdentifierCvr("11111111"));
            message.setSenderPartyIdentifier(new IdentifierCvr("22222222"));
        }
        return message;
    }

    /**
     * Init test document.
     *
     * @return document.
     * @throws Exception
     */
    /* protected OMElement createTestDocument() throws Exception {
        // XMLStreamReader parser =
        // XMLInputFactory.newInstance().createXMLStreamReader( new
        // FileInputStream( INVOICE_TO_SEND ) );
        XMLStreamReader parser = XMLInputFactory.newInstance().createXMLStreamReader(
                new ByteArrayInputStream(NOTIFICATIONXML.getBytes()));
        OMXMLParserWrapper builder = OMXMLBuilderFactory.createStAXOMBuilder(OMAbstractFactory.getOMFactory(), parser);
        OMElement doc = builder.getDocumentElement();
        doc.build();

        return doc;
    }*/

    /**
     * keylabel in keystore.
     *
     * @return keylabel
     */
    protected abstract String getDefaultServerKeylabel();

    /**
     * receiver certificate
     *
     * @param keylabel keylabel in keystore
     * @return certificate
     * @throws Exception On error...
     */
    protected X509Certificate getCertificate(String keylabel) throws Exception {
        // define receiver certificate
        System.out.println("KEYLABEL : " + keylabel);
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST), TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

        return (X509Certificate) ks.getCertificate(keylabel);
    }

    /**
     * Default certificate.
     *
     * @return certificate
     * @throws Exception On error...
     */
    protected X509Certificate getDefaultCertificate() throws Exception {
        // define receiver certificate
        return getCertificate(getDefaultServerKeylabel());
    }

    /*
     * (non-Javadoc)
     *
     * @see junit.framework.TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        Thread.sleep(5000);
    }

    /**
     * test if service is running.
     */
    public void testRaspFullStack_TC_101() {
        System.out.println("Test : testRaspFullStack_TC_101() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_FULL), getDefaultCertificate());
            OiosiMessage message = createMessage(true);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    // Timeout tests...

    /**
     * timeout on payload request. replay of message should solve problem.
     */
    public void testTimeout_RequestTimeoutRM() {
        long raspInteropDelay = 30;
        System.out.println("Test : testTimeout_request() : service sleep = " + raspInteropDelay);
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DELAY_RM), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            message.addMessageHeader("RaspInteropDelay", "" + raspInteropDelay);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * timeout on RM session.
     */
    public void testTimeout_ClientSessionTimoutRM() {
        long raspInteropDelay = 300;
        System.out.println("Test : testTimeout_session() : service sleep = " + raspInteropDelay);
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DELAY_RM), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            message.addMessageHeader("RaspInteropDelay", "" + raspInteropDelay);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            try {
                Thread.sleep(15000);
            } catch (Exception ignore) {
            }
            assertTrue(true);
        }
    }

    /**
     * timeout on payload request. replay of message should solve problem.
     */
    public void testTimeout_RequestTimeoutFull() {
        long raspInteropDelay = 30;
        System.out.println("Test : testTimeout_request() : service sleep = " + raspInteropDelay);
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DELAY_FULL), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            message.addMessageHeader("RaspInteropDelay", "" + raspInteropDelay);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * timeout on RM session.
     */
    public void testTimeout_ClientSessionTimoutFull() {
        long raspInteropDelay = 300;
        System.out.println("Test : testTimeout_session() : service sleep = " + raspInteropDelay);
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DELAY_FULL), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            message.addMessageHeader("RaspInteropDelay", "" + raspInteropDelay);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            try {
                Thread.sleep(15000);
            } catch (Exception ignore) {
            }
            assertTrue(true);
        }
    }

    // Complete to Incomplete

    /**
     * Client FULL contacts Service FULL.
     */
    public void testFromComplete_FULL_TO_FULL() {
        System.out.println("Test : testFromComplete_FULL_TO_FULL() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_FULL), getDefaultCertificate());
            OiosiMessage message = createMessage(true);
            Response response = request.getResponse(message);
            assertNotNull("No signature proof was generated", response.getResponseMessage().getSignatureProof());
            assertNotNull(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * Client FULL contacts Service FULL - specific server for client concurrent test.
     */
    public void testFromComplete_FULL_TO_FULL(int ix) {
        System.out.println("Test : testFromComplete_FULL_TO_FULL() ");
        try {
            String serviceUrl = getEndpointUrl(SERVICE_FULL_CUNCURRENT) + ix + ".svc";
            Request request = createRequest(serviceUrl, getCertificate("server_" + ix));
            OiosiMessage message = createMessage(true);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * Client FULL contacts Service RM.
     */
    public void testFromComplete_FULL_TO_RM() {
        System.out.println("Test : testFromComplete_FULL_TO_RM() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_RM), getDefaultCertificate());
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client FULL contacts Service WSS.
     */
    public void testFromComplete_FULL_TO_WSS() {
        System.out.println("Test : testFromComplete_FULL_TO_WSS() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_WSS), getDefaultCertificate());
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client FULL contacts Service PLAIN.
     */
    public void testFromComplete_FULL_TO_PLAIN() {
        System.out.println("Test : testFromComplete_FULL_TO_PLAIN() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_PLAIN), getDefaultCertificate());
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    // Incomplete to Full Stack

    /**
     * Client WSS contacts Service FULL.
     */
    public void testFromIncomplete_WSS_TO_FULL() {
        System.out.println("Test : testFromIncomplete_WSS_TO_FULL() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_FULL), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client WSS contacts Service RM.
     */
    public void testFromIncomplete_WSS_TO_RM() {
        System.out.println("Test : testFromIncomplete_WSS_TO_RM() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_RM), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client WSS contacts Service WSS.
     */
    public void testFromIncomplete_WSS_TO_WSS() {
        System.out.println("Test : testFromIncomplete_WSS_TO_WSS() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_WSS), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * Client WSS contacts Service PLAIN.
     */
    public void testFromIncomplete_WSS_TO_PLAIN() {
        System.out.println("Test : testFromIncomplete_WSS_TO_PLAIN() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_PLAIN), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    // plain to full ok, plain to wsrm ok, plain to wss ok, plain to plain ok.

    /**
     * Client PLAIN contacts Service FULL.
     */
    public void testFromIncomplete_PLAIN_TO_FULL() {
        System.out.println("Test : testFromIncomplete_PLAIN_TO_FULL() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_FULL), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client PLAIN contacts Service RM.
     */
    public void testFromIncomplete_PLAIN_TO_RM() {
        System.out.println("Test : testFromIncomplete_PLAIN_TO_RM() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_RM), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client PLAIN contacts Service WSS.
     */
    public void testFromIncomplete_PLAIN_TO_WSS() {
        System.out.println("Test : testFromIncomplete_PLAIN_TO_WSS() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_WSS), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client PLAIN contacts Service PLAIN.
     */
    public void testFromIncomplete_PLAIN_TO_PLAIN() {
        System.out.println("Test : testFromIncomplete_PLAIN_TO_PLAIN() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_PLAIN), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    // rm to full, rm to rm, rm to wss, rm to plain

    /**
     * Client RM contacts Service FULL.
     */
    public void testFromIncomplete_RM_TO_FULL() {
        System.out.println("Test : testFromIncomplete_RM_TO_FULL() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_FULL), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client RM contacts Service RM.
     */
    public void testFromIncomplete_RM_TO_RM() {
        System.out.println("Test : testFromIncomplete_RM_TO_RM() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_RM), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * Client RM contacts Service WSS.
     */
    public void testFromIncomplete_RM_TO_WSS() {
        System.out.println("Test : testFromIncomplete_RM_TO_WSS() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_WSS), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    /**
     * Client RM contacts Service PLAIN.
     */
    public void testFromIncomplete_RM_TO_PLAIN() {
        System.out.println("Test : testFromIncomplete_RM_TO_PLAIN() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_PLAIN), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (Exception e) {
            assertTrue(true);
        }
    }

    // Sender Faults

    /**
     * Sender Fault PLAIN.
     */
    public void testSenderFault_PLAIN() {
        System.out.println("Test : testSenderFault_PLAIN() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_SENDER_FAULT_PLAIN), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (MainException e) {
            if (!"InteropSenderFault".equals(e.getSoapFaultAction())) {
                fail("SoapFault Subcode should be : InteropSenderFault");
            }
            if (!"Sender".equals(e.getSoapFaultCode())) {
                fail("SoapFault Code should be : Sender");
            }
            assertTrue(true);
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Sender Fault RM.
     */
    public void testSenderFault_RM() {
        System.out.println("Test : testSenderFault_RM() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_SENDER_FAULT_RM), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (MainException e) {
            if (!"InteropSenderFault".equals(e.getSoapFaultAction())) {
                fail("SoapFault Subcode should be : InteropSenderFault");
            }
            if (!"Sender".equals(e.getSoapFaultCode())) {
                fail("SoapFault Code should be : Sender");
            }
            assertTrue(true);
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Sender Fault WSS.
     */
    public void testSenderFault_WSS() {
        System.out.println("Test : testSenderFault_WSS() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_SENDER_FAULT_WSS), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (MainException e) {
            if (!"InteropSenderFault".equals(e.getSoapFaultAction())) {
                fail("SoapFault Subcode should be : InteropSenderFault");
            }
            if (!"Sender".equals(e.getSoapFaultCode())) {
                fail("SoapFault Code should be : Sender");
            }
            assertTrue(true);
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Sender Fault WSS.
     */
    public void testSenderFault_FULL() {
        System.out.println("Test : testSenderFault_FULL() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_SENDER_FAULT_FULL), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (MainException e) {
            if (!"InteropSenderFault".equals(e.getSoapFaultAction())) {
                fail("SoapFault Subcode should be : InteropSenderFault");
            }
            if (!"Sender".equals(e.getSoapFaultCode())) {
                fail("SoapFault Code should be : Sender");
            }
            assertTrue(true);
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    // Receiver Faults

    /**
     * Receiver Fault PLAIN.
     */
    public void testReceiverFault_PLAIN() {
        System.out.println("Test : testReceiverFault_PLAIN() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_RECEIVER_FAULT_PLAIN), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (MainException e) {
            if (!"InteropReceiverFault".equals(e.getSoapFaultAction())) {
                fail("SoapFault Subcode should be : InteropReceiverFault");
            }
            if (!"Receiver".equals(e.getSoapFaultCode())) {
                fail("SoapFault Code should be : Receiver");
            }
            assertTrue(true);
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Receiver Fault RM.
     */
    public void testReceiverFault_RM() {
        System.out.println("Test : testReceiverFault_RM() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_RECEIVER_FAULT_RM), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (MainException e) {
            if (!"InteropReceiverFault".equals(e.getSoapFaultAction())) {
                fail("SoapFault Subcode should be : InteropReceiverFault");
            }
            if (!"Receiver".equals(e.getSoapFaultCode())) {
                fail("SoapFault Code should be : Receiver");
            }
            assertTrue(true);
        } catch (Exception e) {
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Receiver Fault WSS.
     */
    public void testReceiverFault_WSS() {
        System.out.println("Test : testReceiverFault_WSS() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_RECEIVER_FAULT_WSS), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(false);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (MainException e) {
            if (!"InteropReceiverFault".equals(e.getSoapFaultAction())) {
                fail("SoapFault Subcode should be : InteropReceiverFault");
            }
            if (!"Receiver".equals(e.getSoapFaultCode())) {
                fail("SoapFault Code should be : Receiver");
            }
            assertTrue(true);
        } catch (Exception e) {
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Receiver Fault WSS.
     */
    public void testReceiverFault_FULL() {
        System.out.println("Test : testReceiverFault_FULL() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_RECEIVER_FAULT_FULL), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(true);
            OiosiMessage message = createMessage(true);
            request.getResponse(message);
            fail("A SOAP Fault was expected in return");
        } catch (MainException e) {
            if (!"InteropReceiverFault".equals(e.getSoapFaultAction())) {
                fail("SoapFault Subcode should be : InteropReceiverFault");
            }
            if (!"Receiver".equals(e.getSoapFaultCode())) {
                fail("SoapFault Code should be : Receiver");
            }
            assertTrue(true);
        } catch (Exception e) {
            fail("This test should fail with RaspException..");
        }
    }


    // Custom header tests..

    /**
     * Send with missing mandatory header SenderPartyIdentifier.
     */
    public void testCustomHeader_missingSenderPartyIdentifier() {
        System.out.println("Test : testCustomHeader_missingSenderPartyIdentifier() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_FULL), getDefaultCertificate());
            OiosiMessage message = createMessage(false);
            message.addMessageHeader(Constants.RASP_MESSAGE_IDENTIFIER, "urn:uuid:" + System.nanoTime());
            message.setReceiverPartyIdentifier(new IdentifierCvr("11111111"));
            request.getResponse(message);
            fail("Service did not validate missing custom header : " + Constants.RASP_SENDER_PARTY_IDENTIFIER);
        } catch (FaultReturnedException e) {
            log.info("Failed as expected : " + e + " : " + e.getSoapFaultAction());
            assertTrue(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * Send with missing mandatory header ReceiverPartyIdentifier.
     */
    public void testCustomHeader_missingReceiverPartyIdentifier() {
        System.out.println("Test : testCustomHeader_missingReceiverPartyIdentifier() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_FULL), getDefaultCertificate());
            OiosiMessage message = createMessage(false);
            message.addMessageHeader(Constants.RASP_MESSAGE_IDENTIFIER, "urn:uuid:" + System.nanoTime());
            message.setSenderPartyIdentifier(new IdentifierCvr("22222222"));
            request.getResponse(message);
            fail("Service did not validate missing custom header : " + Constants.RASP_SENDER_PARTY_IDENTIFIER);
        } catch (FaultReturnedException e) {
            log.info("Failed as expected : " + e + " : " + e.getSoapFaultAction());
            assertTrue(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * Send with missing mandatory header MessageIdentifier.
     */
    public void testCustomHeader_missingMessageIdentifier() {
        System.out.println("Test : testCustomHeader_missingMessageIdentifier() ");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_FULL), getDefaultCertificate());
            OiosiMessage message = createMessage(false);
            message.setReceiverPartyIdentifier(new IdentifierCvr("11111111"));
            message.setSenderPartyIdentifier(new IdentifierCvr("22222222"));
            request.getResponse(message);
            fail("Service did not validate missing custom header : " + Constants.RASP_SENDER_PARTY_IDENTIFIER);
        } catch (FaultReturnedException e) {
            log.info("Failed as expected : " + e + " : " + e.getSoapFaultAction());
            assertTrue(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }


    // Documnet tests..

    /**
     * Rasp Document Invoice OK PLAIN.
     */
    public void testRaspDocument_UnknownDocument_PLAIN() {
        System.out.println("Test : testRaspDocument_InvoceOK_PLAIN()");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DOCUMENT_SERVICE_PLAIN), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(TestConstants.PATH_INVOICE_WRONG_NAME_SPACE, true);
            request.getResponse(message);
            log.error("Test should not fail : ");
            fail("Sending Document should not fail " + TestConstants.PATH_INVOICE_WRONG_NAME_SPACE);
        } catch (FaultReturnedException e) {
            if (NoDocumentTypeFoundFromXmlDocumentException.FAULT_ACTION.equals(e.getSoapFaultAction())) {
                log.debug("Test OK : Exception == NoDocumentTypeFoundFromXmlDocumentException: " + e, e);
                assertTrue(true);
            } else {
                log.debug("Wrong SoapFault sub code - expected : " + NoDocumentTypeFoundFromXmlDocumentException.FAULT_ACTION
                        + " - got : " + e.getSoapFaultAction() + " - exception : " + e, e);
                assertTrue("Wrong SoapFault sub code - expected : " + NoDocumentTypeFoundFromXmlDocumentException.FAULT_ACTION
                        + " - got : " + e.getSoapFaultAction(), false);
            }
        } catch (MainException e) {
            log.error("Wrong fault : " + e.getSoapFaultAction());
            log.error("Wrong fault :" + e, e);
            fail("Failed with wrong rault " + e.getSoapFaultAction());
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Rasp Document Invoice OK PLAIN.
     */
    public void testRaspDocument_InvoiceOK_PLAIN() {
        System.out.println("Test : testRaspDocument_InvoiceOK_PLAIN()");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DOCUMENT_SERVICE_PLAIN), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(TestConstants.PATH_INVOICE, true);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (MainException e) {
            log.error("Test should not fail : " + e, e);
            fail("Sending Document should not fail " + TestConstants.PATH_INVOICE);
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Rasp Document Invoice OK RM.
     */
    public void testRaspDocument_InvoiceOK_RM() {
        System.out.println("Test : testRaspDocument_InvoiceOK_RM()");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DOCUMENT_SERVICE_RM), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(TestConstants.PATH_INVOICE, true);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (MainException e) {
            log.error("Test should not fail : " + e, e);
            fail("Sending Document should not fail " + TestConstants.PATH_INVOICE);
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Rasp Document Invoice OK WSS.
     */
    public void testRaspDocument_InvoiceOK_WSS() {
        System.out.println("Test : testRaspDocument_InvoiceOK_WSS()");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DOCUMENT_SERVICE_WSS), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(false);
            OiosiMessage message = createMessage(TestConstants.PATH_INVOICE, true);
            Response response = request.getResponse(message);
            assertNotNull(response);
        } catch (MainException e) {
            log.error("Test should not fail : " + e, e);
            fail("Sending Document should not fail " + TestConstants.PATH_INVOICE);
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Rasp Document Invoice SchemaFault PLAIN.
     */
    public void testRaspDocument_InvoiceSchemaFault_PLAIN() {
        System.out.println("Test : testRaspDocument_InvoiceSchemaFault_PLAIN()");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DOCUMENT_SERVICE_PLAIN), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(false);
            OiosiMessage message = createMessage(TestConstants.PATH_INVOICE_WRONG_ELEMENT, true);
            request.getResponse(message);
            fail("Sending Document should not fail " + TestConstants.PATH_INVOICE_WRONG_ELEMENT);
        } catch (FaultReturnedException e) {
            if (SchemaValidationFailedException.FAULT_ACTION.equals(e.getSoapFaultAction())) {
                log.debug("Test OK : Exception == SchemaValidationFailedException: " + e, e);
                assertTrue(true);
            } else {
                log.error("Wrong SoapFault sub code - expected : " + SchemaValidationFailedException.FAULT_ACTION + " - got : "
                        + e.getSoapFaultAction() + " - exception : " + e, e);
                assertTrue("Wrong SoapFault sub code - expected : " + SchemaValidationFailedException.FAULT_ACTION + " - got : "
                        + e.getSoapFaultAction(), false);
            }
        } catch (MainException e) {
            log.error("Wrong fault : " + e.getSoapFaultAction());
            log.error("Wrong fault :" + e, e);
            fail("Failed with wrong rault " + e.getSoapFaultAction());
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Rasp Document Invoice SchemaFault RM.
     */
    public void testRaspDocument_InvoiceSchemaFault_RM() {
        System.out.println("Test : testRaspDocument_InvoiceSchemaFault_RM()");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DOCUMENT_SERVICE_RM), getDefaultCertificate());
            request.setUseWSS(false);
            request.setUseRM(true);
            OiosiMessage message = createMessage(TestConstants.PATH_INVOICE_WRONG_ELEMENT, true);
            request.getResponse(message);
            fail("Sending Document should not fail " + TestConstants.PATH_INVOICE_WRONG_ELEMENT);
        } catch (FaultReturnedException e) {
            if (SchemaValidationFailedException.FAULT_ACTION.equals(e.getSoapFaultAction())) {
                log.debug("Test OK : Exception == SchemaValidationFailedException: " + e, e);
                assertTrue(true);
            } else {
                log.error("Wrong SoapFault sub code - expected : " + SchemaValidationFailedException.FAULT_ACTION + " - got : "
                        + e.getSoapFaultAction() + " - exception : " + e, e);
                assertTrue("Wrong SoapFault sub code - expected : " + SchemaValidationFailedException.FAULT_ACTION + " - got : "
                        + e.getSoapFaultAction(), false);
            }
        } catch (MainException e) {
            log.error("Wrong fault : " + e.getSoapFaultAction());
            log.error("Wrong fault :" + e, e);
            fail("Failed with wrong rault " + e.getSoapFaultAction());
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Rasp Document Invoice SchemaFault WSS.
     */
    public void testRaspDocument_InvoiceSchemaFault_WSS() {
        System.out.println("Test : testRaspDocument_InvoiceSchemaFault_WSS()");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DOCUMENT_SERVICE_WSS), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(false);
            OiosiMessage message = createMessage(TestConstants.PATH_INVOICE_WRONG_ELEMENT, true);
            request.getResponse(message);
            fail("Sending Document should not fail " + TestConstants.PATH_INVOICE_WRONG_ELEMENT);
        } catch (FaultReturnedException e) {
            if (SchemaValidationFailedException.FAULT_ACTION.equals(e.getSoapFaultAction())) {
                log.debug("Test OK : Exception == SchemaValidationFailedException: " + e, e);
                assertTrue(true);
            } else {
                log.error("Wrong SoapFault sub code - expected : " + SchemaValidationFailedException.FAULT_ACTION + " - got : "
                        + e.getSoapFaultAction() + " - exception : " + e, e);
                assertTrue("Wrong SoapFault sub code - expected : " + SchemaValidationFailedException.FAULT_ACTION + " - got : "
                        + e.getSoapFaultAction(), false);
            }
        } catch (MainException e) {
            log.error("Wrong fault : " + e.getSoapFaultAction());
            log.error("Wrong fault :" + e, e);
            fail("Failed with wrong rault " + e.getSoapFaultAction());
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    /**
     * Rasp Document Invoice SchemaFault FULL.
     */
    public void testRaspDocument_InvoiceSchemaFault_FULL() {
        System.out.println("Test : testRaspDocument_InvoiceSchemaFault_FULL()");
        try {
            Request request = createRequest(getEndpointUrl(SERVICE_DOCUMENT_SERVICE_FULL), getDefaultCertificate());
            request.setUseWSS(true);
            request.setUseRM(true);
            OiosiMessage message = createMessage(TestConstants.PATH_INVOICE_WRONG_ELEMENT, true);
            request.getResponse(message);
            fail("Sending Document should not fail " + TestConstants.PATH_INVOICE_WRONG_ELEMENT);
        } catch (FaultReturnedException e) {
            if (SchemaValidationFailedException.FAULT_ACTION.equals(e.getSoapFaultAction())) {
                log.error("Test OK : Exception == SchemaValidationFailedException: " + e, e);
                assertTrue(true);
            } else {
                log.error("Wrong SoapFault sub code - expected : " + SchemaValidationFailedException.FAULT_ACTION + " - got : "
                        + e.getSoapFaultAction() + " - exception : " + e, e);
                assertTrue("Wrong SoapFault sub code - expected : " + SchemaValidationFailedException.FAULT_ACTION + " - got : "
                        + e.getSoapFaultAction(), false);
            }
        } catch (MainException e) {
            log.error("Wrong fault : " + e.getSoapFaultAction());
            log.error("Wrong fault :" + e, e);
            fail("Failed with wrong rault " + e.getSoapFaultAction());
        } catch (Exception e) {
            log.error("THIS SHOULD BE MAIN EXCEPTION : " + e, e);
            fail("This test should fail with RaspException..");
        }
    }

    public class RunCallback {
        boolean failed = false;

        private int count = 0;

        private int id = 0;

        public int invoke = 0;

        public int failure = 0;

        public int start() {
            count++;
            return (++id);
        }

        public void stop() {
            count--;
        }

        public int getCount() {
            return count;
        }

        public void invoke() {
            invoke++;
        }

        public int getInvoke() {
            return invoke;
        }
    }

    public class RunSequence implements Runnable {
        RunCallback callback;

        int id = 0;

        int iterations = 0;

        boolean rm = false;

        boolean wss = false;

        int serverIx = -1;

        public RunSequence(RunCallback c, int i, boolean rm, boolean wss, int serverIx) {
            this.callback = c;
            this.iterations = i;
            this.rm = rm;
            this.wss = wss;
            this.serverIx = serverIx;

            id = callback.start();
        }

        public void run() {
            for (int i = 0; i < iterations; i++) {
                try {
                    System.out.println("TEST id : " + id + " - iteration " + i);
                    if (rm) {
                        if (wss) {
                            testFromComplete_FULL_TO_FULL(serverIx);
                        } else {
                            testFromIncomplete_RM_TO_RM();
                        }
                    } else {
                        if (wss) {
                            testFromIncomplete_WSS_TO_WSS();
                        } else {
                            testFromIncomplete_PLAIN_TO_PLAIN();
                        }
                    }
                    callback.invoke();
                } catch (Throwable e) {
                    callback.failed = true;
                    callback.failure++;
                }
            }
            callback.stop();
        }
    }

    public void runThreads(int threads, int iterations, boolean rm, boolean wss) {
        long start = System.currentTimeMillis();
        RunCallback callback = new RunCallback();
        try {
            Thread.sleep(1000);

            //TODO: This is a temporary hack to make tests run quicker:
            iterations = 1;
            for (int i = 0; i < threads; i++) {
                Thread.sleep(100);
                new Thread(new RunSequence(callback, iterations, rm, wss, i + 1)).start();
            }
            Thread.sleep(1000);
            while (callback.getCount() > 0) {
                System.out.println("Check Loop - runners : " + callback.getCount() + " - invoke : " + callback.getInvoke() + " : "
                        + callback.failure);
                Thread.sleep(1000);
            }
            System.out.println("End : " + callback.getInvoke() + " - Failure " + callback.failure);
        } catch (Exception e) {
            System.out.println("FAILED : " + e);
            e.printStackTrace();
        }
        long stop = System.currentTimeMillis();
        System.out.println("Time : " + (stop - start));
        if (!callback.failed) {
            assertTrue(true);
        } else {
            fail("Something Failed");
        }
    }

    public void testInterop_5x10_Plain() {
        System.out.println("Test : testInterop_10x10_Plain()");
        runThreads(5, 10, false, false);
    }

    public void testInterop_5x10_RM() {
        System.out.println("Test : testInterop_10x10_RM()");
        runThreads(5, 10, true, false);
    }

    public void testInterop_5x10_WSS() {
        System.out.println("Test : testInterop_10x10_WSS()");
        runThreads(5, 10, false, true);
    }

    public void testInterop_5x10_Full() {
        System.out.println("Test : testInterop_10x10_Full()");
        runThreads(5, 10, true, true);
    }

    public abstract void localCleanup() throws Exception;

    public void testCleanup() {
        try {
            localCleanup();
            assertTrue(true);
        } catch (Exception e) {
            fail("Failed to cleanup " + this.getClass().getCanonicalName());
        }
    }
}