/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.test.integration.security.revocation;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.security.revocation.ocsp.OcspConfig;
import dk.gov.oiosi.security.revocation.ocsp.OcspLookup;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.FileInputStream;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

/**
 * Unit test class for testing certificate against OCSP server.
 */
public class OcspRequestTest extends TestCase {

    private Log log = LogFactory.getLog(OcspRequestTest.class);

    static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
    }

    /**
     * Test crl for revoked certificate.
     *
     * @throws Exception On error...
     */
    public void testRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc_revoked");
        IRevocationLookup crlLookup = new OcspLookup();
        RevocationResponse response = crlLookup.checkCertificate(cert);
        assertFalse("Cert. should be invalid, but is not...", response.isValid());
    }

    /**
     * Test crl for non-revoked certificate.
     *
     * @throws Exception On error...
     */
    public void testNonRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc");

        IRevocationLookup crlLookup = new OcspLookup();
        RevocationResponse response = crlLookup.checkCertificate(cert);

        assertTrue(response.isValid());
    }

    public void testExpiredButNotRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc_expired");

        IRevocationLookup crlLookup = new OcspLookup();
        RevocationResponse response = crlLookup.checkCertificate(cert);

        assertTrue(response.isValid());
    }

    public void testNonRevokedWithOcspServerFromCertificate() throws Exception {
        OcspConfig ocspConfig = new OcspConfig();
        ocspConfig.setDefaultTimeoutMsec(10000);
        OcspLookup ocspLookup = new OcspLookup(ocspConfig);

        X509Certificate cert = getCertificate("server_tdc");
        RevocationResponse response = ocspLookup.checkCertificate(cert);
        assertTrue(response.isValid());
    }

    public void testRevokedWithOcspServerFromCertificate() throws Exception {
        OcspConfig ocspConfig = new OcspConfig();
        ocspConfig.setDefaultTimeoutMsec(10000);
        OcspLookup ocspLookup = new OcspLookup(ocspConfig);

        X509Certificate cert = getCertificate("server_tdc_revoked");
        RevocationResponse response = ocspLookup.checkCertificate(cert);
        assertFalse("This certificate should have been catched as revoked", response.isValid());
    }

    public void testNonRevokedWithOcspServerFromConfig() throws Exception {
        OcspConfig ocspConfig = new OcspConfig();
        ocspConfig.setDefaultTimeoutMsec(10000);
        ocspConfig.setServerUrl("http://test.ocsp.certifikat.dk/ocsp/status");
        OcspLookup ocspLookup = new OcspLookup(ocspConfig);
        X509Certificate cert = getCertificate("server_tdc");
        RevocationResponse response = ocspLookup.checkCertificate(cert);
        assertTrue(response.isValid());
    }

    public void testRevokedWithOcspServerFromConfig() throws Exception {
        OcspConfig ocspConfig = new OcspConfig();
        ocspConfig.setDefaultTimeoutMsec(10000);
        ocspConfig.setServerUrl("http://test.ocsp.certifikat.dk/ocsp/status");
        OcspLookup ocspLookup = new OcspLookup(ocspConfig);
        X509Certificate cert = getCertificate("server_tdc_revoked");
        RevocationResponse response = ocspLookup.checkCertificate(cert);
        assertFalse("This certificate should have been marked as revoked", response.isValid());
    }


    public void testGetServerUriFromCertificateWithoutRootCert() throws Exception {
        OcspConfig ocspConfig = new OcspConfig();
        ocspConfig.setDefaultTimeoutMsec(10000);

        OcspLookup ocspLookup = new OcspLookup(ocspConfig);

        X509Certificate cert = getCertificate("client_person");
        log.info(ocspLookup.GetServerUriFromCertificate(cert));
        assertEquals("http://test.ocsp.certifikat.dk/ocsp/status", ocspLookup.GetServerUriFromCertificate(cert));
    }

    public void testGetServerUriFromCertificateWithRootCert() throws Exception {
        OcspConfig ocspConfig = new OcspConfig();
        ocspConfig.setDefaultTimeoutMsec(10000);
        InputStream inStream = new FileInputStream(TestConstants.PATH_CERTIFICATE_ROOT);
        CertificateFactory cf = CertificateFactory.getInstance("X.509");
        X509Certificate rootCert = (X509Certificate) cf.generateCertificate(inStream);

        OcspLookup ocspLookup = new OcspLookup(ocspConfig, rootCert);

        X509Certificate cert = getCertificate("client_person");
        log.info(ocspLookup.GetServerUriFromCertificate(cert));
        assertEquals("http://test.ocsp.certifikat.dk/ocsp/status", ocspLookup.GetServerUriFromCertificate(cert));
    }

//    public void testLiveOcspLookup() throws Exception {
//        OcspConfig ocspConfig = new OcspConfig();
//        ocspConfig.setDefaultTimeoutMsec(10000);
//        ocspConfig.setServerUrl("http://ocsp.certifikat.dk/ocsp/status");
//        OcspLookup ocspLookup = new OcspLookup(ocspConfig);
//        X509Certificate cert = getCertificate("server_tdc");
//        RevocationResponse response = ocspLookup.checkCertificate(cert);
//        assertTrue(response.isValid());
//    }


    /**
     * Read certificate from keystore.
     *
     * @param keylabel keylabel
     * @return certificate
     * @throws Exception On error...
     */
    protected X509Certificate getCertificate(String keylabel) throws Exception {
        // define receiver certificate
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST), TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

        return (X509Certificate) ks.getCertificate(keylabel);
    }
}