/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.test.service;

import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schematron.SchematronValidator;
import org.apache.axiom.om.OMElement;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.security.cert.X509Certificate;

/**
 * Service which receives real OIO/UBL documents.
 */
public class OiosiDocumentService extends OiosiService {

    /**
     * local log.
     */
    private static final Log log = LogFactory.getLog(OiosiService.class);

    /* (non-Javadoc)
     * @see dk.gov.oiosi.test.service.OiosiService#requestRespond(dk.gov.oiosi.communication.OiosiMessage)
     */
    public OiosiMessage requestRespond(OiosiMessage message) throws MainException {
        OMElement document = message.getMessageXml().getOMDocument();
        String messageId = message.getMessageIdentifier();

        invokeCounter++;

        String msg = "Invokecount : total/confirm/timeout/cancel/abort = " + invokeCounter + "/" + confirmCounter + "/"
                + timeoutCounter + "/" + faultedCounter + " : " + messageId;
        String signingMessage = "Message was not signed ";
        X509Certificate cert = message.getSignerCertificate();
        if (cert != null) {
            signingMessage = "Message was signed with certificate : " + cert.getSubjectDN();
        }

        if (document == null) {
            log.debug("Received empty document : " + msg);
        } else {
            log.debug("Received document - try to detect : " + msg);
        }
        log.debug(" - " + signingMessage);

        // TODO: Why do we have this?
        boolean strangeCondition = true;
        if (strangeCondition) {
            log.debug("ALWAYS - RETURN NULL AS REPLY");

            return null;
        }

        if (log.isDebugEnabled() && message.getMessageHeaders() != null) {
            log.debug(" - service received custom headers ");
            for (String key : message.getMessageHeaders().keySet()) {
                log.debug(" - CustomKey : " + key + " : " + message.getMessageHeaders().get(key));
            }
        } else {
            log.debug(" - service received NO custom headers ");
        }

        if (document == null) {
            log.debug(" - RETURN NULL AS REPLY");

            return null;
        }

        // Find Document
        DocumentTypeConfig documentTypeConfig = null;
        try {
            log.debug("Find DocumentTypeConfig " + document.getLocalName());
            DocumentTypeConfigSearcher documentSearcher = new DocumentTypeConfigSearcher();
            documentTypeConfig = documentSearcher.findUniqueDocumentType(message.getMessageXml());
        } catch (MainException e) {
            throw e;
        } catch (Exception e) {
            log.error("Failed to find DocumentTypeConfig : " + e, e);
            throw new MainException("Failed to find DocumentTypeConfig ", e);
        }

        // Schema Check document
        try {
            log.debug("SchemaValidate Document : " + documentTypeConfig.getSchemaPath());
            SchemaValidator validator = new SchemaValidator();

            // FileStream stream = File.OpenRead(xmlSchemaPath);
            XmlSchema schema = new XmlSchema(documentTypeConfig.getSchemaPath());

            validator.schemaValidateXmlDocument(message.getMessageXml(), schema);
            log.debug("- Schema validate OK ");
        } catch (MainException e) {
            log.error("Failed To Validate Schema: " + e.getMessage(), e);
            throw e;
        } catch (Exception e) {
            log.error("Failed To Validate Schema: " + e.getMessage(), e);
            throw new MainException("Failed To Validate Schema ", e);
        }

        // Schematron Check document
        try {
            log.debug("Schematron Validate Document : " + documentTypeConfig.getSchemaPath());
            if (documentTypeConfig.getSchematronValidationConfig() == null) {
                // System.out.println(" - skip " );
            } else {
                log.debug(" - doit ");
                SchematronValidator validator = new SchematronValidator(documentTypeConfig.getSchematronValidationConfig());

                validator.schematronValidateXmlDocument(message.getMessageXml());
                log.debug("- Schematron validate OK ");
            }
        } catch (MainException e) {
            log.error("Failed To Schematron Validate document: " + e.getMessage(), e);
            throw e;
        } catch (Exception e) {
            log.error("Failed To Schematron Validate document: " + e.getMessage(), e);
            throw new MainException("Failed To Schematron Validate document", e);
        }

        log.debug("Service handled message OK ");
        return null;
    }
}