package dk.gov.oiosi.test.unit.common.cache;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.ICache;
import dk.gov.oiosi.common.cache.LeastFrequentlyUsedQuantityCache;
import dk.gov.oiosi.common.cache.LeastRecentlyUsedQuantityCache;
import dk.gov.oiosi.common.cache.MultipleQuantityCache;

import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/* 
 * dlk, 2012-03-22
 * 
 * By overriding existing test for QuantityCache, this test shows that it has the same functionality.
 */
public class LeastRecentlyUsedQuantityCacheTest extends QuantityCacheTest {

    private static final int CACHE_SIZE = 2;
    private static final int JOB_COUNT = 100;
    private static final int THREAD_POOL_SIZE = 4;
    private static final int TASK_ITERATION_COUNT = 10;

    @Override
    protected ICache<String, String> buildCache(int maxSize) {
        /*
           * LRUQuantityCache
           */
        return new LeastRecentlyUsedQuantityCache<String, String>(buildConfigMap(maxSize));
    }

    protected ICache<String, String> buildCache2(int maxSize) {
        /*
           * LFUQuantityCache
           */
        return new LeastFrequentlyUsedQuantityCache<String, String>(buildConfigMap(maxSize));
    }

//    protected ICache<String, String> buildCache3(int maxSize) {
//        Map<String, String> configMap = buildConfigMap(maxSize);
//        configMap.put(MultipleQuantityCache.CONFIG_MAX_EQUAL_KEY_SIZE, String.valueOf(10));
//        return new MultipleQuantityCache<String, String>(configMap);
//    }

    protected Map<String, String> buildConfigMap(int maxSize) {
        Map<String, String> config = new HashMap<String, String>();
        config.put(MultipleQuantityCache.CONFIG_MAX_SIZE, String.valueOf(maxSize));
        return config;
    }

    /*
      * Additional unit tests to check thread safety and logic
      */

    public void testThreadSafety() throws Exception {
        _cache = buildCache2(CACHE_SIZE);

        ExecutorService pool = Executors.newFixedThreadPool(THREAD_POOL_SIZE);

        List<Callable<CacheResult>> taskList = new ArrayList<Callable<CacheResult>>();
        for (int i = 0; i < JOB_COUNT; i++) {
            /*
                * All cache entries are equally possible
                */
//            int keyValue = i % 10;
            /*
                * Cache entry 0 - 50% probability, 1 - 25%, 2 - 12%, 3 - 6% etc until 7.
                *
                * {1=50, 2=25, 3=13, 4=6, 5=3, 6=2, 7=1}
                */
            int keyValue = fractionPart(i, JOB_COUNT);
            TestCacheClient task = new TestCacheClient("key" + keyValue);
            taskList.add(task);
        }
        List<Future<CacheResult>> futureList = pool.invokeAll(taskList);

        pool.shutdown();

        // Wait until all threads are finish
        while (!pool.isTerminated()) {
            System.out.print(".");
        }
        System.out.println();

        /*
           * Analyse cache results
           */

        Map<String, CacheResult> totalCacheMatch = new HashMap<String, CacheResult>();
        for (Future<CacheResult> future : futureList) {
            CacheResult cr = future.get();
            System.out.println(cr);
            CacheResult total = totalCacheMatch.get(cr.key);
            if (total == null) {
                total = new CacheResult();
                total.key = cr.key;
                total.cacheMatch = cr.cacheMatch;
                total.cacheNoMatch = cr.cacheNoMatch;
                totalCacheMatch.put(cr.key, total);
            } else {
                total.cacheMatch += cr.cacheMatch;
                total.cacheNoMatch += cr.cacheNoMatch;
            }
        }
        List<String> sortKeyList = new ArrayList<String>(totalCacheMatch.keySet());
        Collections.sort(sortKeyList);
        for (String key : sortKeyList) {
            System.out.println(key + "\t total \t" + totalCacheMatch.get(key));
        }
    }

    private int fractionPart(int step, int total) {
        int part = 0;
        int rest;
        int index = 0;
        while (part <= step) {
            index++;
            int prevPart = part;
            rest = total - part;
            part = prevPart + (int) Math.ceil(rest / 2.0);
        }
        return index;
    }

    public void testFractionPart() {
        fractionPart(0, 100);
        Map<Integer, Integer> probabilityMap = new LinkedHashMap<Integer, Integer>();
        for (int i = 0; i < 100; i++) {
            int fractionPart = fractionPart(i, 100);
            Integer integer = probabilityMap.get(fractionPart);
            if (integer == null) {
                probabilityMap.put(fractionPart, 1);
            } else {
                probabilityMap.put(fractionPart, 1 + integer);
            }
            System.out.println(i + " " + fractionPart);
        }
        System.out.println(probabilityMap);
    }

    private class CacheResult {
        public int cacheMatch;
        public String key;
        public int cacheNoMatch;

        @Override
        public String toString() {
            StringBuilder b = new StringBuilder();
            b.append(key);
            b.append("=");
            b.append(cacheMatch);
            b.append("/");
            b.append(cacheNoMatch);
            b.append("->");
            b.append(((double) (cacheMatch)) / (cacheMatch + cacheNoMatch));
            return b.toString();
        }
    }

    private class TestCacheClient implements Callable<CacheResult> {

        private final String keyValue;

        protected TestCacheClient(String keyValue) {
            this.keyValue = keyValue;
        }

        @Override
        public CacheResult call() throws Exception {
            OutVariable<String> out = new OutVariable<String>();
            int cacheMatched = 0;
            int cacheNotMatched = 0;
            for (int i = 0; i < TASK_ITERATION_COUNT; i++) {
                if (!_cache.tryGetValue(keyValue, out)) {
                    _cache.add(keyValue, keyValue);
                    cacheNotMatched++;
                } else {
                    cacheMatched++;
                }
                synchronized (this) {
                    Thread.sleep((long) (Math.random() * 100));
                }
            }
            CacheResult cr = new CacheResult();
            cr.key = keyValue;
            cr.cacheMatch = cacheMatched;
            cr.cacheNoMatch = cacheNotMatched;

            return cr;
        }
    }
}