/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.test.unit.communication.configuration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import junit.framework.TestCase;
import dk.gov.oiosi.common.Guid;
import dk.gov.oiosi.communication.configuration.DocumentAllreadyAddedException;
import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.communication.configuration.NoDocumentTypeFoundFromIdException;
import dk.gov.oiosi.communication.configuration.NoDocumentTypeFoundFromParametersException;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminatorConfig;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminatorConfigCollection;

/**
 * test document type collection.
 */
public class RaspDocumentTypeCollectionConfigTest extends TestCase {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(RaspDocumentTypeCollectionConfigTest.class);

    /**
     * add and remove document.
     */
    public void test_01_AddRemoveDocumentType() {
        try {
            log.info("_01_AddRemoveDocumentTypeTest");
            String rootName = "Test";
            String rootNamespace = "http://oio.dk/test";
            XPathDiscriminatorConfig identifierDiscriminatorA = new XPathDiscriminatorConfig("//Test/Key", "A");
            XPathDiscriminatorConfig identifierDiscriminatorB = new XPathDiscriminatorConfig("//Test/Key", "B");
            XPathDiscriminatorConfigCollection identifierDiscriminators = new XPathDiscriminatorConfigCollection();
            identifierDiscriminators.add(identifierDiscriminatorA);
            identifierDiscriminators.add(identifierDiscriminatorB);
            DocumentTypeConfig documentType = new DocumentTypeConfig(rootName, rootNamespace, identifierDiscriminators);
            DocumentTypeCollectionConfig documentTypeCollection = new DocumentTypeCollectionConfig();
            documentTypeCollection.addDocumentType(documentType);
            DocumentTypeConfig documentTypeFromCollection = documentTypeCollection.getDocumentType(rootName, rootNamespace,
                    identifierDiscriminators);
            assertTrue(documentTypeFromCollection.equals(documentType));
            documentTypeCollection.removeDocumentType(documentTypeFromCollection);
            documentTypeFromCollection = null;
            DocumentTypeConfig empty = documentTypeCollection.tryGetDocumentType(rootName, rootNamespace, identifierDiscriminators);
            assertNull(empty);
            log.info("_01_ OK : ");
        } catch (Exception e) {
            log.error("_01_AddRemoveDocumentTypeTest failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * insert document.
     */
    public void test_02_InsertDocumentWithSameId() {
        try {
            log.info("_02_InsertDocumentWithSameIdTest");
            String rootNameA = "TestA";
            String rootNameB = "TestB";
            String rootNamespace = "http://oio.dk/test";
            XPathDiscriminatorConfig identifierDiscriminatorA = new XPathDiscriminatorConfig("//Test/Key", "A");
            XPathDiscriminatorConfig identifierDiscriminatorB = new XPathDiscriminatorConfig("//Test/Key", "B");
            XPathDiscriminatorConfigCollection identifierDiscriminators = new XPathDiscriminatorConfigCollection();
            identifierDiscriminators.add(identifierDiscriminatorA);
            identifierDiscriminators.add(identifierDiscriminatorB);
            DocumentTypeConfig documentTypeA = new DocumentTypeConfig(rootNameA, rootNamespace, identifierDiscriminators);
            DocumentTypeConfig documentTypeB = new DocumentTypeConfig(rootNameB, rootNamespace, identifierDiscriminators);
            DocumentTypeCollectionConfig documentTypeCollection = new DocumentTypeCollectionConfig();
            documentTypeCollection.addDocumentType(documentTypeA);
            documentTypeB.setId(documentTypeA.getId());
            documentTypeCollection.addDocumentType(documentTypeB);
            log.info("_02_ OK : ");
        } catch (DocumentAllreadyAddedException e) {
            log.info("_02_ OK : " + e);
            assertTrue(true);
        } catch (Exception e) {
            log.error("_02_InsertDocumentWithSameIdTest Failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * insert 2 documents with same key.
     */
    public void test_03_InsertDocumentTypeWithSameKeyValues() {
        try {
            log.info("_03_InsertDocumentTypeWithSameKeyValuesTest");
            String rootName = "Test";
            String rootNamespace = "http://oio.dk/test";
            XPathDiscriminatorConfig identifierDiscriminatorA = new XPathDiscriminatorConfig("//Test/Key", "A");
            XPathDiscriminatorConfig identifierDiscriminatorB = new XPathDiscriminatorConfig("//Test/Key", "B");
            XPathDiscriminatorConfigCollection identifierDiscriminators = new XPathDiscriminatorConfigCollection();
            identifierDiscriminators.add(identifierDiscriminatorA);
            identifierDiscriminators.add(identifierDiscriminatorB);
            DocumentTypeConfig documentTypeA = new DocumentTypeConfig(rootName, rootNamespace, identifierDiscriminators);
            DocumentTypeConfig documentTypeB = new DocumentTypeConfig(rootName, rootNamespace, identifierDiscriminators);
            DocumentTypeCollectionConfig documentTypeCollection = new DocumentTypeCollectionConfig();
            documentTypeCollection.addDocumentType(documentTypeA);
            documentTypeCollection.addDocumentType(documentTypeB);
            log.info("_03_ OK : ");
        } catch (DocumentAllreadyAddedException e) {
            log.info("_03_ OK : " + e);
            assertTrue(true);
        } catch (Exception e) {
            log.error("_03_InsertDocumentTypeWithSameKeyValuesTest Failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * search non existing document by id.
     */
    public void test_04_GetNonExistingDocumentTypeFromId() {
        try {
            log.info("_04_GetNonExistingDocumentTypeFromIdTest");
            String rootName = "Test";
            String rootNamespace = "http://oio.dk/test";
            XPathDiscriminatorConfig identifierDiscriminatorA = new XPathDiscriminatorConfig("//Test/Key", "A");
            XPathDiscriminatorConfig identifierDiscriminatorB = new XPathDiscriminatorConfig("//Test/Key", "B");
            XPathDiscriminatorConfigCollection identifierDiscriminators = new XPathDiscriminatorConfigCollection();
            identifierDiscriminators.add(identifierDiscriminatorA);
            identifierDiscriminators.add(identifierDiscriminatorB);
            DocumentTypeConfig documentType = new DocumentTypeConfig(rootName, rootNamespace, identifierDiscriminators);
            DocumentTypeCollectionConfig documentTypeCollection = new DocumentTypeCollectionConfig();
            documentTypeCollection.addDocumentType(documentType);
            DocumentTypeConfig documentTypeFromCollection = documentTypeCollection.tryGetDocumentType(Guid.newGuid());
            assertNull(documentTypeFromCollection);
            log.info("_04_ OK : ");
        } catch (Exception e) {
            log.error("_04_GetNonExistingDocumentTypeFromIdTest" + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * search non existing document by search parameters.
     */
    public void test_05_GetNonExistingDocumentTypeFromParameters() {
        try {
            log.info("_05_GetNonExistingDocumentTypeFromParameters");
            String rootName = "TestA";
            String rootNamespace = "http://oio.dk/test";
            XPathDiscriminatorConfig identifierDiscriminatorA = new XPathDiscriminatorConfig("//Test/Key", "A");
            XPathDiscriminatorConfig identifierDiscriminatorB = new XPathDiscriminatorConfig("//Test/Key", "B");
            XPathDiscriminatorConfigCollection identifierDiscriminators = new XPathDiscriminatorConfigCollection();
            identifierDiscriminators.add(identifierDiscriminatorA);
            identifierDiscriminators.add(identifierDiscriminatorB);
            DocumentTypeConfig documentType = new DocumentTypeConfig(rootName, rootNamespace, identifierDiscriminators);
            DocumentTypeCollectionConfig documentTypeCollection = new DocumentTypeCollectionConfig();
            documentTypeCollection.addDocumentType(documentType);
            DocumentTypeConfig documentTypeFromCollection = documentTypeCollection.tryGetDocumentType("TestB", rootNamespace,
                    identifierDiscriminators);
            assertNull(documentTypeFromCollection);
            log.info("_05_ OK : ");
        } catch (Exception e) {
            log.error("_05_GetNonExistingDocumentTypeFromParameters Failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * TODO: test ???.
     */
    public void test_06_GetNonExistingDocumentTypeFromId() {
        try {
            log.info("_06_GetNonExistingDocumentTypeFromIdTest");
            String rootName = "Test";
            String rootNamespace = "http://oio.dk/test";
            XPathDiscriminatorConfig identifierDiscriminatorA = new XPathDiscriminatorConfig("//Test/Key", "A");
            XPathDiscriminatorConfig identifierDiscriminatorB = new XPathDiscriminatorConfig("//Test/Key", "B");
            XPathDiscriminatorConfigCollection identifierDiscriminators = new XPathDiscriminatorConfigCollection();
            identifierDiscriminators.add(identifierDiscriminatorA);
            identifierDiscriminators.add(identifierDiscriminatorB);
            DocumentTypeConfig documentType = new DocumentTypeConfig(rootName, rootNamespace, identifierDiscriminators);
            DocumentTypeCollectionConfig documentTypeCollection = new DocumentTypeCollectionConfig();
            documentTypeCollection.addDocumentType(documentType);
            documentTypeCollection.getDocumentType(Guid.newGuid());

            log.error("_06_GetNonExistingDocumentTypeFromIdTest should faile");
            fail("_06_GetNonExistingDocumentTypeFromIdTest should fail");
        } catch (NoDocumentTypeFoundFromIdException e) {
            log.info("_06_ OK : " + e);
            assertTrue(true);
        } catch (Exception e) {
            log.error("_06_GetNonExistingDocumentTypeFromIdTest - Failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * TODO: test what ???.
     */
    public void test_07_GetNonExistingDocumentTypeFromParameters() {
        try {
            log.info("_07_GetNonExistingDocumentTypeFromParameters");
            String rootName = "TestA";
            String rootNamespace = "http://oio.dk/test";
            XPathDiscriminatorConfig identifierDiscriminatorA = new XPathDiscriminatorConfig("//Test/Key", "A");
            XPathDiscriminatorConfig identifierDiscriminatorB = new XPathDiscriminatorConfig("//Test/Key", "B");
            XPathDiscriminatorConfigCollection identifierDiscriminators = new XPathDiscriminatorConfigCollection();
            identifierDiscriminators.add(identifierDiscriminatorA);
            identifierDiscriminators.add(identifierDiscriminatorB);
            DocumentTypeConfig documentType = new DocumentTypeConfig(rootName, rootNamespace, identifierDiscriminators);
            DocumentTypeCollectionConfig documentTypeCollection = new DocumentTypeCollectionConfig();
            documentTypeCollection.addDocumentType(documentType);
            documentTypeCollection.getDocumentType("TestB", rootNamespace, identifierDiscriminators);

            log.error("_07_GetNonExistingDocumentTypeFromParameters should fail");
            fail("_07_GetNonExistingDocumentTypeFromParameters should fail");
        } catch (NoDocumentTypeFoundFromParametersException e) {
            log.info("_07_ OK : " + e);
            assertTrue(true);
        } catch (Exception e) {
            log.error("_07_GetNonExistingDocumentTypeFromParameters Failed : " + e, e);
            fail(e.getMessage());
        }
    }
}
