package dk.gov.oiosi.xml.schematron.research;

import java.io.ByteArrayOutputStream;
import java.text.NumberFormat;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;

import junit.framework.TestCase;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.jaxp.OMSource;

import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.test.util.TickTock;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.schematron.FailedToLoadSchematronStylesheetException;
import dk.gov.oiosi.xml.schematron.SchematronStore;
import dk.gov.oiosi.xml.schematron.SchematronStoreFactory;
import dk.gov.oiosi.xml.xslt.XsltUtility;

/**
 * Test to analyse potential improvement in case of usage multiple instances of
 * transformer for the same document type.<br>
 * 
 * In v.1.2.3 OIORASP for one document type there was only one instance of
 * transformer.<br>
 * 
 * But used transformer implementation
 * (<b>org.apache.xalan.transformer.TransformerImpl</b>) was synchronized inside
 * implementation, so actually if several threads call method transform() at the
 * same transformer instance, they wait each other.<br>
 * <br>
 * 
 * This test allows to try different number of transformer instances with
 * different number of threads.<br>
 * Each thread executes schematron transformation of valid and invalid OIOUBL
 * document without result analysis
 * 
 * <br>
 * <br>
 * <b>Analysis results</b> <br>
 * <br>
 * 
 * Analysis shows, that transformation operations are very CPU consuming
 * (because there is no I/O operations inside), so if some thread starts to
 * work, it uses 100% CPU core. In case if there are several CPU cores, it is
 * reasonable to have not more than this number of transformer instances,
 * beacuse even if they will not wait for each other by synchronized block, they
 * will compete for CPU time. <br>
 * <br>
 * 
 * Number of CPU cores can be accessed via
 * Runtime.getRuntime().availableProcessors().
 * 
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * 
 */
public class MutlipleTransformerVsSingleResearchTest extends TestCase {

	private static final int THREAD_REPEAT = 100;

	private static boolean SHOW_EACH_RESULT = false;
	private static boolean CHECK_EACH_RESULT = false;

	private XmlDocument invoiceValid;
	private XmlDocument invoiceInvalid;
	private String resultValid;
	private String resultInvalid;
	private TickTock t;

	private Object monitor = new Object();
	private volatile int doneCount;
	private String schematronDocumentPath;

	private Transformer[] transformers;

	private int threadCount = 2;

	@Override
	protected void setUp() throws Exception {
		System.out.println("*********************");
		System.out.println("Start setup for MutlipleTransformerVsSingleResearchTest");
		int processors = Runtime.getRuntime().availableProcessors();
		System.out.println("Number of CPUs: " + processors);
		threadCount = processors;

		ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
		t = new TickTock();
		invoiceValid = new XmlDocument();
		invoiceValid.load(TestConstants.PATH_INVOICE_NO_ID);
		t.tockTick("loadValid");

		invoiceInvalid = new XmlDocument();
		invoiceInvalid.load(TestConstants.PATH_INVOICE);
		t.tockTick("loadInvalid");

		DocumentTypeCollectionConfig config = ConfigurationHandler.getInstance().getRaspDocumentTypeCollectionConfig();
		t.tockTick("loadConfigList");
		DocumentTypeConfig documentTypeConfig = config.getDocumentTypes("Invoice", "urn:oasis:names:specification:ubl:schema:xsd:Invoice-2").next();
		t.tockTick("findConfig");
		schematronDocumentPath = documentTypeConfig.getSchematronValidationConfig().getSchematronDocumentPath();
		Transformer transformer = getTransformer(-1);
		t.tockTick("buildTransformer");
		System.out.println("Transformer class: " + transformer.getClass());

		resultValid = doTransform(transformer, invoiceValid.getOMDocument());
		t.tockTick("validTransform");
		resultInvalid = doTransform(transformer, invoiceInvalid.getOMDocument());
		t.tockTick("invalidTransform");

		System.out.println(t.toStatusTableString());
	}

	public void testAnalysePotentialImprovement() throws Exception {
		TickTock t = new TickTock();
		transformers = new Transformer[threadCount];
		for (int i = 0; i < transformers.length; i++) {
			transformers[i] = loadCompiledSchematron(schematronDocumentPath);
			t.tock("buildAverage");
			t.tockTick("build" + i);
		}
		System.out.println(t.toStatusTableString());

		System.out.println("**********************");
		System.out.println("Many transformers for many threads");
		long manyTotalDuration = checkCase2();

		transformers = null;
		System.out.println("**********************");
		System.out.println("One transformer for many threads");
		long oneTotalDuration = checkCase2();

		double improvementRate = (oneTotalDuration - manyTotalDuration) / (double) oneTotalDuration;
		String stringRate = NumberFormat.getPercentInstance().format(improvementRate);

		System.out.println("Perfromance improvement percent: " + stringRate);

		assertTrue("Many transformers usage is supposed to be at least 50% faster on 4 CPU cores, but was " + stringRate, improvementRate * 100 > 50);
	}

	protected long checkCase() throws InterruptedException {
		doneCount = 0;
		Thread[] threads = new Thread[threadCount];
		TransformerJob[] jobs = new TransformerJob[threads.length];
		for (int i = 0; i < threads.length; i++) {
			jobs[i] = new TransformerJob(i);
			threads[i] = new Thread(jobs[i]);
		}

		long startTime = System.currentTimeMillis();
		for (int i = 0; i < threads.length; i++) {
			threads[i].start();
		}

		while (doneCount < threadCount) {
			synchronized (monitor) {
				monitor.wait();
			}
		}
		long totalExecutionTime = System.currentTimeMillis() - startTime;

		long threadTotalExecutionTime = 0;
		for (int i = 0; i < threads.length; i++) {
			threadTotalExecutionTime += jobs[i].getTotalTransformationTime();
		}

		System.out.println("Total transformation time: " + totalExecutionTime);
		return totalExecutionTime;
	}

	/*
	 * Similar to checkCase just with Executors usage, left for history
	 */
	protected long checkCase2() throws InterruptedException {
		doneCount = 0;
		
		ExecutorService executor = Executors.newFixedThreadPool(threadCount);
		TransformerJob[] jobs = new TransformerJob[threadCount];
		long startTime = System.currentTimeMillis();

		for (int i = 0; i < jobs.length; i++) {
			jobs[i] = new TransformerJob(i);
			executor.execute(jobs[i]);
		}
		executor.shutdown();

		// Wait until all threads are finish
		while (!executor.isTerminated()) {
			System.out.print(".");
		}
		System.out.println();
		
		long totalExecutionTime = System.currentTimeMillis() - startTime;

		long threadTotalExecutionTime = 0;
		for (int i = 0; i < jobs.length; i++) {
			threadTotalExecutionTime += jobs[i].getTotalTransformationTime();
		}

		System.out.println("Total transformation time: " + totalExecutionTime);
		return totalExecutionTime;
	}
	
	
	protected Transformer getTransformer(int threadIndex) throws FailedToLoadSchematronStylesheetException {
		if (transformers != null) {
			return transformers[threadIndex];
		}
		SchematronStore s = SchematronStoreFactory.getSchematronStore();
		Transformer transformer = s.getCompiledSchematron(schematronDocumentPath);
		return transformer;
	}

	/*
	 * Copied from private
	 * dk.gov.oiosi.xml.schematron.SchematronStore.loadCompiledSchematron
	 * (String)
	 */
	private Transformer loadCompiledSchematron(String path) throws FailedToLoadSchematronStylesheetException {
		XmlDocument xmlStylesheet = new XmlDocument();
		XsltUtility xsltUtility = new XsltUtility();
		try {
			xmlStylesheet.load(path);
			return xsltUtility.precompiledStyleSheet(xmlStylesheet);
		} catch (Exception ex) {
			throw new FailedToLoadSchematronStylesheetException(path, ex);
		}
	}

	protected String doTransform(Transformer transformer, OMElement doc) throws Exception {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		transformer.transform(new OMSource(doc), new StreamResult(baos));
		String curResult = new String(baos.toByteArray(), "UTF-8");
		return curResult;
	}

	private class TransformerJob implements Runnable {

		private final int index;
		private long totalTransformationTime;

		public TransformerJob(int index) {
			this.index = index;
		}

		@Override
		public void run() {
			for (int i = 0; i < THREAD_REPEAT; i++) {
				boolean validCase = Math.random() > 0.5;
				try {
					Transformer transformer = getTransformer(index);
					OMElement doc = validCase ? invoiceValid.getOMDocument() : invoiceInvalid.getOMDocument();
					long start = System.currentTimeMillis();
					String caseName = null;
					if (SHOW_EACH_RESULT) {
						caseName = (validCase ? "OK" : "KO");
						System.out.println(index + "\tStart \t" + i + "\t" + caseName);
					}
					String curResult = doTransform(transformer, doc);
					long transformationDuration = System.currentTimeMillis() - start;
					totalTransformationTime += transformationDuration;
					if (SHOW_EACH_RESULT) {
						System.out.println(index + "\tDone  \t" + i + "\t" + caseName + "\t" + transformationDuration);
					}
					if (CHECK_EACH_RESULT) {
						if (validCase) {
							assertEquals(resultValid, curResult);
						} else {
							assertEquals(resultInvalid, curResult);
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
					throw new RuntimeException(e);
				}
			}
			doneCount++;
			synchronized (monitor) {
				monitor.notify();
			}
		}

		public long getTotalTransformationTime() {
			return totalTransformationTime;
		}
	}
}
