package dk.gov.oiosi.xml.schematron.research;

import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schematron.research.bulk.*;
import junit.framework.TestCase;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMXMLParserWrapper;
import org.apache.axiom.om.impl.llom.factory.OMXMLBuilderFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * This research test compares performance of 3 ways doing transformation -
 * OIORASP 1.2.3 way, when XSL is loaded as AXIOM document, completely built,
 * then converted to DOM and passed to transformer factory as DOMSource. <br>
 * <p/>
 * In OIORASP 1.3.0 it is proposed to skip conversion from AXIOM to DOM and
 * generate transformer object directly from byte array. Also, in 1.3.0 it is
 * proposed to use optimized parsing of transformation result - stop processing
 * on first error. <br>
 * <p/>
 * Also, comparison includes check that transforming AXIOM built XML is almost
 * equal to transforming raw byte[] array XML by CPU time. <br>
 * <p/>
 * This test supposes to process big amount of documents. It was executed
 * against more than 200 000 real documents, received via NemHandel
 * infrastructure by TrueLink A/S implementation in between 01.01.2012 and
 * 01.03.2012. Most of these documents are schematron valid, but also rejected
 * documents by schematron error are analysed.<br>
 * <p/>
 * Test walks through given folder, which can contain sub-folder with documents.
 * File is considered as XML file for analysis if its name contains ".xml", so
 * it can be 3242342.xml or 324324.xml.tmp.<br>
 * <p/>
 * Also, if test folder contains *.zip.gz archive, it unpacks *.zip file
 * (supposed to be stored with none compression level) to system temporary
 * folder, walks through its contents and deletes temporary *.zip file.<br>
 * <p/>
 * <p/>
 * All kinds of transformation should provide the same results - either equal
 * error message description or no error. <br>
 * <br>
 * <p/>
 * So, goals for this test are: <br>
 * <p/>
 * 1. Check that optimized failed document processing provides the same result
 * as in OIORASP 1.2.3 <br>
 * <p/>
 * 2. Calculate performance improvement on a big amount of documents. <br>
 * <p/>
 * 3. Make decision, is there any reason to switch to transformation of built
 * XML AXIOM document instead of raw XML byte array. <br>
 * <br>
 * <b>Research restuls</b> <br>
 * <p/>
 * After more than 12 hours of execution next figures were receveived: <br>
 * <br>
 * total 428900 ok 391901 ok diff 1564840 ok diff ratio 0.09432041431449292
 * error 36999 error diff 292198 error diff ratio 0.3159105627506246 ok2 diff
 * 1444377 ok2 diff ratio 0.08705953136827045 error diff 282184 error2 diff
 * ratio 0.3050839028303488<br>
 * <br>
 * <p/>
 * It means: 428900 documents in total were processed, there were 36999
 * documents with schematron error. Transformations were done by 1.2.3 way
 * (build Axiom XML document, converted to DOM, is transformed with build Axiom
 * XSL, also converted to DOM) and compared with 1.3.0 way - built Axiom XML
 * converted with byte array parsed XSL). Also compared with byte array XML
 * converted with byte array XSL.<br>
 * <p/>
 * In case of documents without any schematron error:<br>
 * <p/>
 * 1.3.0 way - is 9,4% faster than 1.2.3 way, both byte arrays - 8,7% faster.<br>
 * <p/>
 * In case of documents with errors:<br>
 * <p/>
 * 1.3.0 way - is 31,6% faster than 1.2.3 way, both byte arrays - 30,5% faster.<br>
 * <br>
 * <b>Conclusions</b> <br>
 * <br>
 * Axiom built XML model is a little faster than raw byte array XML. This can be
 * explained by internal caching at Axiom model and little average size of
 * processed XML documents. In case if XML is quite big, raw byte array should
 * be faster than Axiom built model, because it should have lower memory
 * requirements.<br>
 *
 * @author Dmitriy Lapko, TrueLink A/S
 */
public class SchematronComparatorBulkResearchTest extends TestCase {

    private static final Logger log = Logger.getLogger(SchematronComparatorBulkResearchTest.class);

    private static final boolean CLEAR_FILE_ERROR_MAPS = true;

    public static String PATH_TEST_FOLDER = "D:/temp/32";

    private Map<String, String> fileSchematronErrorMap = new HashMap<String, String>();
    private Map<String, String> fileSchemaErrorMap = new HashMap<String, String>();

    private DifferentWayTransformationExecutor transformExecutor;

    @Override
    public void setUp() {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
        ConfigurationHandler.getInstance();
        transformExecutor = new DifferentWayTransformationExecutor();
    }

    public void test() throws Exception {
        checkFolder(PATH_TEST_FOLDER);
    }

    public void checkFolder(String path) throws Exception {
        File testFolder = new File(path);
        if (!testFolder.exists() || !testFolder.isDirectory()) {
            return;
        }

        FileSystemCrowler.log.setLevel(Level.INFO);
        List<IFileListener> fileListenerList = new ArrayList<IFileListener>();
        XmlFileListener xmlListener = new XmlFileListener();
        fileListenerList.add(xmlListener);
        fileListenerList.add(new ErrorFileListener());
        fileListenerList.add(new GZipFileListener());
        List<IFolderListener> folderListenerList = new ArrayList<IFolderListener>();
        folderListenerList.add(new MapCleanerListener());

        FileSystemCrowler fsc = new FileSystemCrowler(fileListenerList, folderListenerList);
        fsc.processFolder(testFolder);
        System.out.println(xmlListener.toStatus());
    }

    protected String buildFileNameKey(String name) {
        String fileKey = name.trim();
        int lastIndexOfSlash = fileKey.lastIndexOf("/");
        if (lastIndexOfSlash > 0) {
            fileKey = fileKey.substring(lastIndexOfSlash + 1);
        }
        if (fileKey.endsWith("_error.txt")) {
            fileKey = fileKey.substring(0, fileKey.length() - "_error.txt".length());
        }
        int lastUnderscore = fileKey.lastIndexOf("_");
        if (lastUnderscore > 0) {
            fileKey = fileKey.substring(0, lastUnderscore);
        }
        return fileKey;
    }

    protected void clearFileErrorMaps() {
        if (CLEAR_FILE_ERROR_MAPS) {
            fileSchemaErrorMap.clear();
            fileSchematronErrorMap.clear();
        }
    }

    private OMElement loadAxiomBuild(byte[] data) throws Exception {
        XMLStreamReader parser;
        parser = XMLInputFactory.newInstance().createXMLStreamReader(new ByteArrayInputStream(data));
        OMXMLParserWrapper builder = OMXMLBuilderFactory.createStAXOMBuilder(OMAbstractFactory.getOMFactory(), parser);
        OMElement doc = builder.getDocumentElement();
        doc.build();
        return doc;
    }

    public class MapCleanerListener implements IFolderListener {
        public void apply(String folderName) {
            if (folderName.equals("rejected")) {
                clearFileErrorMaps();
            }
        }
    }

    private class GZipFileListener implements IFileListener {

        public void apply(String fileName, byte[] data) {
        }

        public boolean isMatch(String fileName) {
            if (fileName.endsWith(".zip.gz")) {
                clearFileErrorMaps();
            }
            return false;
        }
    }

    private class XmlFileListener implements IFileListener {

        private int totalCount = 0;

        private int countError;
        private int countOk;

        private long totalNewOk;
        private long totalOldOk;
        private long totalNew2Ok;

        private long totalNewError;
        private long totalOldError;
        private long totalNew2Error;

        private static final boolean SHOW_RESULT = false;

        public boolean isMatch(String fileName) {
            return fileName.indexOf(".xml") > 0 && fileName.indexOf("_desc.xml") < 0;
        }

        public void apply(String name, byte[] xmlData) throws Exception {
            if (SHOW_RESULT) {
                log.info(name + ":" + xmlData.length);
            }

            totalCount++;

            XmlDocument xd = new XmlDocument();
            xd.setDocument(loadAxiomBuild(xmlData));

            DocumentTypeConfigSearcher dts = new DocumentTypeConfigSearcher();
            DocumentTypeConfig documentType = dts.findUniqueDocumentType(xd);
            if (SHOW_RESULT) {
                System.out.println(documentType.getFriendlyName());
            }
            long start;

            start = System.currentTimeMillis();
            String axiomXmlTransformerSaxResult = transformExecutor.version124Way(documentType, xd);
            long durationAxiomXmlTransformerSax = (System.currentTimeMillis() - start);
            if (SHOW_RESULT) {
                System.out.println(axiomXmlTransformerSaxResult);
            }

            start = System.currentTimeMillis();
            String axiomXmlTransformerDomResult = transformExecutor.version123Way(documentType, xd);
            long durationAxiomXmlTransformerDom = System.currentTimeMillis() - start;
            if (SHOW_RESULT) {
                System.out.println(axiomXmlTransformerDomResult);
            }

            start = System.currentTimeMillis();
            String byteArrayXmlTransformerSaxResult = transformExecutor.byteArrayXmlTransformerSax(documentType, xmlData);
            long durationByteArrayXmlTransformerSax = System.currentTimeMillis() - start;
            if (SHOW_RESULT) {
                System.out.println(byteArrayXmlTransformerSaxResult);
            }

            if (axiomXmlTransformerSaxResult == null) {
                totalNewOk += durationAxiomXmlTransformerSax;
                totalOldOk += durationAxiomXmlTransformerDom;
                totalNew2Ok += durationByteArrayXmlTransformerSax;

                countOk++;
            } else {
                totalNewError += durationAxiomXmlTransformerSax;
                totalOldError += durationAxiomXmlTransformerDom;
                totalNew2Error += durationByteArrayXmlTransformerSax;

                countError++;
            }

            if (axiomXmlTransformerSaxResult == null && axiomXmlTransformerDomResult != null) {
                System.out.println("KO\t" + name + ": new way is null, old way returned\n" + axiomXmlTransformerDomResult);
            } else {
                if (axiomXmlTransformerSaxResult != null && axiomXmlTransformerDomResult == null) {
                    System.out.println("KO\t" + name + ": old way is null, new way returned\n" + axiomXmlTransformerSaxResult);
                } else {
                    if (axiomXmlTransformerSaxResult != null && axiomXmlTransformerDomResult != null) {
                        if (!axiomXmlTransformerSaxResult.equals(axiomXmlTransformerDomResult)) {
                            System.out.println("KO\t" + name + ", new way and old way are not equal, new way\n" + axiomXmlTransformerSaxResult + "\noldWay:\n" + axiomXmlTransformerDomResult);
                        } else {
                            if (SHOW_RESULT) {
                                System.out.println("OK\t" + name + " both returned equal ERROR");
                            }
                        }
                    } else {
                        if (SHOW_RESULT) {
                            System.out.println("OK\t" + name + " both returned OK");
                        }
                    }
                }
            }

            if (byteArrayXmlTransformerSaxResult == null && axiomXmlTransformerDomResult != null) {
                System.out.println("KO\t" + name + ": new2 way is null, old way returned\n" + axiomXmlTransformerDomResult);
            } else {
                if (byteArrayXmlTransformerSaxResult != null && axiomXmlTransformerDomResult == null) {
                    System.out.println("KO\t" + name + ": old way is null, new2 way returned\n" + byteArrayXmlTransformerSaxResult);
                } else {
                    if (byteArrayXmlTransformerSaxResult != null && axiomXmlTransformerDomResult != null) {
                        if (!byteArrayXmlTransformerSaxResult.equals(axiomXmlTransformerDomResult)) {
                            System.out.println("KO\t" + name + ", new2 way and old way are not equal, new2 way\n" + byteArrayXmlTransformerSaxResult + "\noldWay:\n" + axiomXmlTransformerDomResult);
                        } else {
                            if (SHOW_RESULT) {
                                System.out.println("OK\t" + name + " both returned equal ERROR");
                            }
                        }
                    } else {
                        if (SHOW_RESULT) {
                            System.out.println("OK\t" + name + " both returned OK");
                        }
                    }
                }
            }

            if (totalCount % 100 == 0) {
                System.out.println(toStatus());
            }
        }

        public String toStatus() {
            StringBuffer sb = new StringBuffer();
            long diffOk = totalOldOk - totalNewOk;
            long diff2Ok = totalOldOk - totalNew2Ok;
            long diffError = totalOldError - totalNewError;
            long diff2Error = totalOldError - totalNew2Error;

            sb.append("total\t");
            sb.append(totalCount);
            sb.append("\tok\t" + countOk + "\tok diff\t");
            sb.append((diffOk));
            sb.append("\tok diff ratio\t");
            if (totalOldOk == 0) {
                sb.append("unknown");
            } else {
                sb.append((diffOk / (double) totalOldOk));
            }

            sb.append("\terror\t" + countError + "\terror diff\t");
            sb.append((diffError));
            sb.append("\terror diff ratio\t");
            if (totalOldError == 0) {
                sb.append("unknown");
            } else {
                sb.append((diffError / (double) totalOldError));
            }

            sb.append("\tok2 diff\t");
            sb.append((diff2Ok));
            sb.append("\tok2 diff ratio\t");
            if (totalOldOk == 0) {
                sb.append("unknown");
            } else {
                sb.append((diff2Ok / (double) totalOldOk));
            }

            sb.append("\terror diff\t");
            sb.append((diff2Error));
            sb.append("\terror2 diff ratio\t");
            if (totalOldError == 0) {
                sb.append("unknown");
            } else {
                sb.append((diff2Error / (double) totalOldError));
            }

            return sb.toString();
        }
    }

    /*
      * Not used in this test actually
      */
    private class ErrorFileListener implements IFileListener {

        public boolean isMatch(String fileName) {
            return fileName.indexOf("_error.txt") > 0;
        }

        public void apply(String name, byte[] data) throws Exception {
            String s = new String(data, "UTF-8").trim();
            String schemaError = ErrorMessageTextHandler.extractSchemaError(s);
            String schematronError = ErrorMessageTextHandler.extractSchematronError(s);

            if (schemaError == null && schematronError == null) {
                return;
            }

            String fileKey = buildFileNameKey(name);
            if (schemaError != null) {
                if (fileSchemaErrorMap.put(fileKey, schemaError) != null) {
                    System.err.println("fileSchemaErrorMap already contains value for key " + fileKey + ": " + fileSchemaErrorMap.get(fileKey) + ", tried to add value " + schemaError);
                    fail("");
                }
            }

            if (schematronError != null) {
                if (fileSchematronErrorMap.put(fileKey, schematronError) != null) {
                    System.err.println("fileSchematronErrorMap already contains value for key " + fileKey + ": " + fileSchematronErrorMap.get(fileKey) + ", tried to add value " + schematronError);
                    fail("");
                }
            }
        }
    }
}