package dk.gov.oiosi.xml.schematron.research;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.List;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;

import junit.framework.TestCase;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMXMLParserWrapper;
import org.apache.axiom.om.impl.jaxp.OMSource;
import org.apache.axiom.om.impl.llom.factory.OMXMLBuilderFactory;
import org.apache.axiom.om.xpath.AXIOMXPath;
import org.apache.axis2.util.XMLUtils;
import org.apache.commons.configuration.ConfigurationUtils;
import org.apache.commons.io.IOUtils;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.test.util.TickTock;
import dk.gov.oiosi.xml.XmlDocument;

/**
 * In OIORASP 1.2.3 next procesure is used for XSLT transformations:<br>
 * 
 * 1) Completely built AXIOM model (when XmlDocument is created) of XSL
 * transformation is converted to DOM for DOMSource and transformer is built via
 * DOMSource FOR EACH transformation;<br>
 * 
 * 2) Completely built AXIOM model of document (when XmlDocument is created) is
 * converted to DOM for DOMSource and passed to transformer as input DOMSource.<br>
 * <br>
 * 
 * The goal of this test - compare performance of all combinations of next sets:<br>
 * 
 * A. XSLT loading (enum DocumentLoading): AXIOM completely built (current) /
 * AXIOM not built / DOM / raw byte array;<br>
 * 
 * B. transformer source (enum DocumentSource): OMSource / DOMSource (current) /
 * SAXSource (on raw byte array);<br>
 * 
 * C. document loading (enum DocumentLoading): AXIOM completely built (current)/
 * AXIOM not built / DOM / raw byte array;<br>
 * 
 * D. document source (enum DocumentSource): OMSource / DOMSource (current) /
 * SAXSource (on raw byte array).<br>
 * <br>
 * 
 * Together with AXIOM completely built by call of build() method, let's also
 * compare it with similar method buildWithAttachment().<br>
 * 
 * Each case should be executed 5 times, first 2 executions should be ignored
 * (JVM warm-up), average figure of last 3 executions should be printed.<br>
 * 
 * Timing should include XSLT/XML loading time, building (if supposed) of AXIOM
 * document, trasnformer creation and actual transformation time.<br>
 * 
 * Test should be done done on a valid OIOUBL Invoice with size 8,4 KB (typical
 * size), ~1 MB with many lines and ~1 MB with big attachment, against the same
 * schematron validation of size 2,8 MB<br>
 * 
 * The whole test should be executed with maximum potentially required memory -
 * 4 GB via JRE keys -Xmx256m -Xms256m (set inital and maximum heap size to
 * maximum limit).<br>
 * 
 * After each attempt all objects should be cleared and GC is called.<br>
 * <br>
 * 
 * <b>Research result</b><br>
 * <br>
 * 
 * Excel file with result figures is commited as
 * /dk.gov.oiosi.library/test/src/dk/gov/oiosi
 * /xml/schematron/research/TransformationSourceResearchTest_Result.xlsx<br>
 * 
 * Tab "BIG_LINES" contains results for big document (~1 MB) with almost 800
 * invoice lines.<br>
 * 
 * Tab "BIG_PDF" contains results for big document (~1 MB) but with only 4
 * invoice lines, rest is Base64 encoded PDF document.<br>
 * 
 * Tab "LITTLE" contains results for usual document - 8,4 KB with 2 invoice
 * lines.<br>
 * <br>
 * Each tab contains next columns:<br>
 * 
 * <b>Index</b> - case index<br>
 * 
 * <b>Case xml size</b> - XML document size<br>
 * 
 * <b>xslt source</b> - type of XSLT source: SAX - SAXSource, AXIOM - OMSource,
 * DOM - DOMSource.<br>
 * 
 * <b>xsl load</b> - type of XSLT loading: DOM, AXIOM, AXIOM_BUILT,
 * AXIOM_BUILT_ATTACHMENT<br>
 * 
 * <b>xml source</b> - type of XML source: SAX - SAXSource, AXIOM - OMSource,
 * DOM - DOMSource.<br>
 * 
 * <b>xml load</b> - type of XML loading: DOM, AXIOM, AXIOM_BUILT,
 * AXIOM_BUILT_ATTACHMENT<br>
 * 
 * <b>xsltLoad</b> - duration of XSLT loading in ms<br>
 * 
 * <b>xsltBuild</b> - duration of XSLT building (for AXIOM_BUILT and
 * AXIOM_BUILT_ATTACHMENT) in ms<br>
 * 
 * <b>xsltSource</b> - duration of XSLT Source building in ms<br>
 * 
 * <b>xmlLoad</b> - duration of XML loading in ms<br>
 * 
 * <b>xmlBuild</b> - duration of XML building (for AXIOM_BUILT and
 * AXIOM_BUILT_ATTACHMENT) in ms<br>
 * 
 * <b>xmlSource</b> - duration of XML Source building in ms<br>
 * 
 * <b>transBuild</b> - duration of transformer object building in ms<br>
 * 
 * <b>transRun</b> - duration of first transformeration in ms<br>
 * 
 * <b>transRunMulti</b> - average duration of first transformeration and next 2
 * the same transformations in ms<br>
 * 
 * <b>total</b> - total time for the whole case - loading, building of XML+XSLT
 * and all transformations in ms<br>
 * 
 * <b>gc</b> - duration of garbage collection phase in ms <br>
 * 
 * <b>transTotal</b> - sum of durations to build transformer and first
 * transformation in ms<br>
 * 
 * <b>xsltTotal</b> - sum of durations to load, build and prepare Source for
 * XSLT and execute first transformation in ms <br>
 * 
 * <b>xml+transRun</b> - sum of durations to load, build and preapre Source for
 * XML and execute multiple transformations in ms<br>
 * 
 * All durations are average of 3 executions of the whole case.<br>
 * 
 * OIORASP 1.2.3 way is highlighted with blue background color. <br>
 * 
 * Tab "BIG_LINES" is sorted by column "transRunMulti".<br>
 * 
 * Tabs "BIG_PDF" and "LITTLE" are sorted by column "transTotal".<br>
 * <br>
 * <b>Conclusions</b><br>
 * <br>
 * 
 * 1. OIORASP 1.2.3 way is not optimal - it is not reasonable to combine
 * different ways of loading XML/XSLT and building Source objects for
 * transfomation.<br>
 * 
 * 2. SAX way of loading XSLT is preferred.<br>
 * 
 * 3. SAX/AXIOM built XML loading/sourcing are similar by performance.<br>
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * 
 */
public class TransformationSourceResearchTest extends TestCase {

	/*
	 * Configuration - fields - how many times repeat each case to collect
	 * average statistics.
	 * 
	 * In case of a first case, JVM is not warmed-up, so let's do a little more
	 * warming executions before average calculation.
	 */
	private static final int CASE_REPEAT_FIRST_CASE_START_AVERAGE_CALCULATION = 7;
	private static final int CASE_REPEAT_FIRST_CASE = 10;
	private static final int CASE_REPEAT_START_AVERAGE_CALCULATION = 2;
	private static final int CASE_REPEAT = 5;
	/*
	 * Set to 1 to not repeat
	 */
	private static final int CASE_REPEAT_WTH_THE_SAME_TRANSFORMER = 3;
	/*
	 * Configuration fields - list of path to files for processing. If there is
	 * no such file on a file system where test is started, the case is ignored.
	 */
	private static final String PATH_BIG_INVOICE_WITH_MANY_LINES = "D:/task/nh/work/2013.03.13_Source_Perf_Analysis/OIOUBL_Invoice_1MB_ManyLines.xml";
	private static final String PATH_BIG_INVOICE_WITH_BIG_ATTACHMENT = "D:/task/nh/work/2013.03.13_Source_Perf_Analysis/OIOUBL_Invoice_1MB_Attachment.xml";
	private static final String PATH_XSLT = "Resources/Schematrons/OIOUBL_v2.0/OIOUBL_Invoice_Schematron.xsl";

	/**
	 * This is specific research test, its results should be used only for
	 * architecture decision, so it should not be executed each time at CI. But
	 * it is good to keep it in the project.
	 */
	private static final boolean DO_ACTUAL_TEST = true;

	private static final boolean FAIL_ON_DIFFERENT_RESULT = true;

	/*
	 * To exclude file system delays interference, we load data bytes into
	 * memory before any usage.
	 */

	/**
	 * XSLT file bytes, loaded once per test
	 */
	private byte[] xsltBytes;
	/**
	 * XML file bytes, loaded for each value of DocumentSize
	 */
	private byte[] xmlBytes;

	/**
	 * Enumeration with analysed ways to load documents.
	 * 
	 * @author dlk
	 */
	private enum DocumentLoading {
		/**
		 * Load into memory AXIOM model and build it with build() method
		 */
		AXIOM_BUILT,
		/**
		 * Load into memory AXIOM model and build it with buildWithAttachment()
		 * method
		 */
		AXIOM_BUILT_ATTACHMENT(false),
		/**
		 * Load into memory AXIOM model, do not build it
		 */
		AXIOM,
		/**
		 * Load into memory W3C DOM model - it is automatically built
		 */
		DOM;

		protected final boolean skip;

		private DocumentLoading() {
			this.skip = false;
		}

		/**
		 * Convenience constructor to exclude some case
		 */
		private DocumentLoading(boolean skip) {
			this.skip = skip;
		}
	}

	/**
	 * Enumeration to describe possible sources for transformer building and
	 * processing
	 * 
	 * @author dlk
	 */
	private enum DocumentSource {
		/**
		 * OMSource
		 */
		AXIOM,
		/**
		 * DOMSource for
		 */
		DOM,
		/**
		 * SAXSource for raw byte array
		 */
		SAX;

		protected final boolean skip;

		private DocumentSource() {
			this.skip = false;
		}

		/**
		 * Convenience constructor to exclude some case
		 */
		private DocumentSource(boolean skip) {
			this.skip = skip;
		}
	}

	/**
	 * Enumeration to describe available documents for test of different size of
	 * intention
	 * 
	 * @author dlk
	 */
	private enum DocumentSize {
		/**
		 * Non-valid OIOUBL Invoice, used to check correctness of
		 * transformations - nevertheless which way was used to transform
		 * document, result should be the same.
		 */
		INVALID(true),

		/**
		 * OIOUBL Invoice with many lines. Builds the biggest AXIOM/DOM document
		 * in memory, requires transformer to check cases many-many times.
		 */
		BIG_LINES,

		/**
		 * OIOUBL Invoice with little amount of lines, but big attachment
		 * inside.
		 */
		BIG_PDF,
		/**
		 * Typical OIOUBL Invoice with little amount of lines and without big
		 * attachment inside.
		 */
		LITTLE;

		protected final boolean skip;

		private DocumentSize() {
			this.skip = false;
		}

		/**
		 * Convenience constructor to exclude some case
		 */
		private DocumentSize(boolean skip) {
			this.skip = skip;
		}
	}

	/**
	 * Before test - load XSLT bytes
	 */
	public void setUp() throws Exception {
		URL url = ConfigurationUtils.locate(PATH_XSLT);
		xsltBytes = loadBytesByURL(url);
	}

	protected byte[] loadBytesByURL(URL url) throws IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream(3 * 1024 * 1024);
		InputStream stream = url.openStream();
		IOUtils.copy(stream, out);
		stream.close();
		return out.toByteArray();
	}

	/**
	 * Main GLOBAL test
	 * 
	 * @throws Exception
	 */
	public void testComparePerformanceDifferentSource() throws Exception {
		if (!DO_ACTUAL_TEST) {
			return;
		}

		int caseIndex = 0;
		for (DocumentSize xmlSize : DocumentSize.values()) {
			if (xmlSize.skip) {
				continue;
			}
			/*
			 * Remember result of first case for this document
			 */
			String firstResult = null;
			/*
			 * Load XML bytes for each different DocumentSize value, if not
			 * found - skip it
			 */
			String xmlPath = getXmlPath(xmlSize);
			try {
				xmlBytes = loadBytesByURL(ConfigurationUtils.locate(xmlPath));
				if (xmlBytes == null || xmlBytes.length == 0) {
					throw new RuntimeException("File is empty: " + xmlPath);
				}
			} catch (Exception e) {
				System.err.println("TransformationSourceResearchTest failed to load " + xmlPath + ", skip this case");
				continue;
			}
			for (DocumentSource xsltSource : DocumentSource.values()) {
				if (xsltSource.skip) {
					continue;
				}
				for (DocumentSource xmlSource : DocumentSource.values()) {
					if (xmlSource.skip) {
						continue;
					}
					/*
					 * If xslt source mode is SAX, we do not need to check it
					 * with different values of xslt loading - just byte array
					 * is used. So replace array of values with only one value,
					 * does not matter which one.
					 */
					DocumentLoading[] xsltLoadingValues = DocumentLoading.values();
					if (xsltSource == DocumentSource.SAX) {
						xsltLoadingValues = new DocumentLoading[] { xsltLoadingValues[0] };
					}
					for (DocumentLoading xsltLoading : xsltLoadingValues) {
						if (xsltLoading.skip) {
							continue;
						}
						/*
						 * If xsl source mode is SAX, we do not need to check it
						 * with different values of xslt loading - just byte
						 * array is used. So replace array of values with only
						 * one value, does not matter which one.
						 */
						DocumentLoading[] xmlLoadingValues = DocumentLoading.values();
						if (xmlSource == DocumentSource.SAX) {
							xmlLoadingValues = new DocumentLoading[] { xmlLoadingValues[0] };
						}
						for (DocumentLoading xmlLoading : xmlLoadingValues) {
							if (xmlLoading.skip) {
								continue;
							}
							caseIndex++;
							UseCase useCase = new UseCase(xsltLoading, xsltSource, xmlSize, xmlLoading, xmlSource);
							XsltTickTock t = new XsltTickTock();
							String currentResult = runCaseCycle(caseIndex, t, useCase);
							if (firstResult == null) {
								firstResult = currentResult;
							}
							if (caseIndex == 1) {
								System.out.println("Index\tCase xml size\txslt source\txml source\txsl load\txml load" + t.toHeaderString());
							}
							System.out.println(caseIndex + "\t" + useCase.toString() + t.toStatusString());
							if (FAIL_ON_DIFFERENT_RESULT) {
								assertEquals("Case " + caseIndex + ") " + useCase + " gave different result:\n expected:\n" + firstResult + "\ncurrent:\n" + currentResult, firstResult.length(), currentResult.length());
							}
						}
					}
				}
			}
		}
	}

	protected String runCaseCycle(int caseIndex, XsltTickTock t, UseCase useCase) throws Exception {
		String currentResult = null;
		int repeatCount = CASE_REPEAT;
		int clearAverage = CASE_REPEAT_START_AVERAGE_CALCULATION;
		if (caseIndex == 1) {
			repeatCount = CASE_REPEAT_FIRST_CASE;
			clearAverage = CASE_REPEAT_FIRST_CASE_START_AVERAGE_CALCULATION;
		}
		for (int i = 0; i < repeatCount; i++) {
			if (i == clearAverage) {
				t.clearAverage();
			}
			currentResult = runCase(t, useCase);
			t.finish();
			/*
			 * Call Garbage Collector and ask to clean memory after previous run
			 */
			Runtime.getRuntime().gc();
			t.tockTick("gc");
			t.restartTotal();
		}
		return currentResult;
	}

	/**
	 * Help test to see details of execution of one case
	 * 
	 * @throws Exception
	 */
	public void testOneCase() throws Exception {
		if (!DO_ACTUAL_TEST) {
			return;
		}
		DocumentSize size = DocumentSize.BIG_LINES;
		xmlBytes = loadBytesByURL(ConfigurationUtils.locate(getXmlPath(size)));
		UseCase useCase = new UseCase(DocumentLoading.AXIOM_BUILT_ATTACHMENT, DocumentSource.SAX, size, DocumentLoading.AXIOM_BUILT_ATTACHMENT, DocumentSource.SAX);
		XsltTickTock t = new XsltTickTock();
		runCaseCycle(1, t, useCase);
		System.out.println("Case xml size\txslt source\txml source\txsl load\txml load" + t.toHeaderString());
		System.out.println(useCase.toString() + t.toStatusString());
	}

	private String runCase(XsltTickTock t, UseCase u) throws Exception {
		DocumentLoading xsltLoadingMode = u.xsltLoading;
		DocumentSource xsltSourceMode = u.xsltSource;
		DocumentLoading xmlLoadingMode = u.xmlLoading;
		DocumentSource xmlSourceMode = u.xmlSource;
		t.tick();
		Object xslt = null;
		/*
		 * SAXSource does not require separate document loading and building -
		 * we use raw byte array
		 */
		if (xsltSourceMode != DocumentSource.SAX) {
			xslt = loadXslt(xsltLoadingMode);
			t.tockTick("xsltLoad");

			if (xsltLoadingMode == DocumentLoading.AXIOM_BUILT) {
				((OMElement) xslt).build();
			}
			if (xsltLoadingMode == DocumentLoading.AXIOM_BUILT_ATTACHMENT) {
				((OMElement) xslt).buildWithAttachments();
			}
			t.tockTick("xsltBuild");
		} else {
			t.tockTick("xsltLoad");
			t.tockTick("xsltBuild");
		}
		Source xsltSource = buildSource(true, xslt, xsltLoadingMode, xsltSourceMode);
		t.tockTick("xsltSource");

		Object xml = null;
		/*
		 * SAXSource does not require separate document loading and building -
		 * we use raw byte array
		 */
		if (xmlSourceMode != DocumentSource.SAX) {
			xml = loadXml(xmlLoadingMode);
			t.tockTick("xmlLoad");

			if (xmlLoadingMode == DocumentLoading.AXIOM_BUILT) {
				((OMElement) xml).build();
			}
			if (xmlLoadingMode == DocumentLoading.AXIOM_BUILT_ATTACHMENT) {
				((OMElement) xml).buildWithAttachments();
			}
			t.tockTick("xmlBuild");
		} else {
			t.tockTick("xmlLoad");
			t.tockTick("xmlBuild");
		}
		Source xmlSource = buildSource(false, xml, xmlLoadingMode, xmlSourceMode);
		t.tockTick("xmlSource");

		TransformerFactory transFact = TransformerFactory.newInstance();
		Transformer transformer = transFact.newTransformer(xsltSource);
		t.tockTick("transBuild");

		ByteArrayOutputStream byteArrayOutputStream = null;

		/*
		 * Let's check how time is changed if the same transformer object is
		 * reused several times, results of first attempt is stored as transRun,
		 * for all attempts are calculated everage in transRunMulti
		 */
		for (int i = 0; i < CASE_REPEAT_WTH_THE_SAME_TRANSFORMER; i++) {
			byteArrayOutputStream = new ByteArrayOutputStream();
			StreamResult streamResult = new StreamResult(byteArrayOutputStream);
			transformer.transform(xmlSource, streamResult);
			if (i == 0) {
				t.tock("transRun");
				t.tockTick("transRunMulti");
			} else {
				t.tockTick("transRunMulti");
			}
			/*
			 * In case of a SAXSource of XML, it should be reset (restart from
			 * the beginning of array), because it is straighforward source.
			 * 
			 * If it is not done, parser throws exception.
			 */
			if (xmlSourceMode == DocumentSource.SAX) {
				SAXSource saxSource = (SAXSource) xmlSource;
				saxSource.getInputSource().getByteStream().reset();
			}
		}

		t.finish();

		/*
		 * Return LAST result, so we check that multiple transformation with the
		 * same transformer works properly.
		 * 
		 * Actually, this requires addtional test - what if it works ok only
		 * because the same document is passed in?
		 */
		return new String(byteArrayOutputStream.toByteArray(), "UTF-8");
	}

	private String getXmlPath(DocumentSize xmlSizeMode) {
		switch (xmlSizeMode) {
		case BIG_PDF:
			return PATH_BIG_INVOICE_WITH_BIG_ATTACHMENT;
		case BIG_LINES:
			return PATH_BIG_INVOICE_WITH_MANY_LINES;
		case LITTLE:
			return TestConstants.PATH_INVOICE;
		case INVALID:
			return TestConstants.PATH_INVOICE_NO_ID;
		}
		return null;
	}

	private Object loadXml(DocumentLoading xmlLoadingMode) throws Exception {
		switch (xmlLoadingMode) {
		case AXIOM:
		case AXIOM_BUILT:
		case AXIOM_BUILT_ATTACHMENT:
			return loadAxiom(xmlBytes);
		case DOM:
			return loadDOM(xmlBytes);
		}
		return null;
	}

	private Source buildSource(boolean xslt, Object data, DocumentLoading loadingMode, DocumentSource sourceMode) throws Exception {
		if (loadingMode == DocumentLoading.AXIOM || loadingMode == DocumentLoading.AXIOM_BUILT || loadingMode == DocumentLoading.AXIOM_BUILT_ATTACHMENT) {
			OMElement dataOM = (OMElement) data;
			if (sourceMode == DocumentSource.AXIOM) {
				return new OMSource(dataOM);
			}
			if (sourceMode == DocumentSource.SAX) {
				ByteArrayInputStream baos = new ByteArrayInputStream(xslt ? xsltBytes : xmlBytes);
				SAXSource saxSource = new SAXSource(new InputSource(baos));
				return saxSource;
			}
			return new DOMSource(XMLUtils.toDOM(dataOM).getOwnerDocument());
		} else if (loadingMode == DocumentLoading.DOM) {
			Document dataDOM = (Document) data;
			if (sourceMode == DocumentSource.AXIOM) {
				return new OMSource(XMLUtils.toOM(dataDOM.getDocumentElement()));
			}
			if (sourceMode == DocumentSource.SAX) {
				return new SAXSource(new InputSource(new ByteArrayInputStream(xslt ? xsltBytes : xmlBytes)));
			}
			return new DOMSource(dataDOM);
		}
		return null;
	}

	private Object loadXslt(DocumentLoading xsltLoading) throws Exception {
		switch (xsltLoading) {
		case AXIOM:
		case AXIOM_BUILT:
		case AXIOM_BUILT_ATTACHMENT:
			OMElement axiom = loadAxiom(xsltBytes);
			return axiom;
		case DOM:
		default:
			Document dom = loadDOM(xsltBytes);
			return dom;
		}
	}

	private Document loadDOM(byte[] data) throws Exception {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		return factory.newDocumentBuilder().parse(new ByteArrayInputStream(data));
	}

	private OMElement loadAxiom(byte[] data) throws Exception {
		XMLStreamReader parser = XMLInputFactory.newInstance().createXMLStreamReader(new ByteArrayInputStream(data));
		OMXMLParserWrapper builder = OMXMLBuilderFactory.createStAXOMBuilder(OMAbstractFactory.getOMFactory(), parser);
		return builder.getDocumentElement();
	}

	/**
	 * Description of case - for convenient logging
	 * 
	 * @author dlk
	 */
	private static class UseCase {

		public final DocumentLoading xsltLoading;
		public final DocumentSource xsltSource;
		public final DocumentSize xmlSize;
		public final DocumentLoading xmlLoading;
		public final DocumentSource xmlSource;

		public UseCase(DocumentLoading xsltLoading, DocumentSource xsltSource, DocumentSize xmlSize, DocumentLoading xmlLoading, DocumentSource xmlSource) {
			this.xsltLoading = xsltLoading;
			this.xsltSource = xsltSource;
			this.xmlSize = xmlSize;
			this.xmlLoading = xmlLoading;
			this.xmlSource = xmlSource;
		}

		@Override
		public String toString() {
			StringBuilder builder = new StringBuilder();
			builder.append(xmlSize);
			builder.append("\t");
			builder.append(xsltSource);
			builder.append("\t");
			builder.append(xmlSource);
			builder.append("\t");
			builder.append(xsltLoading);
			builder.append("\t");
			builder.append(xmlLoading);
			return builder.toString();
		}

	}

	private static class XsltTickTock extends TickTock {

		private static final long serialVersionUID = 4134404840340758366L;

		/**
		 * The only specific method in class - calculates trans related time
		 */
		public long getTransformTotal() {
			Long transBuild = averageMap.get("transBuild");
			Long transRun = averageMap.get("transRun");
			if (transBuild != null && transRun != null) {
				return transBuild + transRun;
			}
			return -1;
		}

		@Override
		public String toStatusString() {
			StringBuilder sb = new StringBuilder();
			sb.append(super.toStatusString());
			sb.append("\t");
			sb.append(getTransformTotal());
			return sb.toString();
		}

		@Override
		public String toHeaderString() {
			StringBuilder sb = new StringBuilder();
			sb.append(super.toHeaderString());
			sb.append("\t");
			sb.append("transTotal");
			return sb.toString();
		}

	}

	/**
	 * Test (disabled) to calculate number of lines in each test document.<br>
	 * <br>
	 * Results:<br>
	 * <br>
	 * 
	 * INVALID: 2 <br>
	 * 
	 * BIG_LINES: 794 <br>
	 * 
	 * BIG_PDF: 4 <br>
	 * 
	 * LITTLE: 2<br>
	 */
	public void xtestCalculateLines() {
		DocumentSize[] values = DocumentSize.values();
		for (DocumentSize xmlSize : values) {
			String xmlPath = getXmlPath(xmlSize);
			File xmlFile = new File(xmlPath);
			if (!xmlFile.exists()) {
				continue;
			}
			try {
				XmlDocument xd = new XmlDocument();
				xd.load(xmlPath);

				AXIOMXPath xp = new AXIOMXPath("/main:Invoice/cac:InvoiceLine");
				xp.addNamespace("main", "urn:oasis:names:specification:ubl:schema:xsd:Invoice-2");
				xp.addNamespace("cac", "urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2");
				List<?> nodes = xp.selectNodes(xd.getOMDocument());
				System.out.println(xmlSize + ": " + nodes.size());

			} catch (Exception e) {
				e.printStackTrace();
				continue;
			}

		}
	}
}
