package dk.gov.oiosi.xml.schematron.sax;

import junit.framework.TestCase;
import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationException;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.schematron.SchematronErrorException;

/**
 * Test compatibility of configured document types in RaspConfiguration.xml,
 * test correctness of parsing XPath values and their validation.
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * 
 */
public class SchematronValidatorSAXHandlerConfigTest extends TestCase {

	/**
	 * Checks compatibility of configured document types in
	 * RaspConfiguration.xml with a new way of schematron validation - walk
	 * through existing configuration and check that config object is built
	 * successfully
	 */
	public void testCheckListOfCurrentSchematronXPathConfiguration() throws Exception {
		configPath(TestConstants.PATH_RASPCONFIGURATION_TEST);
	}

	protected void configPath(String pathRaspconfigurationTest) throws ConfigurationException {
		ConfigurationHandler.setConfigurationFile(pathRaspconfigurationTest);
		DocumentTypeCollectionConfig config = ConfigurationHandler.getInstance().getRaspDocumentTypeCollectionConfig();
		DocumentTypeConfig[] configs = config.getDocumentTypes();

		for (int i = 0; i < configs.length; i++) {
			DocumentTypeConfig typeConfig = configs[i];
			String errorMessageXPath = typeConfig.getSchematronValidationConfig().getErrorMessageXPath();
			String errorXPath = typeConfig.getSchematronValidationConfig().getErrorXPath();
			try {
				SchematronValidatorSAXHandlerConfig.build(errorXPath, errorMessageXPath);
			} catch (Exception e) {
				e.printStackTrace();
				fail("Failed building result for " + typeConfig.getFriendlyName() + ": " + errorXPath + ", " + errorMessageXPath);
			}
		}
	}

	public void testBuild() throws SchematronErrorException {
		SchematronValidatorSAXHandlerConfig c = SchematronValidatorSAXHandlerConfig.build("/schematron/error", "/schematron/error");
		assertEquals("error", c.getErrorMessageTag());
		assertEquals(1, c.getTagList().length);
		assertEquals("schematron", c.getTagList()[0]);

		c = SchematronValidatorSAXHandlerConfig.build("/Schematron/Error", "/Schematron/Error/Description");
		assertEquals("Description", c.getErrorMessageTag());
		assertEquals(2, c.getTagList().length);
		assertEquals("Schematron", c.getTagList()[0]);
		assertEquals("Error", c.getTagList()[1]);

		try {
			SchematronValidatorSAXHandlerConfig.build(null, null);
			fail("Build method is expected to throw an exception if one of fields is not set");
		} catch (SchematronErrorException see) {
		}

		try {
			SchematronValidatorSAXHandlerConfig.build("", null);
			fail("Build method is expected to throw an exception if one of fields is not set");
		} catch (SchematronErrorException see) {
		}

		try {
			SchematronValidatorSAXHandlerConfig.build(null, "");
			fail("Build method is expected to throw an exception if one of fields is not set");
		} catch (SchematronErrorException see) {
		}

		try {
			SchematronValidatorSAXHandlerConfig.build("/A", "/B");
			fail("XPath for error should be equal or start with XPath for error description");
		} catch (SchematronErrorException see) {
		}

		try {
			SchematronValidatorSAXHandlerConfig.build("/Schematron[1]/Error[1]", "/Schematron[1]/Error[1]/Description[1]");
			fail("Only simple list of tags is supported, real XPath expressions - not.");
		} catch (SchematronErrorException see) {
			assertEquals("XPath for SchematronValidatorSAXHandler should contain only list of tags delimited with slash, no XPath functions or selectors are supported. But in XPath '/Schematron[1]/Error[1]' part 'Schematron[1]' was found", see.getMessage());
		}
	}

	public void testIsOneWordString() {
		assertFalse(s(""));
		assertFalse(s("  "));
		assertFalse(s("A[1]"));
		assertFalse(s("first-child()"));

		assertTrue(s("A1234"));
		assertTrue(s("ABCDabcd"));
		assertTrue(s("ABCDabcd "));
		assertTrue(s("ABCDabcd1"));
		assertTrue(s("1AB2CDa3bc5d"));
	}

	private static boolean s(String s) {
		return SchematronValidatorSAXHandlerConfig.isOneWordString(s);
	}
}
